/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2013.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function(){   
	
	dojo.provide("com.ibm.team.vvc.server.core.ui.configurations.steps.ConfigurationBase");

	dojo.require("dijit._Widget");
	dojo.require("dijit._Templated");
	dojo.require("dojo.date.stamp");
	dojo.require("jazz.ui.wizard.WizardStep");
	dojo.require("dijit.form.Select");
	dojo.require("jazz.app.ua.PageHoverHelp");
		
	dojo.require('dojo.i18n');
	dojo.requireLocalization("com.ibm.team.vvc.server.core.ui.configurations.steps", "ConfigurationBaseMessages");
	
	dojo.require("com.ibm.team.vvc.shared.web.ui.VvcConstants");
	dojo.require("com.ibm.team.vvc.shared.web.ui.picker.WorkspacePicker");
	
	dojo.declare("com.ibm.team.vvc.server.core.ui.configurations.steps.ConfigurationBase", [jazz.ui.wizard.WizardStep, dijit._Widget, dijit._Templated], {
	
		templatePath: dojo.moduleUrl('com.ibm.team.vvc.server.core', 'ui/configurations/steps/templates/ConfigurationBase.html'),
		messages: dojo.i18n.getLocalization("com.ibm.team.vvc.server.core.ui.configurations.steps", "ConfigurationBaseMessages"),
		widgetsInTemplate: true,
		_storageKey: null,
		_data: null,
		isLoaded: null,
		parentConfigurationMode: null,
		
		parentConfigurationPicker: null,
		isPickerVisible: null,
		isPickerDimensionsVisible: null,
		
		_disconnects: null,
		_destroys: null,
		
		_sizeOpen: {	
			width: 600,
			height: 590
		},
			
		_sizePickerDimensionsOpen: {
			width: 899,
			height: 575
		},
			
		_sizeClosed: {
			width: 600,
			height: 195
		},
			
		_disabed: null,
			
		parentConfiguration: null,

		constructor: function(args){
			if(args && args.storageKey){
				this._storageKey = args.storageKey;
			}
			this._disconnects = [];
			this._destroys = [];
			
			// IE8 is doing something weird with sizes...
			if(dojo.isIE <= 8){
				this._sizeClosed.height = 175;
			}
		},
		
		destroy: function(args){
			this.inherited(args);
			dojo.forEach(this._disconnects, function(handle){dojo.disconnect(handle);});
			dojo.forEach(this._destroys, function(obj){dojo.destroyRecursive(obj);});
		},
		
		postCreate: function(){
			this.stepTitle = this.messages.title;
			this.isPickerVisible = false;
		},
		
		focusStep: function(){
			this._data = dojo.fromJson(sessionStorage.getItem(this._storageKey));
			
			var nc = this._data.newConfiguration;
			
			if(!this.isLoaded){
				var message = "";
				var uaKeyString = "";
				if(nc.type === "workspace"){
					var pc = this._data.input.configuration? this._data.input.configuration : this._data.input.space;
					message = nc.type === pc.type ? this.messages.automaticNewWorkspace : this.messages.automaticNewWorkspaceOffSnapshot;
					uaKeyString = "com.ibm.team.vvc.server.core.wizard_ConfigurationBase001";
				} else if(nc.type === "snapshot"){
					dojo.empty(this.parentLabel);
					this.parentLabel.appendChild(dojo.doc.createTextNode(this.messages.snapshotParentLabel));
					this.stepTitle = this.messages.createSnapshotTitle;
					message = this.messages.automaticNewSnapshot;
					uaKeyString = "com.ibm.team.vvc.server.core.wizard_ConfigurationBase002";
				}
				
				var uaHelp = new jazz.app.ua.PageHoverHelp({uaKey: uaKeyString});
				this.cshSelectParentNode.appendChild(uaHelp.domNode);
				this._destroys.push(uaHelp);
				var selectorExistingMessage = nc.type === "snapshot" ? this.messages.selectExistingWorkspace : this.messages.selectExistingSnapshot;
				this.parentConfigurationMode = new dijit.form.Select({
					name: "parentConfigurationMode",
					style: "width: 340px;", // Ugly but necessary, see http://dojotoolkit.org/reference-guide/1.7/dijit/form/Select.html#setting-width
					options: [
					          { label: message, value: message, selected: true},
					          { label: selectorExistingMessage, value: selectorExistingMessage}
					          ] 
				});
				this._destroys.push(this.parentConfigurationMode);
				this.selectContentCol.appendChild(this.parentConfigurationMode.domNode);
				this._disconnects.push(dojo.connect(this.parentConfigurationMode, "onChange", dojo.hitch(this, this.togglePicker)));
				this.isLoaded = true;
			}
			this.updateSize();
		},
					
		customOnNext: function(){
			var config = this._data.newConfiguration;					// New created configuration
			var inParentConfiguration = this._data.input.configuration;	// Input Parent Configuration
			var inSpaceConfiguration = this._data.input.space;			// Input Space Configuraton
			
			if(this.isParentCreateModeAutomatic()){
				
				if(inParentConfiguration){
					// If we have a parent configuration input in auto mode, we're creating against that parent.

					// Detect workspace on workspace
					if(inParentConfiguration.type === config.type && config.type === "workspace") {
						var ims = {
								title: inParentConfiguration.title,
								description: dojo.string.substitute(this.messages.imsDescription, {"pcTitle": inParentConfiguration.title, "ncTitle": config.title}),
								parent: inParentConfiguration,
								type: "snapshot",
								typeLabel: this.messages.snapshot,
								space: inSpaceConfiguration,
								uri: null	// Indicates this configuration needs to be created
						};					
												
						this.parentConfiguration = ims;
					} else {
						this.parentConfiguration = inParentConfiguration;
					}
					
				} else /*if(inSpaceConfiguration)*/{
					// If we only have a configuration space in auto mode, we're creating directly against the space.  This case is handled below
				}
				
//			} /*else if(this.isParentCreateModeManual()){
//				// Manual mode
				// This case is handled below by the presence of this.parentConfiguration
//
			} else if(this.isParentCreateModeNone()){
				// Create directly against the configuration space.  Ensure this.parentConfiguration is not set
				this.parentConfiguration = null;
			}
			
			// Set the parent, if we have one.  Otherwise wizard will default to creating directly against the space afters
			if(this.parentConfiguration){
				this._data.newConfiguration.parent = {
						title: this.parentConfiguration.title,
						description: this.parentConfiguration.description,
						type: this.parentConfiguration.type,
						uri: this.parentConfiguration.uri,
						space: this.parentConfiguration.space,
						parent: this.parentConfiguration.parent
				};
			}
			
			this._data.newConfiguration.parent.typeLabel = this.messages[this._data.newConfiguration.parent.type];

			// Set the space, required
			this._data.newConfiguration.space = inSpaceConfiguration;
							
			sessionStorage.setItem(this._storageKey, dojo.toJson(this._data));
		},
		
		canFlipToNextPage: function(){
			if(this.isParentCreateModeAutomatic() || this.isParentCreateModeNone()){
				// In "automatic" or "none" mode, no input is required.
				return true;
			}
			
			return this.parentConfiguration; //this.parentConfiguration !== null && this.parentConfiguration !== undefined;
		},
				
		isParentCreateModeAutomatic: function(){
			return !this.isParentCreateModeManual();
		},
		
		isParentCreateModeManual: function(){
			return 	this.parentConfigurationMode.value === this.messages.selectExistingWorkspace || 
					this.parentConfigurationMode.value === this.messages.selectExistingSnapshot;
		},
		
		isParentCreateModeNone: function(){
			return this.parentConfigurationMode.value === this.messages.none;
		},
		
		togglePicker: function(){
			if(this.isParentCreateModeAutomatic() || this.isParentCreateModeNone()){
				this.hideBasePicker();
				this.isPickerVisible = false;
			} else {
				this.showBasePicker();
				this.isPickerVisible = true;
			}
			this.triggerButtonStateUpdate();
			this.updateSize();
		},
		
		showBasePicker: function(){
			// Lazily instanciate the picker
			if(!this.parentConfigurationPicker){
				var validParentTypes = this._data.input.validParentTypes;
				this.parentConfigurationPicker = new com.ibm.team.vvc.shared.web.ui.picker.WorkspacePicker(
					{
						spaceUri : this._data.input.space.uri,
						validTypes : this._data.newConfiguration.type === "workspace" ? "snapshot" : "workspace",
						closeable : false
					});
				this.parentConfigurationPicker.hideOkCancelButtons();

				this._disconnects.push(dojo.connect(this.parentConfigurationPicker, "onOk", dojo.hitch(this, this.changeParentConfiguration)));
				this._disconnects.push(dojo.connect(this.parentConfigurationPicker, "onSelect", dojo.hitch(this, this.changeParentConfiguration)));
				this._disconnects.push(dojo.connect(this.parentConfigurationPicker, "onShowDimensions", dojo.hitch(this, function(){
					this.isPickerDimensionsVisible = true;
					this.updateSize();
				})));
				
				this._disconnects.push(dojo.connect(this.parentConfigurationPicker, "onHideDimensions", dojo.hitch(this, function(){
					this.isPickerDimensionsVisible = false;
					this.updateSize();
				})));
				
				this.parentConfigurationPickerNode.appendChild(this.parentConfigurationPicker.domNode);
			}
			dojo.removeClass(this.parentConfigurationPickerNode, "hidden");
			dojo.addClass(this.parentConfigurationPickerNode, "visible");
		},
		
		hideBasePicker: function(){
			if(this.parentConfigurationPicker){
				dojo.addClass(this.parentConfigurationPickerNode, "hidden");
				dojo.removeClass(this.parentConfigurationPickerNode, "visible");
			}
		},
		
		updateSize: function(){
			if(this.isParentCreateModeAutomatic() || this.isParentCreateModeNone()){
				this.setSize(this._sizeClosed);
			} else {
				if(this.isPickerDimensionsVisible){
					this.setSize(this._sizePickerDimensionsOpen);
				} else {
					this.setSize(this._sizeOpen);
				}
			}
		},
		
		changeParentConfiguration: function(newConfiguration){
			this.parentConfiguration = newConfiguration;
			this.triggerButtonStateUpdate();
		},
				
	});

})();