/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2013.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function(){   
	
	dojo.provide("com.ibm.team.vvc.server.core.ui.configurations.steps.ConfigurationDetails");

	dojo.require("dijit._Widget");
	dojo.require("dijit._Templated");
	dojo.require("jazz.ui.wizard.WizardStep");
	dojo.require("jazz.app.ua.PageHoverHelp");
	
	dojo.require('dojo.i18n');
	dojo.requireLocalization("com.ibm.team.vvc.server.core.ui.configurations.steps", "ConfigurationDetailsMessages");
	
	dojo.require("com.ibm.team.vvc.shared.web.ui.VvcConstants");
	dojo.require("com.ibm.team.vvc.server.core.ui.internal.util.PageUtil");
	dojo.require("com.ibm.team.vvc.shared.web.ui.notifications.notifications.VvcNotificationFactory");
	var notificationFactory = com.ibm.team.vvc.shared.web.ui.notifications.notifications.VvcNotificationFactory;

	dojo.declare("com.ibm.team.vvc.server.core.ui.configurations.steps.ConfigurationDetails", [jazz.ui.wizard.WizardStep, dijit._Widget, dijit._Templated], {
	
		templatePath: dojo.moduleUrl('com.ibm.team.vvc.server.core', 'ui/configurations/steps/templates/ConfigurationDetails.html'),
		messages: dojo.i18n.getLocalization("com.ibm.team.vvc.server.core.ui.configurations.steps", "ConfigurationDetailsMessages"),
		widgetsInTemplate: false,
		_storageKey: null,
		_data: null,
		_size: {
			width: 600, 
			height: 235
		},
		
		_validEditors: null,
		_nameChangeTimer: null,
		_descriptionChangeTimer: null,
		_validationInProgress: null,
		
		_disconnects: null,
		_destroys: null,
		
		_err: null,
		
		_getConfigurationNameDeferred: null,
		_inputWithFocus: null,
		
		_illegalCharacters: ['\\'],
			
		constructor: function(args){
			if(args && args.storageKey){
				this._storageKey = args.storageKey;
			}
			
			this._validEditors = false;
			
			this._disconnects = [];
			this._destroys = [];
			
			// IE8 is doing something weird with sizes...
			if(dojo.isIE <= 8){
				this._size.height = 200;
			}
		},
		
		destroy: function(args){
			this.inherited(args);
			dojo.forEach(this._disconnects, function(handle){dojo.disconnect(handle);});
			dojo.forEach(this._destroys, function(obj){dojo.destroyRecursive(obj);});
		},
		
		postCreate: function(){
			this.stepTitle = this.messages.title;

			var events = ["keyup", "onpaste", "oncut"];
			for (var i = 0; i < events.length; i++) {
				this._disconnects.push(dojo.connect(this.configurationNameInput, events[i], dojo.hitch(this, this.onNameInputChange)));
				this._disconnects.push(dojo.connect(this.configurationDescriptionInput, events[i], dojo.hitch(this, this.onDescriptionInputChange)));
			}
		
			var uaHelp = new jazz.app.ua.PageHoverHelp({uaKey: "com.ibm.team.vvc.server.core.wizard_ConfigurationDetails001"});
			this._destroys.push(uaHelp);
			this.cshNameNode.appendChild(uaHelp.domNode);
			
			// force the focus on the name field...
			var context = this;
			setTimeout(function() {
				context.configurationNameInput.focus();
			}, 300);

			this._inputWithFocus = this.configurationNameInput;
		},
		
		_setConfigurationNameInputFocus: function(){
			this._inputWithFocus = this.configurationNameInput;
		},
		
		_setConfigurationDescriptionInputFocus: function(){
			this._inputWithFocus = this.configurationDescriptionInput;
		},
		
		focusStep: function(){
			this._data = dojo.fromJson(sessionStorage.getItem(this._storageKey));	
			
			var theTitle = this.getConfigurationTitle();
			if (theTitle && theTitle.length > 0) {
				this._validateConfigurationName();
				//configuration description isn't actually validated
			}
			
			this.setSize(this._size);
			
			if(this._data.input.type === "workspace"){
				this.stepTitle = this.messages.workspaceTitle;
			} else if(this._data.input.type === "snapshot"){
				this.stepTitle = this.messages.snapshotTitle;
			}
		},
				
		customOnNext: function(){
			
			// Protect against a fast-moving user and a slow server.
			this._validateConfigurationDescription();
			this._validateConfigurationName();
			if(!this._validEditors){
				return;
			}
			
			this._data.newConfiguration.title = this.getConfigurationTitle();
			this._data.newConfiguration.description = this.getConfigurationDescription();
			this._data.newConfiguration.type = this._data.input.type;
			this._data.newConfiguration.typeLabel = this.messages[this._data.input.type];
			sessionStorage.setItem(this._storageKey, dojo.toJson(this._data));
		},
				
		getConfigurationTitle: function(){
			return dojo.trim(this.configurationNameInput.value);
		},
		
		getConfigurationDescription: function(){
			return dojo.trim(this.configurationDescriptionInput.value);
		},
		
		canFlipToNextPage: function(){
			return this._validEditors;
		},
		
		onNameInputChange: function(){
			this._validEditors = false;
			if(this._nameChangeTimer){
				clearTimeout(this._nameChangeTimer);
			}
			
			//Auto timeout of 200ms between keystrokes, then validate the name
			this._nameChangeTimer = setTimeout(dojo.hitch(this, this._validateConfigurationName), 350);
		},
		
		onDescriptionInputChange: function(){
			this._validEditors = false;
			if(this._descriptionChangeTimer){
				clearTimeout(this._nameChangeTimer);
			}
			//Auto timeout of 200ms between keystrokes, then validate the description
			this._descriptionChangeTimer = setTimeout(dojo.hitch(this, this._validateConfigurationDescription), 200);
		},
		
		_displayBadNameError: function(reason) {
			var notification = notificationFactory.createError({
				title: this.messages.badName,
				description: reason,
				solution: this.messages.solution
			});
			this._err = notification;
			notification.dispatch();
		},
						
		_validateConfigurationName: function(){
			var text = this.getConfigurationTitle();
			var isValid = false;
			var reason = this.messages.blankConfigurationName;
			
			if(this._err){
				this._err.close();
			}
			
			if(this._validateText(text)){
				var PageUtil = com.ibm.team.vvc.server.core.ui.internal.util.PageUtil;
				var spaceUri = this._data.input.space.uri;
				var mutable = this._data.input.type === "workspace" ? true : false;

				// Make sure we don't hit the server unnecessarily.
				if(this._getConfigurationNameDeferred){
					this._getConfigurationNameDeferred.cancel();
				}
				
				this._getConfigurationNameDeferred = PageUtil.findConfigurationName(text, spaceUri, mutable, false);
				dojo.when(this._getConfigurationNameDeferred, dojo.hitch(this, function(conflictingUri){
					if(!conflictingUri){
						isValid = true;
					} else {
						reason = this.messages.invalidConfigurationName;
					}
							
					if(isValid){
						dojo.addClass(this.configurationNameFeedbackRow, "hidden");
						this.configurationNameFeedbackNode.innerHTML = "";
					} else {
						this._displayBadNameError(reason);
					}
					
					this._validEditors = isValid;
					this.triggerButtonStateUpdate();
					this._inputWithFocus.focus();
				}));
			} else if (text.length === 0) {
				this._displayBadNameError(reason);
				this._validEditors = false;
			}
			
			this.triggerButtonStateUpdate();
			
			/*
			 *  triggerButtonStateupdate() occasionally steals focus from the inputs,
			 *   we need to restore focus to the proper elements.
			 */
			this._inputWithFocus.focus();
		},
		
		_validateConfigurationDescription: function(){
			// Stub
			this._validEditors = true;
		},
		
		_validateText: function(text){
			if(text.length > 0){
				var index;
				for(index = 0; index < this._illegalCharacters.length; index++){
					var badChar = this._illegalCharacters[index];
					if(text.indexOf(badChar) >= 0){
						var reason = dojo.string.substitute(this.messages.illegalCharacters, {"char" : badChar});
						this._displayBadNameError(reason);
						return false;
					}
				}
				return true;
			}
			return false;
		}
		
	});

})();