/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2013.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function(){   
	
	dojo.provide("com.ibm.team.vvc.server.core.ui.configurations.steps.ConfigurationSummary");

	dojo.require("dijit._Widget");
	dojo.require("dijit._Templated");
	dojo.require("jazz.ui.wizard.WizardStep");
	dojo.require("com.ibm.team.vvc.server.core.ui.configurations.steps.ConfigurationProgressPage");

	
	dojo.require('dojo.i18n');
	dojo.requireLocalization("com.ibm.team.vvc.server.core.ui.configurations.steps", "ConfigurationSummaryMessages");
	
	dojo.require("com.ibm.team.vvc.shared.web.ui.VvcConstants");
	dojo.require("com.ibm.team.vvc.shared.web.ui.context.ContextUtils");
	var ContextUtils = com.ibm.team.vvc.shared.web.ui.context.ContextUtils;
	
	dojo.require("com.ibm.team.vvc.shared.web.ui.notifications.notifications.VvcNotificationFactory");
	var notificationFactory = com.ibm.team.vvc.shared.web.ui.notifications.notifications.VvcNotificationFactory;
	
	dojo.declare("com.ibm.team.vvc.server.core.ui.configurations.steps.ConfigurationSummary", [jazz.ui.wizard.WizardStep, dijit._Widget, dijit._Templated], {
	
		templatePath: dojo.moduleUrl('com.ibm.team.vvc.server.core', 'ui/configurations/steps/templates/ConfigurationSummary.html'),
		messages: dojo.i18n.getLocalization("com.ibm.team.vvc.server.core.ui.configurations.steps", "ConfigurationSummaryMessages"),
		widgetsInTemplate: false,
		_storageKey: null,
		_size: {
			width: 600,
			height: 400
		},
		_data: null,
		_supported: null,
		
		_disconnects: null,
		_destroys: null,
		
		_error: null,
		
		constructor: function(args){
			if(args && args.storageKey){
				this._storageKey = args.storageKey;
			}
			this._supported = true;
			this._disconnects = [];
			this._destroys = [];
			
			// IE8 is doing something weird with sizes...
			if(dojo.isIE <= 8){
				this._size.height = 380;
			}
		},
		
		destroy: function(args){
			this.inherited(args);
			dojo.forEach(this._disconnects, function(handle){dojo.disconnect(handle);});
			dojo.forEach(this._destroys, function(obj){dojo.destroyRecursive(obj);});
		},
		
		postCreate: function(){
			this.stepTitle = this.messages.title;
		},
		
		focusStep: function(){
			
			if(this._error){
				this._error.close();
			}
			
			this._data = dojo.fromJson(sessionStorage.getItem(this._storageKey));
			
			if(this._data.newConfiguration.type === "workspace"){
				this.stepTitle = this.messages.workspaceTitle;
			} else if(this._data.newConfiguration.type === "snapshot"){
				this.stepTitle = this.messages.snapshotTitle;
			}
			
			this._updateNameText();
			this._updateDescriptionText();
			this._updateParentConfigurationText();
			this._updateBaseSnapshotText();
			this._updatePendingActionsText();
			this.updateSize();
			this.resetLastStep();
		},
		
		resetLastStep: function(){
			if (this.wizard.steps[this.wizard.steps.length - 1]._isStarted) {
				var newStep = new com.ibm.team.vvc.server.core.ui.configurations.steps.ConfigurationProgressPage({storageKey: this._storageKey});
				this.wizard.removeStep(this.wizard.steps[this.wizard.steps.length - 1]);
				this.wizard.addStep(newStep);
				this.stepAdded(newStep);
			}
		},
				
		_updateNameText: function(){
			this.configurationName.innerHTML = jazz.util.html.escape(this._data.newConfiguration.title);
		},
				
		_updateDescriptionText: function(){
			var description = this._data.newConfiguration.description;
			if(this.description !== ""){
				this.configurationDescriptionTextArea.innerHTML = jazz.util.html.escape(description);
			} else {
				this.configurationDescriptionTextArea.innerHTML = this.messages.notProvided;
			}
		},
		
		_updateParentConfigurationText: function(){
			var nc = this._data.newConfiguration;
			var pc = nc.parent;
			var title = new dojo.Deferred();
			
			if(nc.type === "workspace"){
				if(pc && pc.type === "snapshot"){
					if(pc.parent && pc.parent.title){
						title.resolve(pc.parent.title);
					} else if(pc.uri){
						var conf = ContextUtils.getConfiguration(pc.uri);
						dojo.when(conf, dojo.hitch(this, function(config){
							title.resolve(conf.getParentName());
						}));
					}else if(this._data.input.configuration){
						title.resolve(this._data.input.configuration.title);
					} else {
						title.resolve(this._data.input.space.title);
					}
				} else {
					// create: none mode
					title.resolve(pc.title);
					
				}
			} else if (nc.type === "snapshot"){
				title.resolve(pc.title);
			}
			dojo.when(title, dojo.hitch(this, function(title){
				dojo.empty(this.configurationParent);
				this.configurationParent.appendChild(dojo.doc.createTextNode(jazz.util.html.escape(title)));
			}));
		},
		
		_updateBaseSnapshotText: function(){
			var nc = this._data.newConfiguration;
			
			// Not shown for Snapshots
			if(nc.type === "snapshot"){
				return;
			}				
			
			dojo.empty(this.baseSnapshot);
			
			if(nc.parent.type === "snapshot"){
				this.baseSnapshot.appendChild(dojo.doc.createTextNode(jazz.util.html.escape(nc.parent.title)));
			} else {
				this.baseSnapshot.appendChild(dojo.doc.createTextNode(this.messages.none));
			}
			
			dojo.removeClass(this.baseSnapshotRow, "hidden");
		},
		
		_updatePendingActionsText: function(){
			
			var nc = this._data.newConfiguration;
			var ncTitle = nc.title;
			var pcTitle = nc.parent.title;
			var imcTitle = nc.parent.title;
			var message = "";
			
			if(nc.parent){				
				if(nc.parent.uri){
					// Not a special operation.
					var nt = nc.type;
					var pt = nc.parent.type;
					if(nt === "workspace" && pt === "workspace"){
						message = this.messages.manualWorkspaceOffWorkspace;
					}else if(nt === "workspace" && pt === "snapshot"){
						message = this.messages.manualWorkspaceOffSnapshot;
					}else if(nt === "snapshot" && pt === "workspace"){
						message = this.messages.manualSnapshotOffWorkspace;
					} else {
						this._supported = false;
						this._error = notificationFactory.createError({
							title: this.messages.unsupportedTitle,
							description: dojo.string.substitute(this.messages.unsupportedDescription,
									{ncType: nc.typeLabel, pcType: nc.parent.typeLabel}),
							solution: this.messages.unsupportedSolution
						}).dispatch();
					}
					
				} else {
					// No Parent Configuration URI indicates the parent configuration is to be created as well, probably workspace-on-workspace in Auto mode
					imcTitle = pcTitle;
					pcTitle = nc.parent.parent.title;
					message = this.messages.autoWorkspaceOffWorkspace;
				}
			} else {
				// No parent indicates we're creating this directly on the configuration space
				pcTitle = nc.space.title;
				message = this.messages.manualWorkspaceOffWorkspace;
			}
			
			this.pendingActionsTextArea.innerHTML = dojo.string.substitute(message, {
				"ncTitle": jazz.util.html.escape(ncTitle),
				"pcTitle": jazz.util.html.escape(pcTitle),
				"imcTitle": jazz.util.html.escape(imcTitle)
			});
			
		},
				
		canFlipToNextPage: function(){
			return true && this._supported;
		},
		
		canFinish: function(){
			return false;
		},
		
		updateSize: function(){
			this.setSize(this._size);
		},
			
	});

})();