/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide('com.ibm.team.vvc.shared.web.ui.context.ConfigurationContextBannerButton');

dojo.require("jazz.ui.ScrollableMenu");
dojo.require('com.ibm.team.vvc.shared.web.ui.widgets.HideableMenuItem');
dojo.require('com.ibm.team.vvc.shared.web.ui.picker.WorkspacePickerDialog');
dojo.require("com.ibm.team.vvc.shared.web.ui.context.ContextUtils");

dojo.require("com.ibm.team.vvc.shared.web.ui.changesets.CreateChangesetForm");
dojo.require("com.ibm.team.vvc.shared.web.ui.configurations.CreateConfigurationForm");

dojo.require("com.ibm.team.vvc.shared.core.ui.objects.ConfigurationResource");
dojo.require("com.ibm.team.vvc.shared.web.ui.util.VvcUtils");

dojo.require("com.ibm.team.vvc.shared.web.ui.changesets.ChangesetsClient");
dojo.require("com.ibm.team.vvc.shared.web.ui.util.UIUtils");
dojo.require("com.ibm.team.vvc.shared.web.ui.util.LinkToWorkItemUtil");
dojo.require("com.ibm.team.vvc.shared.oslc.web.utils.CLMUtils");
var CLMUtils = com.ibm.team.vvc.shared.oslc.web.utils.CLMUtils;

dojo.require("com.ibm.team.vvc.shared.web.ui.util.SecurityServiceUtils");
var SecurityUtils = com.ibm.team.vvc.shared.web.ui.util.SecurityServiceUtils;

dojo.require("com.ibm.team.vvc.shared.web.ui.util.SecurityConstants");
var SecurityConstants = com.ibm.team.vvc.shared.web.ui.util.SecurityConstants;

dojo.require("com.ibm.team.vvc.shared.web.ui.util.VvcUtils");
var VvcUtils = com.ibm.team.vvc.shared.web.ui.util.VvcUtils;

dojo.require("com.ibm.team.vvc.shared.web.ui.util.InfoContentUtils");
var InfoContentUtils = com.ibm.team.vvc.shared.web.ui.util.InfoContentUtils;

dojo.require("jazz.util.html");
dojo.require("com.ibm.team.vvc.shared.web.ui.VvcConstants");


dojo.requireLocalization("com.ibm.team.vvc.shared.web.ui.context", "ConfigurationContextMessages");

(function ()
{
	var vvcConstants = com.ibm.team.vvc.shared.web.ui.VvcConstants;
	var UIUtils = com.ibm.team.vvc.shared.web.ui.util.UIUtils;
	var ContextUtils = 	com.ibm.team.vvc.shared.web.ui.context.ContextUtils;
	var ChangesetsClient = com.ibm.team.vvc.shared.web.ui.changesets.ChangesetsClient;
	
	var DCTERMS_TITLE = "http://purl.org/dc/terms/title";

	/**
	 * ConfigurationContextBannerButton is a configuration context combo box to be placed in the Jazz UI bannner.
	 * It allows the user to choose a configuration context, and remembers the last 5 used. It also gives entry points
	 * into the VVC application by allowing the user to go the main VVC configuration explorer page or create workspaces
	 * or snapshots.
	 * 
	 * Pass "workspaceBasedOffSnapshotOnly" optional property to the first arguments object in the constructor to only 
	 * allow workspace creation off of a snapshot.
	 */
	dojo.declare('com.ibm.team.vvc.shared.web.ui.context.ConfigurationContextBannerButton', [dijit._Widget, dijit._Templated], 
		{
			messages: dojo.i18n.getLocalization("com.ibm.team.vvc.shared.web.ui.context", "ConfigurationContextMessages"),
			templatePath: dojo.moduleUrl('com.ibm.team.vvc.shared.web', 'ui/context/templates/ConfigurationContextBannerButton.html'),
			
			/**
			 * The index of the selected item, or the item itself.
			 * @field
			 * @type Number|Widget
			 */
			selected: null,
			redirectUriSelector: null,
			onContextChange: null,
			authenticated: false,
			authenticationDeferred: null,
			
			constructor: function (args)
			{
				if (args)
				{
					if (args.workspaceBasedOffSnapshotOnly)
						this._workspaceBasedOffSnapshotOnly = args.workspaceBasedOffSnapshotOnly;
					
					if (args.redirectUriSelector)
						this.redirectUriSelector = args.redirectUriSelector;
				}
			},

			postCreate: function() {
				this._initializing = true;
				this._initializeChangesetSubscribers();
				this._initializeMenuButton();
				this._initializeRecentlyUsed();
				this._initializeMoreMenuItem();
				this._initializeQuickCreateMenuItem();
				this._initializeSelectedMenuItem();
				this._initializeManageConfigurations();
				this._initializeManageChangesets();
				
				this._toggleMenuItems();
				
				this._initializing = false;
				//Listen to update the widget when the context is completely initialized.
				this.subscribe("vvc.context_initialized",this._initializeSelectedMenuItem);
			},
			
			_initializeChangesetSubscribers: function()
			{
				dojo.subscribe("vvc.changeset_created", dojo.hitch(this, function() 
				{
					var changeset = arguments[0];
					if (this.redirectUriSelector != null && changeset != null && changeset.configurationUri != null && this.onContextChange != null)
					{
						var csConfiguration = new com.ibm.team.vvc.shared.core.ui.objects.ConfigurationResource({uri : changeset.configurationUri}); //not required to load yet
						this.onContextChange(csConfiguration);
					}
				} ));
			},
			
			_checkValid: function(){
				var configuration = ContextUtils.getCurrentContext(true);
				if (!configuration) {
					InfoContentUtils.showSimpleContentDialog(this.messages.archivedTitle, this.messages.archivedUnknownMessage);
					return false;
				}
				if (configuration.isArchived()) {
					InfoContentUtils.showSimpleContentDialog(this.messages.archivedTitle, dojo.string.substitute(this.messages.archivedMessage, [configuration.getTitle()]));
					return false;
				}
				
				return true;
			},
			
			addChild: function(widget, insertIndex){
				//add it to the menu
				this.connect(widget, "onClick", dojo.hitch(this, this._wasClicked, widget));
				this._menu.addChild.apply(this._menu, arguments);
				
				if (widget.configuration == null)
					return;
				
				ContextUtils.addConfigurationToRecentlyUsed(widget.configuration);
				
				var context = {
					self: this,
					recentConfig: widget.configuration,
					configurationPopup: null
				};

				var configurationPopup = new jazz.ui.HoverPopup({
					around: widget.domNode,
					html: this.messages.loadingConfigurationPopup,
					openDelay: 300,
					maxWidth: 800,
					onOpen: dojo.hitch(context, function() {
						if (!this.self.authenticated) {
							if (!this.self.authenticationDeferred) {
								this.self.authenticationDeferred = new dojo.Deferred();
							}
							CLMUtils.authenticate(this.self.authenticationDeferred);
							this.self.authenticationDeferred.addCallback(this, dojo.hitch(this, function(result){
								this.self.authenticated = true;
								if (result) {
									for (var i = 0; i < result.length; i++) {
										if (result[i].length >= 2) {
											if (result[i][1]) {
												context.self.providers = result[i][1];
												break;
											}
										}
									}
								}
								this.self._makePopupContents(this.configurationPopup, this.recentConfig);
							}));
						} else {
							this.self._makePopupContents(this.configurationPopup, this.recentConfig);
						}
					})
				});
				
				context.configurationPopup = configurationPopup;
				
				//remove any menuItems that are no longer in the recently used list
				var menuItems = this._menu.getChildren();
				for(var i in menuItems) {
					var menuItem = menuItems[i];
					var configuration = menuItem.configuration;
					if (configuration && !ContextUtils.isConfigurationInRecentlyUsed(configuration))
						this._menu.removeChild(menuItem);
				}
			},
			
			_initializeMenuButton: function()
			{
				this._menu = new jazz.ui.ScrollableMenu({
					selectable: true,
					onOpen: dojo.hitch(this, this._onMenuOpen),
					onClose: dojo.hitch(this, this._onMenuClose)
				});
				this._menu.addChild(new jazz.ui.menu.NodeMenuItem({isTitle: true, label: this.messages.menuTitle}));
				
				this._menuPopup = new jazz.ui.MenuPopup({group: "jazz-app-BannerButtons", align: "right", menuProvider: dojo.hitch(this, this._menuProvider), menuClass:"jazz-app-Navbar-menu", injectCaret: true}, this._button);
			},
			
			_menuProvider:function() {
				return this._menu;
			},
			
			_isValid:function(configuration) {
				if (!configuration){
					return true;
				}

				var uri = configuration.uri;
				var config = new com.ibm.team.vvc.shared.core.ui.objects.ConfigurationResource();
				var uriget = VvcUtils.getVvcUri() + "/repository/configuration?uri="+uri;
				config.load(uriget);
				
				// This checks to see if the change set is shared and provides user with the message that they cannot select a shared change set.
				if (!config.uri ) {
					InfoContentUtils.showSimpleContentDialog(this.messages.archivedTitle, this.messages.sharedMessage);
					 return false;
				}

				else if (config.isWorkspace() || config.isSnapshot()) {
					if (config.isArchived()) {
						 InfoContentUtils.showSimpleContentDialog(this.messages.archivedTitle, this.messages.archivedUnknownMessage);
						return false;
					}
				}					
				
				return true;
			},
			_setSelectedAttr: function(item) {
				if (typeof item == "number") {
					var widgets = this._menu.getChildren();
					if (item < widgets.length) {
						item = widgets[item];
					}
				}
				if (this.selected == item || !item.isConfiguration) return;
				
				// if archived workspace or completed change set, provide error to user
				if (item.configuration != null) {
					if (!this._isValid(item.configuration)) {
						return;
					}
				}
				
				if (item != null && item.domNode) {
					this._menu&&this._menu.setSelected(item);			
					
					if (item.iconClass) {
						if (this._previousIconClass){
							dojo.removeClass(this._icon, this._previousIconClass);
						}
						dojo.addClass(this._icon, item.iconClass);
						this._previousIconClass = item.iconClass;
					} else {
						dojo.empty(this._icon);
					}
					this._label.innerHTML = UIUtils.truncateString(item.label);
				} 
				
				if (typeof this.selected !== "number" && this._initializing == false) this.onChange(item);
				this.selected = item;
			},

			_wasClicked: function(item) {
				this.set("selected", item);
			},

			_initializeSelectedMenuItem: function()
			{
				var currentContext = ContextUtils.getCurrentContext();
				
				if (currentContext == null)
					selectedMenuItem = new com.ibm.team.vvc.shared.web.ui.widgets.HideableMenuItem({
						label : this.messages.pleaseSelect,
						"class" : "com-ibm-team-vvc-context-menuItem",
					});
				else
				{
					selectedMenuItem = this._getMenuItemForConfiguration(currentContext);
					if (selectedMenuItem == null)
					{
						selectedMenuItem = this._constructMenuItem(currentContext);
						//43291 - If the currect context is archived, It should not be added to Recently Used List
						if (!currentContext.isArchived())
						  this.addChild(selectedMenuItem, 1);
					}
				}
				this._setSelectedAttr(selectedMenuItem);
			},
			
			_getMenuItemForConfiguration: function(configuration)
			{
				var menuItems = this._menu.getChildren();
				for(var i in menuItems) {
					var menuItem = menuItems[i];
					var menuConfig = menuItem.configuration;
					if (menuConfig && configuration.uri == menuConfig.uri)
						return menuItem;
				}
				return null;
			},
			
			_onMenuOpen: function() {
				this._open = true;
				this._toggleActiveClass(true);
				this._menu.domNode.style.width = "";
				var menuWidth = dojo.position(this._menu.domNode).w;
				var thisWidth = dojo.position(this.domNode).w;
				if (menuWidth < thisWidth) {
					dojo.style(this._menu.domNode, 'width', thisWidth + 'px');
				}
				if (this.selected) {
					this._menu.setSelected(this.selected);
				}
			},

			_onMenuClose: function() {
				this._open = false;
				this._toggleActiveClass(false);
			},
			
			_initializeRecentlyUsed: function()
			{
				var recentlyUsed = ContextUtils.getRecentlyUsed();
				
				for(var i in recentlyUsed) {
					var recentConfig = recentlyUsed[i];
					var menuItem = this._constructMenuItem(recentConfig);
					this.addChild(menuItem, 1);
				}
			},
			
			_initializeQuickCreateMenuItem: function(){
				
				var self = this;
				
				this._menu.addChild(new dijit.MenuSeparator());
				this._menu.addChild(new jazz.ui.menu.NodeMenuItem({isTitle: true, label: this.messages.quickCreateMenuTitle}));	
				
				this._newWorkspaceMenuItem = new com.ibm.team.vvc.shared.web.ui.widgets.HideableMenuItem({
					label : this.messages.newWorkspace,
					iconClass : "com-ibm-team-vvc-context-icon-new-workspace",
					onClick: function() {
						if (self._checkValid()) {
							self._newWorkspace();
						}
					},
					isInvisible: dojo.hitch(this, function(context) {
						if (this._workspaceBasedOffSnapshotOnly)
						{
							if (context.isSnapshot())
								return true;
							else
								return false;
						}
						return true;
					}),
					
					getDisabledText: dojo.hitch(this, function(context){
						if (!context.permissions || dojo.indexOf(context.permissions, SecurityConstants.permissions.createOrModifyWorkspace) < 0) {
							return SecurityUtils.getNoPermissionMessage(context.getProperty("controllingAreaName"));
						}
						return null;
					})
				});		
				this.addChild(this._newWorkspaceMenuItem);
				
				this._newSnapshotMenuItem = new com.ibm.team.vvc.shared.web.ui.widgets.HideableMenuItem({
					label : this.messages.newSnapshot,
					iconClass : "com-ibm-team-vvc-context-icon-new-snapshot",
					onClick: function() {
						if (self._checkValid()) {
							self._newSnapshot();
						}
					},
					isInvisible: function(context) {
						if (context.isWorkspace())
							return true;
						else
							return false;
					},
					
					getDisabledText: dojo.hitch(this, function(context){
						if (!context.permissions || dojo.indexOf(context.permissions, SecurityConstants.permissions.createOrModifySnapshot) < 0) {
							return SecurityUtils.getNoPermissionMessage(context.getProperty("controllingAreaName"));							
						}
						return null;
					})
				});				
				this.addChild(this._newSnapshotMenuItem);
			},
			
			_initializeMoreMenuItem: function()
			{
				var self = this;
				var moreMenuItem = new com.ibm.team.vvc.shared.web.ui.widgets.HideableMenuItem({
					label : this.messages.more,
					onClick: function() {
						var args = {};
						args.spaceUri = ContextUtils.getSpace();
							
						var picker = new com.ibm.team.vvc.shared.web.ui.picker.WorkspacePickerDialog(args);
						picker.onOk = function(configuration) {
							if (!self._isValid(configuration))							
								return;
							ContextUtils.setCurrentContext(configuration);
							self._initializeSelectedMenuItem();
							
							if (self.onContextChange) {
								self.onContextChange(configuration);
							}
						};
					}
				});				
				this.addChild(moreMenuItem);
			},
			
			_initializeManageChangesets: function()
			{
				var self = this;
				
				this._menu.addChild(new dijit.MenuSeparator());
				this._menu.addChild(new jazz.ui.menu.NodeMenuItem({isTitle: true, label: this.messages.manageChangesets}));	
				
				this._createChangesetsMenuItem = new com.ibm.team.vvc.shared.web.ui.widgets.HideableMenuItem({
					label : this.messages.createChangeset,
					iconClass : "com-ibm-team-vvc-context-icon-create-changeset",
					onClick: function() {
						if (self._checkValid()) {
							var form = new com.ibm.team.vvc.shared.web.ui.changesets.CreateChangesetForm({"configuration": ContextUtils.getCurrentContext().uri});
							form.show();
						}
					},
					isInvisible: function(context) {
						if (context != null && context.isWorkspace())
							return true;
						return false;
					},
					getDisabledText: dojo.hitch(this, function(context){
						if (!context.permissions || dojo.indexOf(context.permissions, SecurityConstants.permissions.update) < 0) {
							return SecurityUtils.getNoPermissionMessage(context.getProperty("controllingAreaName"));
						}
						return null;
					})
					
				});				
				this.addChild(this._createChangesetsMenuItem);
				
				this._renameChangesetMenuItem = new com.ibm.team.vvc.shared.web.ui.widgets.HideableMenuItem({
					label : this.messages.renameChangeset,
					iconClass : "com-ibm-team-vvc-context-icon-rename-changeset",
					onClick: dojo.hitch(this, function() {
						var changeset = this.selected.changeset;
						var name = prompt(this.messages.renamePrompt, changeset.getTitle());
						if (!name || name === changeset.getTitle()) {
							// if user has cancelled the input prompt
							return;
						}
						var deferred = ChangesetsClient.rename(changeset.getUri(), name);
						deferred.addCallback(this, dojo.hitch(this, function(){
							
							//update the cached context for the popup menu
							var currentContext = ContextUtils.getCurrentContext();
							var children = this._menu.getChildren();
							if (currentContext && children) {
								for (var i = 0; i < children.length; i++) {
									var child = children[i];
									if (child.configuration && (child.configuration.uri === currentContext.getUri())) {
										child.configuration.setTitle(name);
									}
								}							
							}
							
							//update the changeset object name (faster than reloading)
							changeset.setProperty(DCTERMS_TITLE, name);
							var escapedName = jazz.util.html.escape(name);
							this.selected.setLabel(escapedName);
							this._label.innerHTML = escapedName;
							UIUtils.showPageMessage(this.messages.changesetRenamed, 5000);
						}));
						deferred.addErrback(this, dojo.hitch(this, function(error) {
							alert(dojo.string.substitute(this.messages.failedToRenameChangeset, [deferred.ioArgs.xhr.responseText]));
						}));
					}),
					isInvisible: function(context) {
						if (context != null && context.isChangeset())
							return true;
						return false;
					},
					
					getDisabledText: dojo.hitch(this, function(context){
						var changeset = ChangesetsClient.getCurrentChangeset();
						if(changeset){
							if (!SecurityUtils.canAdministerOrModifyChangeset(changeset, context)) {
								return SecurityUtils.getNoPermissionChangesetMessage(context.getProperty("controllingAreaName"));
							}
						}
					})
				});				
				this.addChild(this._renameChangesetMenuItem);
				
				this._completeChangesetMenuItem = new com.ibm.team.vvc.shared.web.ui.widgets.HideableMenuItem({
					label : this.messages.completeChangeset,
					iconClass : "com-ibm-team-vvc-context-icon-complete-changeset",
					onClick: dojo.hitch(this, function() {
						var changeset = this.selected.changeset;
						InfoContentUtils.showQuestionDialog(
								this.messages.completeConfirmationTitle,
								dojo.string.substitute(this.messages.completeConfirmationContent, {changesetTitle: changeset.getTitle()}),
								this.messages.completeConfirmationOkButton,
								this.messages.cancelButton,
								dojo.hitch(this, function(){
									var deferred = ChangesetsClient.complete(changeset.getUri());
									deferred.addCallback(this, dojo.hitch(this, function(){
										this.selected.setAttribute("iconClass", "com-ibm-team-vvc-context-completed-changeset");
										dojo.addClass(this._icon, "com-ibm-team-vvc-context-completed-changeset");
										this._toggleMenuItems();
										UIUtils.showPageMessage(this.messages.changesetCompleted, 5000);
									}));
									deferred.addErrback(this, dojo.hitch(this, function(error) {
										alert(dojo.string.substitute(this.messages.failedToCompleteChangeset, [deferred.ioArgs.xhr.responseText]));
									}));
								})
						);
					}),
					isInvisible: function(context) {
						if (context != null && context.isChangeset() && context.isMutable())
							return true;
						return false;
					},
					
					getDisabledText: dojo.hitch(this, function(context){
						var changeset = ChangesetsClient.getCurrentChangeset();
						if(changeset){
							if (!SecurityUtils.canAdministerOrModifyChangeset(changeset, context)) {
								return SecurityUtils.getNoPermissionChangesetMessage(context.getProperty("controllingAreaName"));
							}
						}
					})
				});
				this.addChild(this._completeChangesetMenuItem);
				
				this._commitChangesetMenuItem = new com.ibm.team.vvc.shared.web.ui.widgets.HideableMenuItem({
					label : this.messages.commitChangeset,
					iconClass : "com-ibm-team-vvc-context-icon-commit-changeset",
					onClick: dojo.hitch(this, function() {
						var changeset = this.selected.changeset;
						InfoContentUtils.showQuestionDialog(
							this.messages.shareConfirmationTitle,
							dojo.string.substitute(this.messages.shareConfirmationContent, {changesetTitle: changeset.getTitle()}),
							this.messages.shareConfirmationOkButton,
							this.messages.cancelButton,
							dojo.hitch(this, function(){
								var deferred = ChangesetsClient.commit(changeset.getUri());
								deferred.addCallback(this, dojo.hitch(this, function(){
									alert(this.messages.changesetCommitted);
									this.onContextChange({uri: this.selected.configuration.getParentWorkspaceUri()});
								}));
								deferred.addErrback(this, dojo.hitch(this, function(error) {
									alert(dojo.string.substitute(this.messages.failedToCommitChangeset, [deferred.ioArgs.xhr.responseText]));
								}));
							})
						);
					}),
					isInvisible: function(context) {
						if (context != null && context.isChangeset())
							return true;
						return false;
					},
					
					getDisabledText: dojo.hitch(this, function(context){
						var changeset = ChangesetsClient.getCurrentChangeset();
						if(changeset){
							if (!SecurityUtils.canAdministerChangeset(changeset, context)) {
								return SecurityUtils.getNoPermissionChangesetMessage(context.getProperty("controllingAreaName"));
							}
						}
					})
				});				
				this.addChild(this._commitChangesetMenuItem);
				
				this._discardChangesetsMenuItem = new com.ibm.team.vvc.shared.web.ui.widgets.HideableMenuItem({
					label : this.messages.discardChangeset,
					iconClass : "com-ibm-team-vvc-context-icon-discard-changeset",
					onClick: dojo.hitch(this, function() {
						var changeset = this.selected.changeset;
						InfoContentUtils.showQuestionDialog(
							this.messages.discardConfirmationTitle,
							dojo.string.substitute(this.messages.discardConfirmationContent, {changesetTitle: changeset.getTitle()}),
							this.messages.discardConfirmationOkButton,
							this.messages.cancelButton,
							dojo.hitch(this, function(){
								var deferred = ChangesetsClient.discard(changeset.getUri());
								deferred.addCallback(this, dojo.hitch(this, function(){
									this.onContextChange({uri: this.selected.configuration.getParentWorkspaceUri()});
								}));
								deferred.addErrback(this, dojo.hitch(this, function(error) {
									alert(dojo.string.substitute(this.messages.failedToDiscardChangeset, [deferred.ioArgs.xhr.responseText]));
								}));
							})
						);
					}),
					isInvisible: function(context) {
						if (context != null && context.isChangeset() && context.isMutable())
							return true;
						return false;
					},
					
					getDisabledText: dojo.hitch(this, function(context){
						var changeset = ChangesetsClient.getCurrentChangeset();
						if(changeset){
							if (!SecurityUtils.canAdministerChangeset(changeset, context)) {
								return SecurityUtils.getNoPermissionChangesetMessage(context.getProperty("controllingAreaName"));
							}
						}
					})
				});				
				this.addChild(this._discardChangesetsMenuItem);
				
				this._linkWorkItemMenuItem = new com.ibm.team.vvc.shared.web.ui.widgets.HideableMenuItem({
					label : this.messages.linkToWorkItemTitle,
					iconClass : "com-ibm-team-vvc-link-to-workitem",
					onClick: function() {
						com.ibm.team.vvc.shared.web.ui.util.LinkToWorkItemUtil.linkToWorkItemAction();
					},
					isInvisible: function(context) {
						return (context != null && context.isChangeset());
					}
					
				});				
				this.addChild(this._linkWorkItemMenuItem);
				
				this._linkedWorkItemsMenuItem = new com.ibm.team.vvc.shared.web.ui.widgets.HideableMenuItem({
					label : this.messages.showLinkedWorkItemsTitle,
					onClick: function() {
						com.ibm.team.vvc.shared.web.ui.util.LinkToWorkItemUtil.showLinkedWorkItemsAction();
					},
					isInvisible: function(context) {
						return (context != null && context.isChangeset());
					}
				});				
				//31282: this.addChild(this._linkedWorkItemsMenuItem);				

				var showChangesetsExploreLabelHref = "<a class=\"noDecoration\" href=\"" + this._getShowChangesetsHref() + "\">" + this.messages.showChangesets + "</a>";
				this._showChangesetsMenuItem = new com.ibm.team.vvc.shared.web.ui.widgets.HideableMenuItem({
					label : showChangesetsExploreLabelHref,
					iconClass: "com-ibm-team-vvc-changesets-icon",
					onClick: function(e) {
						if((e instanceof MouseEvent && e.which != 2) || (e instanceof KeyboardEvent && (e.which === dojo.keys.ENTER || e.which == dojo.keys.SPACE))){
							self._showChangesets();
						}
					},
				});
				this.addChild(this._showChangesetsMenuItem);
			},
			
			_makePopupContents: function(configurationPopup, currentContext) {
				
				if (!configurationPopup) {
					configurationPopup = this._configurationPopup;
				}
				
				if (!currentContext) {
					currentContext = ContextUtils.getCurrentContext();
				}
				
				var contextTitle = jazz.util.html.escape(currentContext.getTitle());
				
				var configurationSpaceName = jazz.util.html.escape(currentContext.getSpaceName());
				if (!configurationSpaceName || configurationSpaceName.length === 0) {
					configurationSpaceName = this.messages.noParent;
				}
				
				var configurationSpace = "<div>" + this.messages.configurationSpace + configurationSpaceName + "</div>";
				
				var parentSnapshotName = jazz.util.html.escape(currentContext.getParentSnapshotName());
				if (!parentSnapshotName || parentSnapshotName.length === 0) {
					parentSnapshotName = this.messages.noParent;
				}

				var parentSnapshot = "";
				var parentWorkspaceName = jazz.util.html.escape(currentContext.getParentWorkspaceName());
				if (!parentWorkspaceName || parentWorkspaceName.length === 0) {
					parentWorkspaceName = this.messages.noParent;
					//don't bother showing the parent snapshot when there is no parent workspace
				} else if (currentContext.isWorkspace()) {
					var parentSnapshotName = jazz.util.html.escape(currentContext.getParentSnapshotName());
					if (!parentSnapshotName || parentSnapshotName.length === 0) {
						parentSnapshotName = this.messages.noParent;
					}
					
					parentSnapshot = "<div>" + this.messages.parentSnapshot + parentSnapshotName + "</div>";
				}
				
				var parentWorkspace = "<div>" + (currentContext.isWorkspace() ? this.messages.parentWorkspace : this.messages.workspace) + (currentContext.getParentWorkspaceName() ? jazz.util.html.escape(currentContext.getParentWorkspaceName()) : this.messages.noParent) + "</div>";
				
				var descriptionText = jazz.util.html.escape(currentContext.getDescription());
				if (!descriptionText || descriptionText.length === 0) {
					descriptionText = this.messages.noDescription;
				}
				
				description = "<div>" + this.messages.description + descriptionText + "</div>";
				
				configurationPopup.content.innerHTML =  "<div class = \"configurationPopup-divider\"><div style=\"float:left\"><div class=\"" + ContextUtils.getConfigurationIconClass(currentContext) + "\"></div></div><div class = \"configurationPopup-title\">" + contextTitle + "</div>" + "</div>" + configurationSpace + parentWorkspace + parentSnapshot + description;
	
				if (currentContext.isChangeset()) {
					
					if (!currentContext.changesetResource) {
						currentContext.changesetResource = new com.ibm.team.vvc.shared.core.ui.objects.ChangesetResource();
						currentContext.changesetResource.loadByConfiguration(currentContext);
					}
					
					
					if (currentContext.changesetResource) {
						var callback = new dojo.Deferred();
						CLMUtils.doOSLCQuery(this.providers, currentContext.changesetResource.uri, callback);
						
						callback.then(dojo.hitch(this, function(response){
							var uris = [], anchors = [], nodes = [];
							for(var i = 0; i < response.length; i++){

								var node = dojo.create("div", { "class":"configurationPopup-indent" });
								
								if (i === 0) {
									var mainChangeRequestsNode = dojo.create("div");
									mainChangeRequestsNode.appendChild(dojo.doc.createTextNode(this.messages.associatedWorkItems));
									configurationPopup.content.appendChild(mainChangeRequestsNode);
									
								}
								
								var idString = "-";
								var indexOfSlash = response[i].target.lastIndexOf("/");
								if (indexOfSlash > 0 && indexOfSlash < response[i].target.length - 1) {
									idString = response[i].target.substring(indexOfSlash + 1);
								}
								
								var img = dojo.doc.createElement("img");
								
								var targetUrl = dojo.doc.createTextNode(idString + ": " + response[i].target);
								
								var targetUrlAnchor = dojo.doc.createElement("a");
								targetUrlAnchor.href = response[i].target;
								targetUrlAnchor.appendChild(targetUrl);

								var workItemObj = {
									node: node,
									id: idString
								}
								nodes.push(workItemObj);
								node.appendChild(img);
								node.appendChild(targetUrlAnchor);
								
								uris.push(response[i].target);
								
								var obj = {
									url: response[i].target,
									targetUrl: targetUrl,
									img: img
								};
								anchors.push(obj);
							}
							
							nodes.sort(CLMUtils.idSorter);
							
							for (var i = 0; i < nodes.length; i++) {
								configurationPopup.content.appendChild(nodes[i].node);
							}
							
							var deferredCompact = jazz.client.xhrPost({
								url: net.jazz.ajax.contextRoot() + "/_compactRendering",
								handleAs: "json",
								content: {
									uri: uris
								}
							});

							var context = {
								anchors: anchors,
								messages: this.messages
							};
							
							//_err for individual failures
							deferredCompact.addCallback(context, dojo.hitch(context, function(result){
								for (var i = 0; i < this.anchors.length; i++) {
									var compact = result[this.anchors[i].url];
									var textContent = this.anchors[i].targetUrl.textContent;
									var nodeValue = this.anchors[i].targetUrl.nodeValue; //IE8
									
									if (compact._err) {
										if (textContent) {
											this.anchors[i].targetUrl.textContent = this.messages.unableToLoadTitle;
										} else if (nodeValue) {
											this.anchors[i].targetUrl.nodeValue = this.messages.unableToLoadTitle;
										}
									}
									if (compact && compact.oslcCompact) {
										if (compact.oslcCompact.title) {
											if (textContent) {
												this.anchors[i].targetUrl.textContent = compact.oslcCompact.title;
											} else if (nodeValue) {
												this.anchors[i].targetUrl.nodeValue = compact.oslcCompact.title;
											}
										} else {
											if (textContent) {
												this.anchors[i].targetUrl.textContent = this.messages.unableToLoadTitle;
											} else if (nodeValue) {
												this.anchors[i].targetUrl.nodeValue = this.messages.unableToLoadTitle;
											}
										}
										
										if (compact.oslcCompact.icon) {
											this.anchors[i].img.src = compact.oslcCompact.icon;
										}
									} else {
										if (textContent) {
											this.anchors[i].targetUrl.textContent = this.messages.unableToLoadTitle;
										} else if (nodeValue) {
											this.anchors[i].targetUrl.nodeValue = this.messages.unableToLoadTitle;
										}
									}
								}
							}));
							
							//all failed
							deferredCompact.addErrback(context, dojo.hitch(context, function(result){
								for (var i = 0; i < this.anchors.length; i++) {
									if (this.anchors[i].targetUrl.textContent) {
										this.anchors[i].targetUrl.textContent = this.messages.unableToLoadTitle;
									} else if (this.anchors[i].targetUrl.nodeValue) {
										this.anchors[i].targetUrl.nodeValue = this.messages.unableToLoadTitle;
									}
								}
							}));						

						}));
						
					}

				}
			},
			
			_initializeManageConfigurations: function()
			{
				var self = this;
				
				this._menu.addChild(new dijit.MenuSeparator());
				this._menu.addChild(new jazz.ui.menu.NodeMenuItem({isTitle: true, label: this.messages.manageConfigurations}));
				var title = dojo.string.substitute(this.messages.openCurrentConfiguration, {
					configTitle: ContextUtils.getCurrentContext().getTitle() // Should never be null!
				});
				var openCurrentConflabelHref = "<a class=\"noDecoration\" href=\"" + this._getOpenCurrentConfigurationHref() + "\">" + title + "</a>"; 
				this._openCurrentConfigurationMenuItem = new com.ibm.team.vvc.shared.web.ui.widgets.HideableMenuItem({
					label : openCurrentConflabelHref,
					iconClass : "com-ibm-team-vvc-context-icon-edit",
					onClick: function(e) {
						if((e instanceof MouseEvent && e.which != 2) || (e instanceof KeyboardEvent && (e.which === dojo.keys.ENTER || e.which == dojo.keys.SPACE))){
							self._openCurrentConfiguration();
						}
					},
				});				
				this.addChild(this._openCurrentConfigurationMenuItem);
				
				var explorerLabelHref = "<a class=\"noDecoration\" href=\"" + this._getExplorerHref() + "\">" + this.messages.exploreConfigurations + "</a>";
				var exploreMenuItem = new com.ibm.team.vvc.shared.web.ui.widgets.HideableMenuItem({
					label : explorerLabelHref,
					iconClass: "com-ibm-team-vvc-explorer-icon",
					onClick: function(e) {
						if((e instanceof MouseEvent && e.which != 2) || (e instanceof KeyboardEvent && (e.which === dojo.keys.ENTER || e.which == dojo.keys.SPACE))){
							self._showExplorer();
						}
					}
				});	
				this.addChild(exploreMenuItem);
			},
			
			_toggleMenuItems: function()
			{
				var currentContext = ContextUtils.getCurrentContext();
				if(currentContext){
					var menuItems = this._menu.getChildren();
					dojo.forEach(menuItems, dojo.hitch(this, function(item, index){
						if(item.isInvisible){
							var visible = item.isInvisible(currentContext);
							item.setVisible(!visible);
							if(visible && item.getDisabledText){									
								dojo.when(item.getDisabledText(currentContext), dojo.hitch(this, function(result){
									if (result) {
										item.set('disabled', true);
										item.set('title', result);
									}
								}));
							}
						}
					}));
				}
			},
			
			_constructMenuItem: function(configuration)
			{
				var menuItem = new com.ibm.team.vvc.shared.web.ui.widgets.HideableMenuItem({
					label : UIUtils.truncateString(jazz.util.html.escape(configuration.getTitle())),
					"class" : "com-ibm-team-vvc-context-menuItem",
					iconClass : ContextUtils.getConfigurationIconClass(configuration),
					configuration: configuration,
					changeset: ChangesetsClient.getCurrentChangeset(),
					isConfiguration: true
				});		
				return menuItem;
			},
			
			onChange: function(item)
			{
				if (item.configuration != null)
					ContextUtils.setCurrentContext(item.configuration);
				
				this._toggleMenuItems();
				
				if (this.onContextChange) {
					this.onContextChange(item.configuration);
				}
			},

			_onClick: function(event) {
				this._menuPopup.open();
			},

			_onMouseOver: function(event) {
				this._over = true;
				this._toggleActiveClass(true);
			},
			
			_onMouseOut: function(event) {
				this._over = false;
				this._toggleActiveClass(false);
			},
			
			_menuProvider: function()
			{
				return this._menu;
			},
			
			_toggleActiveClass: function(active) {
				dojo.toggleClass(this.domNode, 'active', active || this._open || this._over);
			},
			
			_openCurrentConfiguration: function()
			{
					window.location.href = this._getOpenCurrentConfigurationHref();
			},
			
			// Do not propogate this!!
			_openConfigurationHref: null,
			_getOpenCurrentConfigurationHref: function(){
				if(!this._openConfigurationHref){
					if (VvcUtils.getVvcUri() != null)
					{
						var currentContext = ContextUtils.getCurrentContext();
						if (currentContext == null)
						{
							alert(this.noConfigurationSelected);
							return;
						}
						var newLocation = VvcUtils.getVvcUri() + "/web?" + this.getUriOptions() + "#action=";
						if (!currentContext.isMutable() || currentContext.getBase() != null)
							newLocation += vvcConstants.EDIT_SNAPSHOT; 
						else
							newLocation += vvcConstants.EDIT_WORKSPACE;
						
						this._openConfigurationHref = newLocation;
					}	
				}
				return this._openConfigurationHref;
			},
			
			_newSnapshot:function()
			{
				this._newConfigurationForm({type: "snapshot"});
			},
			
			_newWorkspace: function()
			{
				this._newConfigurationForm({type: "workspace"});
			},
			
			_newConfigurationForm: function(args){
				if (VvcUtils.getVvcUri())
				{
					var form = new com.ibm.team.vvc.shared.web.ui.configurations.CreateConfigurationForm(args);
					
					form.loadConfiguration = dojo.hitch(this, function(configuration){	
						ContextUtils.setCurrentContext(configuration);
						this._initializeSelectedMenuItem();
						
						if (this.onContextChange) {
							this.onContextChange(configuration);
						}
					});
				}
			},
			
			_showExplorer: function()
			{
				if (VvcUtils.getVvcUri() != null)
				{
					window.location.href = this._getExplorerHref();
				}
			},
			
			// Do not propagate this!
			_explorerHref: null,
			_getExplorerHref: function(){
				if(!this._explorerHref){
					this._explorerHref = VvcUtils.getVvcUri() + "/web?" + this.getUriOptions() + "#action=" + vvcConstants.EXPLORE_CONFIGURATIONS;		
				}
				return this._explorerHref;
			},
			
			_showChangesets: function()
			{
				if (VvcUtils.getVvcUri() != null)
				{
					window.location.href = this._getShowChangesetsHref();
				}
			},
			
			// Do not propagate this!
			_showChangesetExplorerHref: null,
			_getShowChangesetsHref: function(){
				if(!this._showChangesetExplorerHref){
					this._showChangesetExplorerHref = VvcUtils.getVvcUri() + "/web?" + this.getUriOptions(true) + "#action=" + vvcConstants.EXPLORE_CHANGESETS;	
				}
				return this._showChangesetExplorerHref;
			},
			
			// sub class can override to include more additional parameters
			getUriOptions: function(returnOnlyWorkspace) 
			{
				var uriOptions = {};
				var currentContext = ContextUtils.getCurrentContext();
				var contextUri;
				uriOptions.spaceUri = ContextUtils.getSpace();
				
				if (currentContext != null){
					if (returnOnlyWorkspace && !currentContext.isMutable()){
						// want only workspace and snapshots are not mutable
						contextUri = currentContext.getParentWorkspaceUri();
					} else if (returnOnlyWorkspace && currentContext.isChangeset()){
						// want only workspace and changesets have a Change Set dimension 
						// and their owning workspace is their base
						contextUri = currentContext.getBase();
					} else {
						contextUri = currentContext.uri;
					}
					uriOptions.contextUri = contextUri;
				}
				return dojo.objectToQuery(uriOptions);
			}	
		}
	);
})();