/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide('com.ibm.team.vvc.shared.web.ui.context.ConfigurationContextSelect');

dojo.require('jazz.ui.Select');
dojo.require('dijit.MenuItem');
dojo.require('com.ibm.team.vvc.shared.web.ui.picker.WorkspacePickerDialog');
dojo.require("com.ibm.team.vvc.shared.web.ui.context.ContextUtils");



dojo.requireLocalization("com.ibm.team.vvc.shared.web.ui.context", "ConfigurationContextMessages");

(function ()
{
	var ContextUtils = 	com.ibm.team.vvc.shared.web.ui.context.ContextUtils;
	
	dojo.declare('com.ibm.team.vvc.shared.web.ui.context.ConfigurationContextSelect', [jazz.ui.Select], 
		{
			constructor: function (applicationKey)
			{
				this.inherited(arguments);
				this.messages = dojo.i18n.getLocalization("com.ibm.team.vvc.shared.web.ui.context", "ConfigurationContextMessages");
			},

			postCreate: function() {
				this.inherited(arguments);
				
				this._initializeRecentlyUsed();
				this._initializeMoreMenuItem();
				this._initializeSelectedMenuItem();
				
				this.domNode.style.display = "inline-block";
				this.domNode.style.width = "300px";
				this.domNode.style.margin = "2px 0px -7px 0px";
			},
			
			addChild: function(widget, insertIndex){
				this.inherited(arguments);
				
				if (widget.configuration == null)
					return;
				
				ContextUtils.addConfigurationToRecentlyUsed(widget.configuration);
				
				//remove any menuItems that are no longer in the recently used list
				var menuItems = this.getChildren();
				for(var i in menuItems) {
					var menuItem = menuItems[i];
					var configuration = menuItem.configuration;
					if (configuration && !ContextUtils.isConfigurationInRecentlyUsed(configuration))
						this.removeChild(menuItem);
				}
			},
			
			_initializeSelectedMenuItem: function()
			{
				var currentContext = ContextUtils.getCurrentContext();
				
				if (currentContext == null)
					selectedMenuItem = new dijit.MenuItem({
						label : this.messages.pleaseSelect,
						"class" : "com-ibm-team-vvc-context-menuItem",
					});
				else
				{
					selectedMenuItem = this._getMenuItemForConfiguration(currentContext);
					if (selectedMenuItem == null)
					{
						selectedMenuItem = this._constructMenuItem(currentContext);
						this.addChild(selectedMenuItem, 0);
					}
				}
				this._setSelectedAttr(selectedMenuItem);
			},
			
			_getMenuItemForConfiguration: function(configuration)
			{
				var menuItems = this.getChildren();
				for(var i in menuItems) {
					var menuItem = menuItems[i];
					var menuConfig = menuItem.configuration;
					if (menuConfig && configuration.uri == menuConfig.uri)
						return menuItem;
				}
				return null;
			},
			
			_initializeRecentlyUsed: function()
			{
				var recentlyUsed = ContextUtils.getRecentlyUsed();
				
				for(var i in recentlyUsed) {
					var recentConfig = recentlyUsed[i];
					this.addChild(this._constructMenuItem(recentConfig));
				}
			},
			
			_initializeMoreMenuItem: function()
			{
				var self = this;
				var moreMenuItem = new dijit.MenuItem({
					label : this.messages.more,
					isFocusable : function() {
						return false;
					},
					onClick: function() {
						var picker = new com.ibm.team.vvc.shared.web.ui.picker.WorkspacePickerDialog();
						picker.onOk = function(configuration) {
							ContextUtils.setCurrentContext(configuration);
							self._initializeSelectedMenuItem();
						};
					}
				});				
				this.addChild(moreMenuItem);
			},
			
			_constructMenuItem: function(configuration)
			{
				var menuItem = new dijit.MenuItem({
					label : configuration.title,
					"class" : "com-ibm-team-vvc-context-menuItem",
					iconClass : ContextUtils.getConfigurationIconClass(configuration),
					configuration: configuration
				});
				
				return menuItem;
			},
			
			onChange: function(item)
			{
				ContextUtils.setCurrentContext(item.configuration);
			},
			
			//Override the getChildren() method because of a bug in jazz.ui.Select. In the life-cycle of 
			//select the children get moved from container to menu when the menu button is invoked. Bad design in jazz.ui.Select I should submit a bug.
			getChildren: function() 
			{
				if (this._menu != null)
					return this._menu.getChildren();
				return this.inherited(arguments);
			},
			
			uninitialize: function() {
				if (this._menu) this._menu.destroyRecursive();
				this.inherited(arguments);
			}
		}
	);
})();