/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide('com.ibm.team.vvc.shared.web.ui.context.ConfigurationSpaceSelect');

dojo.require('jazz.ui.Select');
dojo.require('dijit.MenuItem');
dojo.require('com.ibm.team.vvc.shared.web.ui.picker.WorkspacePickerDialog');
dojo.require("com.ibm.team.vvc.shared.web.ui.context.ContextUtils");

dojo.require("com.ibm.team.vvc.shared.web.ui.util.VvcUtils");
var VvcUtils = com.ibm.team.vvc.shared.web.ui.util.VvcUtils;

dojo.requireLocalization("com.ibm.team.vvc.shared.web.ui.context", "ConfigurationContextMessages");

(function ()
{

	var ContextUtils = 	com.ibm.team.vvc.shared.web.ui.context.ContextUtils;
	
	dojo.declare('com.ibm.team.vvc.shared.web.ui.context.ConfigurationSpaceSelect', [jazz.ui.Select], 
		{
			constructor: function (applicationKey)
			{
				this.inherited(arguments);
				this.messages = dojo.i18n.getLocalization("com.ibm.team.vvc.shared.web.ui.context", "ConfigurationContextMessages");
			},

			postCreate: function() {
				this.inherited(arguments);
				
				this._initializeNewSpaceItem();
				this._initializeExistingSpaces();
				
				this.domNode.style.display = "inline-block";
				this.domNode.style.width = "300px";
				this.domNode.style.margin = "2px 0px -7px 0px";
			},
			
			_initializeExistingSpaces: function()
			{
				var existingSpaces = this._getExistingSpaces();
				
				for(var uri in existingSpaces) {
					var space = new Object();
					space.uri = uri;
					space.title = existingSpaces[uri];
					this.addChild(this._constructMenuItem(space));
				}
			},
			
			_initializeNewSpaceItem: function()
			{
				var self = this;
				var newMenuItem = new dijit.MenuItem({
					label : this.messages.createNewConfigurationSpace,
					"class" : "com-ibm-team-vvc-context-menuItem",
					space : null
				});				
				this.addChild(newMenuItem);
				this._setSelectedAttr(newMenuItem);
			},
			
			_constructMenuItem: function(space)
			{
				var menuItem = new dijit.MenuItem({
					label : space.title,
					"class" : "com-ibm-team-vvc-context-menuItem",
					iconClass : "com-ibm-team-vvc-context-space",
					space: space
				});
				
				return menuItem;
			},

			onChange: function(item)
			{
				dojo.publish("vvc.space_updated", [item.space]);
			},
	
			//Override the getChildren() method because of a bug in jazz.ui.Select. In the life-cycle of 
			//select the children get moved from container to menu when the menu button is invoked. Bad design in jazz.ui.Select I should submit a bug.
			getChildren: function() 
			{
				if (this._menu != null)
					return this._menu.getChildren();
				return this.inherited(arguments);
			},
			
			uninitialize: function() {
				if (this._menu) this._menu.destroyRecursive();
				this.inherited(arguments);
			},
			
			_getExistingSpaces: function()
			{
				self = this;
				
				jazz.client.xhrGet({
					handleAs: 'json',
					url:  VvcUtils.getVvcUri() + "/spaces/allspaces", //$NON-NLS-1$
					sync: true,
					handle: dojo.hitch(this, function(response, ioargs) {
						if (ioargs.xhr.status == 200) {
							self._existingSpaces = response;
						} else {
							console.log('error', response);
							alert("Invalid Response (Status:" + ioargs.xhr.status + "): "+ response.responseText);
						}
					}),

				});
				
				return self._existingSpaces;
			}
		}
	);
})();