/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012, 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {	

	dojo.provide("com.ibm.team.vvc.shared.web.ui.context.ContextUtils");
	
	dojo.require("com.ibm.team.vvc.shared.core.ui.objects.ConfigurationResource");
	dojo.require("com.ibm.team.vvc.shared.core.ui.objects.Setting");
	
	dojo.require("jazz.client.xhr");
	dojo.require("jazz.ui.toolbar.Button");

	dojo.require("net.jazz.ajax.contextRoot");
	dojo.requireLocalization("com.ibm.team.vvc.shared.web.ui.context", "ConfigurationContextMessages");
	
	dojo.require("com.ibm.team.vvc.shared.web.ui.util.VvcUtils");
	var VvcUtils = com.ibm.team.vvc.shared.web.ui.util.VvcUtils;
	
	var Setting = com.ibm.team.vvc.shared.core.ui.objects.Setting;
	
	var _spaceMapSetting = null;
	var messages = dojo.i18n.getLocalization("com.ibm.team.vvc.shared.web.ui.context", "ConfigurationContextMessages");
	
	com.ibm.team.vvc.shared.web.ui.context.ContextUtils = {
		
		MAX_RECENTLY_USED_SIZE : 5,
		spaceUri : null,
		_rootVvcUri: null,
		_currentContextCache : null,
		
		setSpace: function(space) {
			this.spaceUri = space;
		},
		
		getSpace: function() {
			return this.spaceUri;
		},
		
		getUserSetting: function() {
			if (!_spaceMapSetting) {
				var settingURI =  VvcUtils.getVvcUri() + "/settings/spaceMap";
				_spaceMapSetting = new Setting({
					uri: settingURI
				});
				_spaceMapSetting.load(settingURI);
			}
			return _spaceMapSetting;
		},
		
		getCurrentContext: function(resetCache)
		{
			if (this._currentContextCache && !resetCache) {
				return this._currentContextCache;
		 	}
		 			
			if (!this.spaceUri) {
				return null;
			}
			
			var spaceMap = null;
			var setting = this.getUserSetting();
			if (setting) {
				spaceMap = setting.getData();
			}

			if (!spaceMap) {
				spaceMap = [];
			}
			
			var entry = this.findSpaceEntry(spaceMap);
			if (entry && entry.currentContextUri) {
				var config = this.getConfiguration(entry.currentContextUri);
				if (config)
				{
					this._currentContextCache = config;
					return config;
				}
				else
				{
					delete entry.currentContextUri;
					if (setting) {
						setting.setData(spaceMap);
						setting.save();						
					}
				}
			}
			return null;
		},
		
		/*DEPRECATED, use getConfigurationAsync instead*/
		getConfiguration: function(configurationUri)
		{
			var config = new com.ibm.team.vvc.shared.core.ui.objects.ConfigurationResource();
			var uri = VvcUtils.getVvcUri() + "/repository/configuration?uri="+configurationUri;
			config.load(uri, function(response, ioArgs) {
				console.info("Could not load '" + configurationUri + "' Response:" + response.message); 
			});
			
			if (config.loaded)
				return config;
			return null;
		},
		
		getConfigurationAsync: function(configurationUri){
			var config = new com.ibm.team.vvc.shared.core.ui.objects.ConfigurationResource();
			var uri = VvcUtils.getVvcUri() + "/repository/configuration?uri="+configurationUri;
			return config.load(uri);
		},
		
		setCurrentContext: function(configuration)
		{
			var spaceMap = null;
			var setting = this.getUserSetting();
			if (setting) {
				spaceMap = setting.getData();
			}
			
			if (!spaceMap) {
				spaceMap = [];
			}
			
			var entry = this.findSpaceEntry(spaceMap);
			if (!entry) {
				entry = {};
				entry.spaceUri = this.spaceUri;
				spaceMap.push(entry);
			}
			
			if (configuration && configuration.uri) {
				if (entry.currentContextUri === configuration.uri) {
					return; // current context is not changed
				}
				entry.currentContextUri = configuration.uri;
			}
			
			if (setting) {
				setting.setData(spaceMap);
				setting.save();				
			}
			
			this._currentContextCache = null;
			dojo.publish("vvc.context_updated", [configuration]);
		},
		
		getRecentlyUsed: function()
		{
			return this._loadRecentlyUsedUris(this._getRecentlyUsedUris());
		},
		
		_getRecentlyUsedUris: function()
		{
			var spaceMap = null;
			var setting = this.getUserSetting();
			if (setting) {
				spaceMap = setting.getData();
			}
			if (!spaceMap) {
				spaceMap = [];
			}
			
			var entry = this.findSpaceEntry(spaceMap);
			if (entry && entry.recentlyUsedUris) {
				return entry.recentlyUsedUris;
			} else {
				return [];
			}
		},
		
		_loadRecentlyUsedUris: function(recentlyUsedUris)
		{
			var recentlyUsedConfigurations = [];
			for (i in recentlyUsedUris)
			{
				var recentlyUsedUri = recentlyUsedUris[i];
				if (recentlyUsedUri != null)
				{
					var config = this.getConfiguration(recentlyUsedUri);
					if (config != null && !config.isArchived())
						recentlyUsedConfigurations.push(config);
					else
						this._removeConfigurationFromRecentlyUsed(recentlyUsedUri);
				}
			}
			return recentlyUsedConfigurations;
		},
		
		_removeConfigurationFromRecentlyUsed: function(configUri)
		{
			var spaceMap = null;
			var setting = this.getUserSetting();
			if (setting) {
				spaceMap = setting.getData();
			}
			if (!spaceMap) {
				spaceMap = [];
			}
			
			var entry = this.findSpaceEntry(spaceMap);
			if (!entry) {
				entry = {};
				entry.spaceUri = this.spaceUri;
				spaceMap.push(entry);
			}			
			
			if (!entry.recentlyUsedUris) {
				entry.recentlyUsedUris = [];
			}
			
			var i = entry.recentlyUsedUris.length - 1;
			while (i >= 0)
			{
				if (entry.recentlyUsedUris[i] == configUri)
					entry.recentlyUsedUris.splice(i, 1);
				i--;
			}
			if (setting) {
				setting.setData(spaceMap);
				setting.save();				
			}
		},
		
		addConfigurationToRecentlyUsed: function(configuration)
		{
			var spaceMap = null;
			var setting = this.getUserSetting();
			if (setting) {
				spaceMap = setting.getData();
			}
			if (!spaceMap) {
				spaceMap = [];
			}
			
			var entry = this.findSpaceEntry(spaceMap);
			if (!entry) {
				entry = {};
				entry.spaceUri = this.spaceUri;
				spaceMap.push(entry);
			}			
			
			if (!entry.recentlyUsedUris) {
				entry.recentlyUsedUris = [];
			}
			
			if (this.isConfigurationInRecentlyUsed(configuration))
				return;
			
			entry.recentlyUsedUris.unshift(configuration.uri);
			while (entry.recentlyUsedUris.length > this.MAX_RECENTLY_USED_SIZE)
			{
				entry.recentlyUsedUris.pop();
			}
			
			if (setting) {
				setting.setData(spaceMap);
				setting.save();				
			}
		},
		
		isConfigurationInRecentlyUsed: function(configuration)
		{
			var recentlyUsedUris = this._getRecentlyUsedUris();
			if (recentlyUsedUris) {
				for (i in recentlyUsedUris)
				{
					if (configuration.uri == recentlyUsedUris[i])
						return true;
				}
			}
			return false;
		},
		
		getConfigurationIconClass: function(configuration) {
			if (configuration.isSnapshot())
				return "com-ibm-team-vvc-context-snapshot";
			else if (configuration.isChangeset())
			{
				if (configuration.isMutable())
					return "com-ibm-team-vvc-context-changeset";
				else
					return "com-ibm-team-vvc-context-completed-changeset";
			}
			return "com-ibm-team-vvc-context-workspace";
		},
		
		findSpaceEntry: function(spaceMap) {
			if (this.spaceUri != null && spaceMap != null) {
				for (var i in spaceMap) {
					var entry = spaceMap[i];
					if (entry && entry.spaceUri) {
						if (entry.spaceUri == this.spaceUri) {
							return entry;
						}
					}
				}
			}
			return null;
		},
		
		// the 2 functions below are copied from PageUtils.js
		getContextUri: function() {
			var uri = this.getRequestParmValue("contextUri");
			if (uri != null) {
				uri = unescape(uri);
			}
			return uri;			
		},
		
		getRequestParmValue: function(paramName, uri) {
			var params = uri;
			if (!params) {
				params = window.location.href;
			}
			var i = params.indexOf("?");
			if (i !== -1) {
				params = params.substring(i+1);
				params = params.replace("#", "&");
			} else {
				i = params.indexOf("#");
				params = params.substring(i);
			}
			var nvps = [];
			var rawNvps = params.split("&");
			for(var i=0; i<rawNvps.length; i++) {
				var nvp = rawNvps[i].split("=");
				if (nvp[0] === paramName) {
					return nvp[1];
				}
			}
			return null;
		},
		
		/**
		 * Builds a button which, when clicked, will redirect the browser to the VVC history service for the supplied
		 * resource.
		 * @param args.conceptUri The concept URI For the resource
		 * @param args.contextUri (optional) The current context URI, defaults to the current context URI.
		 * @param args.displayName (optional) The name of the resource.
		 * @param args.displayNameHref (optional) The name of the resource wrapped in a link back to the resource.
		 * @return jazz.ui.toolbar.Button
		 */
		buildHistoryButtonForResource: function(args){
			if(!args.contextUri){
				args.contextUri = this.getCurrentContext().uri;
			}
							
			if(args.resourceDisplayName){
				if(!args.resourceDisplayNameHref){
					args.resourceDisplayNameHref = "";
				}	
			} 
					
			var uri = VvcUtils.getVvcUri() + "/web?" + dojo.objectToQuery(args) + "#action=exploreHistory";
			var button = new jazz.ui.toolbar.Button({
				imgSrc: dojo.moduleUrl("com.ibm.team.vvc.shared.web", "graphics/history.gif"),
				title: messages.showHistory,
				href: uri
			});
			return button;
		}	
	};


}());