/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2013.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function(){
   
dojo.provide("com.ibm.team.vvc.server.core.ui.explorer.VvcExplorer");

dojo.require('dojo.i18n');
dojo.require("dijit._Widget");

dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("jazz.app.ua.PageHoverHelp");
dojo.require("jazz.ui.toolbar.Separator");

dojo.require("com.ibm.team.vvc.server.core.ui.internal.grid.VvcTreeGrid");

dojo.require("com.ibm.team.vvc.shared.web.ui.util.VvcCache");

dojo.require("com.ibm.team.vvc.server.core.ui.internal.util.PageUtil");
var PageUtil = com.ibm.team.vvc.server.core.ui.internal.util.PageUtil;

dojo.require("com.ibm.team.vvc.shared.web.ui.VvcConstants");
var vvcConstants = com.ibm.team.vvc.shared.web.ui.VvcConstants;

dojo.require("com.ibm.team.vvc.server.core.ui.internal.util.VvcButtonFactory");
var buttonFactory = com.ibm.team.vvc.server.core.ui.internal.util.VvcButtonFactory;

dojo.require("com.ibm.team.vvc.shared.web.ui.util.VvcUtils");
var VvcUtils = com.ibm.team.vvc.shared.web.ui.util.VvcUtils;

dojo.require("com.ibm.team.vvc.server.core.ui.explorer.VvcExplorerTable");

dojo.requireLocalization("com.ibm.team.vvc.server.core.ui.explorer", "VvcExplorerMessages");

dojo.require("com.ibm.team.vvc.shared.web.ui.notifications.notifications.VvcNotificationFactory");
var ncFactory = com.ibm.team.vvc.shared.web.ui.notifications.notifications.VvcNotificationFactory;

dojo.declare("com.ibm.team.vvc.server.core.ui.explorer.VvcExplorer", [com.ibm.team.vvc.server.core.ui.internal.grid.VvcTreeGrid], {
	  
	messages: dojo.i18n.getLocalization("com.ibm.team.vvc.server.core.ui.explorer", "VvcExplorerMessages"),
	_currentView: null,
	_viewStructureCache: null,
	_configurationCache: null,
	_showArchived: null,
		
	constructor: function(args){
		this._showArchived = false;
	},
		
	_initializeDatastore: function(args){
		if(this._configurationCache){
			this._configurationCache.clear();
		} else {
			this._configurationCache = new com.ibm.team.vvc.shared.web.ui.util.VvcCache({
				target: VvcUtils.getVvcUri() + "/client/single",
				bulkTarget: VvcUtils.getVvcUri() + "/client/bulk",
				idProperty: "uri",
				storageKey: vvcConstants.CONFIGURATION_CACHE_KEY
			});
		}
		
		if(this._viewStructureCache){
			this._viewStructureCache.clear();
		} else {
			this._viewStructureCache = new com.ibm.team.vvc.shared.web.ui.util.VvcCache({
				target: VvcUtils.getVvcUri() + "/explore/configurations",
				idProperty: "uri",
				storageKey: "viewCache"
			});	
		}
	},
		
	buildTitleNode: function(){
		var container = dojo.create("span");
		var iconNode = dojo.create("span", {
			innerHTML: "&nbsp;",
			"class": "icons-explorer-configuration"
		});
		var titleNode = dojo.create("span", {
			innerHTML: this.messages.configurationsPageTitle,
			"class": "explorerTableTitle"
		});
		var titleUa = new jazz.app.ua.PageHoverHelp({uaKey: "com.ibm.team.vvc.server.core.page_ConfigurationsExplorer001"});
		
		container.appendChild(iconNode);
		container.appendChild(titleNode);
		container.appendChild(titleUa.domNode);
		return container;
	},
	

	buildActionsNode: function(){
		var actionsBar = new jazz.ui.Toolbar();
		var compareButton = buttonFactory.getCompareButton(null,{disabled: true},dojo.hitch(this, function(){
			var length = this._selectedRows.order.length;
			var args = {
				descendantUri: this._selectedRows.order[0],
				otherDescendantUri: length === 3 ? this._selectedRows.order[2] : null,
				ancestorUri: length >= 2 ? this._selectedRows.order[1] : vvcConstants.COMPARE_MERGE_FIND_ANCESTOR
			};
			return args;
		}));
		actionsBar.addChild(compareButton);
		
		var deliverButton = buttonFactory.getDeliverButton(null,{disabled: true},dojo.hitch(this, function(){
			var length = this._selectedRows.order.length;
			var args = {
				descendantUri: this._selectedRows.order[0],
				otherDescendantUri: length === 2 ? this._selectedRows.order[1] : vvcConstants.COMPARE_MERGE_FIND_ANCESTOR,
			};
			return args;
		}));
		actionsBar.addChild(deliverButton);		
		actionsBar.addChild(new jazz.ui.toolbar.Separator());
		actionsBar.addChild(buttonFactory.getCollapseAllButton(null,{actionCallback: dojo.hitch(this, this.collapseAll)}));
		actionsBar.addChild(buttonFactory.getRefreshButton(null,{actionCallback: dojo.hitch(this, this.refresh, true)}));
		actionsBar.addChild(new jazz.ui.toolbar.Separator());
		var showArchivedButton = buttonFactory.getShowArchivedButton(null,{actionCallback: dojo.hitch(this, function(){
			this.showArchived(showArchivedButton.pressed);
		})}) 
		actionsBar.addChild(showArchivedButton);
//		actionsBar.addChild(this._getViewpointsButton());
		
		dojo.connect(this, "onRowSelectionChanged", dojo.hitch(this, function(){
			var length = this._selectedRows.order.length;
			var order = this._selectedRows.order;
			var data = this._selectedRows.data;
			
			// Need at least one selection
			if(length < 2){
				compareButton.set("enabled", false);
				deliverButton.set("enabled", false);
				return;
			}
			
			// Compare is good for any 1-3 selected items.
			if(length < 4){
				var enabled = true;
				for(var index = 0; index < length; index++){
					enabled = enabled && !data[order[index]].archive;
				}
				compareButton.set("enabled", enabled);
			} else {
				compareButton.set("enabled", false);
			}
			
			
			// Deliver needs 1-2 selections 
			if(length >= 1 && length <= 2){
				/*
				 * Enables/disables the deliver button based on the criteria that at least one of
				 * the selected items must be a workspace. 
				 */
				var enabled = false;
				if (order[0] && order[1] && 
						(data[order[0]].type === "workspace" || data[order[1]].type === "workspace")) {
					enabled = true;
				}
				var spaceUri = data[order[0]].spaceUri;
				for(var index = 0; index < length; index++){
					// Nobody is archived
					enabled = enabled && !data[order[index]].archive;
					// Everyone must belong to the same configuration space
					enabled = enabled && data[order[index]].spaceUri == spaceUri;
				}
				deliverButton.set("enabled", enabled);
			} else {
				deliverButton.set("enabled", false);
			}
			
			if(length > 3){
				ncFactory.createInfo({
					title: this.messages.warningTooManySelectionsTitle,
					subtitle: this.messages.warningTooManySelectionsSubtitle,
					description: this.messages.warningTooManySelectionsDescription,
					solution: this.messages.warningTooManySelectionsSolution,
					closeOnEvent: {context: this, event: "onRowSelectionChanged"}
				}).dispatch();
			}
			
		}));
		return actionsBar.domNode;
	},
	
	buildTreeGridTableNode: function(){
		this._gridTable = new com.ibm.team.vvc.server.core.ui.explorer.VvcExplorerTable();
		return this._gridTable.domNode;
	},
	
	refresh: function(/*boolean*/hardRefresh){
		this._gridTable.refresh();
		this._idRowMap = [];
		
		this._initializeDatastore();
				
		var query = {
				viewpoint: this._currentView,
				includeArchived: this._showArchived
		}
		var rootViewDef = this._viewStructureCache.get("roots", query);
		dojo.when(rootViewDef, dojo.hitch(this, function(rootView){
			dojo.when(this._configurationCache.startBulkTransaction(), dojo.hitch(this, function(){
				for(var rootUri in rootView){
					var confDef = this._configurationCache.get(rootUri, {configurationUri : rootUri});
					dojo.when(confDef, dojo.hitch(this, function(conf){
						conf.children = rootView[conf.uri];
						this._gridTable.addRow(conf);
					}))
					
					// Bootstrap the views!
					var viewDef = this._viewStructureCache.get(rootUri, {viewpoint: "flow", includeArchived: this._showArchived});
				}
				this._configurationCache.endBulkTransaction();
			}));
		}));

	},
		
	showArchived: function(/*boolean*/showArchived){
		this._showArchived = showArchived;
		this.refresh();
	},
	
	setFlowView: function(){
		return this._setView(vvcConstants.EXPLORER_FLOW_VIEW);
	},
	
	setDependencyView: function(){
		return this._setView(vvcConstants.EXPLORER_DEPENDENCY_VIEW);
	},
		
	_setView: function(view){
		this._currentView = view;
		this.refresh();
		return null;
	},
	
	/*
	 * This is specific to the VVC Explorer.  If it's needed elsewhere, move it into the 
	 * VvcButtonFactory.
	 */
	_getViewpointsButton: function(){
		var menu = new jazz.ui.Toolbar({
			"class": "actionMenuPopout",
		});
		
		var flowViewButton = new jazz.ui.toolbar.Button({
			imgClass: "icons-compare",
			title: "Flow View",
			label: "Flow View",
			onClick: dojo.hitch(this, function(){
				button.set("label", "Flow View");
				this.setFlowView();
			}),
		});
		
		var dependencyViewButton = new jazz.ui.toolbar.Button({
			imgClass: "icons-compare",
			title: "Dependency View",
			label: "Dependency View",
			onClick: dojo.hitch(this, function(){
				this.setDependencyView();
			}),
		});
		
		menu.addChild(flowViewButton);
		menu.addChild(dependencyViewButton);
		
		var button = new jazz.ui.toolbar.MenuButton({
			menuProvider: function(){
				return menu;
			},
			cacheMenu: true,
			title: "View",
			label: "View",
			imgClass: "icons-show-actions",
		});
		return button;
	},
	
	onRefreshRowById: function(id){
		this._configurationCache.clear(id);
		this._viewStructureCache.clear(id);
		var rows = this._idRowMap[id];
		this._idRowMap[id] = null;
		
		var structureQuery = {
			viewpoint: this._currentView, 
			includeArchived: this._showArchived,
			configurationUri: id,
		}
		
		dojo.forEach(rows, dojo.hitch(this, function(row, index, rows){
			dojo.when(this._configurationCache.get(id), dojo.hitch(this, function(conf){
				dojo.when(this._viewStructureCache.get(id, structureQuery), dojo.hitch(this, function(view){
					conf.children = view[id];
					var newRow = this._gridTable._buildRow(conf, row._childOf);
					dojo.place(newRow.domNode, row.domNode, "replace");
					var expanded = row._isExpanded;
					row.collapse();
					if(expanded){
						newRow.expand();
					}
				}));
			}))
		}));
	},
	
	onExpandRow: function(row, data, childOf){
		/*
		 * Need to guard against a race condition:  User expands row, then 
		 * quickly collapses it.  If the row needs to goto the server or takes
		 * some time to render, it may still appear, even though the parent row
		 * has been collapsed!
		 */
		if(row._isExpanded){
			var structureQuery = {
					viewpoint: this._currentView, 
					includeArchived: this._showArchived,
					configurationUri: data.uri,
					configurationSpace: data.spaceUri
			};
			var structureDef = this._viewStructureCache.get(data.uri, structureQuery);
			dojo.when(structureDef, dojo.hitch(this, function(results){
				var childUris = results[data.uri];
				dojo.when(this._configurationCache.startBulkTransaction(), dojo.hitch(this, function(){
					dojo.forEach(childUris, dojo.hitch(this, function(child, index, children){
						dojo.when(this._configurationCache.get(child, {configurationUri: child}), dojo.hitch(this, function(conf){
							var childQuery = {
									viewpoint: this._currentView, 
									includeArchived: this._showArchived,
									configurationUri: conf.uri,
									configurationSpace: data.spaceUri
							};
							dojo.when(this._viewStructureCache.get(conf.uri, childQuery), dojo.hitch(this, function(view){
								var children = view[conf.uri];
								conf.children = children;
								this._gridTable.addRow(conf, row);
							}));
						}));
					}));
					this._configurationCache.endBulkTransaction();
				}));
				this._expandedIds[data.uri] = true;
				sessionStorage.setItem(vvcConstants.PRESERVE_EXPANSION_STATE_STORAGE_KEY, dojo.toJson(this._expandedIds));
			}));
		}
	},
	
});
})();
