/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corporation 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
	dojo.provide("com.ibm.team.vvc.server.core.ui.internal.admin.ProjectAreaEditorVVCContribution");
	
	dojo.require("com.ibm.team.vvc.shared.web.ui.util.VvcUtils");
	var VvcUtils = com.ibm.team.vvc.shared.web.ui.util.VvcUtils;
	
	dojo.requireLocalization("com.ibm.team.vvc.server.core.ui.internal.admin", "ProjectAreaEditorVVCContributionMessages");
	
	dojo.declare("com.ibm.team.vvc.server.core.ui.internal.admin.ProjectAreaEditorVVCContribution", [dijit._Widget, dijit._Templated], {

		messages: null,
		
		templatePath: dojo.moduleUrl('com.ibm.team.vvc.server.core', 'ui/internal/admin/templates/ProjectAreaEditorVVCContribution.html'),
		
	    constructor: function(args) {
	    	this.messages = dojo.i18n.getLocalization("com.ibm.team.vvc.server.core.ui.internal.admin", "ProjectAreaEditorVVCContributionMessages"); 
	    },
	    
		postCreate: function() {
			this.inherited(arguments);
			
			this._checkbox.checked = false;
		},
		
		_checkboxOnChange: function(checked) {
			//if clearing the checkbox after a save, do not dirty the editor
			if (this._checkbox.checked) {
				this.editor.setDirty(true);
			}
		},
		
		/**
		 * This checkbox should never prevent saving
		 */
		canSave: function() {
			return true;
		},

		/**
		 * doSave() only needs to be called when the checkbox is checked
		 */
		changesPending: function() {
			return this._checkbox && this._checkbox.checked;
		},
		
		doSave: function(callback) {
    		jazz.client.xhrPost({
    			url: VvcUtils.getVvcUri() + "/spaces/updateSpaceName?projectUri=" + VvcUtils.getVvcUri() + "/process/project-areas/" + encodeURIComponent(this.editor.projectAreaItemId) + "&spaceName=" + encodeURIComponent(this.editor.name.value),
    			headers: {
    				"accept": "application/json"
    			},
    			handleAs: "json", //$NON-NLS-1$
    			error: dojo.hitch(this, function(error, result){
    				console.error(error);
    				callback(this.constructError(error, result.xhr.status));
    			}),
    			load: dojo.hitch(this, function(result) {
    				callback();
    			})
    		});
		},
		
		constructError: function(originalError, status) {
			var error = {
				headerMessage: dojo.string.substitute(this.messages.updateSpaceError, [ this.editor.name.value ])
			};
			
			if (status === 403) {
				error.responseText = this.messages.updateSpaceError403; 
			} else if (status === 404) {
				error.responseText = this.messages.updateSpaceError404; 
			} else if (status) {
				error.responseText = this.messages.updateSpaceErrorGeneral;
			} else {
				error.responseText = originalError.message;
			}
			
			return error;
		},
		
		/**
		 * Called from ProjectAreaEditor#customRefresh()
		 */
		refresh: function() {
			//do nothing
		}
			
	});

})();
