/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2013.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.team.vvc.server.core.ui.internal.configurations.ConfigurationCompactRendering");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("jazz.util.iframe");
dojo.require("jazz.client.xhr");
dojo.require("com.ibm.team.vvc.shared.core.ui.objects.Constants");
var Constants = com.ibm.team.vvc.shared.core.ui.objects.Constants;

dojo.require("com.ibm.team.vvc.shared.web.ui.util.VvcUtils");
var VvcUtils = com.ibm.team.vvc.shared.web.ui.util.VvcUtils;

dojo.require("com.ibm.team.vvc.shared.core.ui.objects.ConfigurationResource");

dojo.requireLocalization("com.ibm.team.vvc.server.core.ui.internal.configurations", "ConfigurationCompactRenderingMessages");


dojo.declare("com.ibm.team.vvc.server.core.ui.internal.configurations.ConfigurationCompactRendering", [dijit._Widget, dijit._Templated], {
	widgetsInTemplate: true,

	messages: dojo.i18n.getLocalization("com.ibm.team.vvc.server.core.ui.internal.configurations", "ConfigurationCompactRenderingMessages"),
	
	templatePath: dojo.moduleUrl('com.ibm.team.vvc.server.core', 'ui/internal/configurations/templates/ConfigurationCompactRendering.html'),
	
	postCreate: function() {
		this.inherited(arguments);
		
		if (location.search.length > 0) {
		var obj = dojo.queryToObject(location.search.substring(1));
		if (obj) {
			if (obj.configurationUri) {
				var context1 = {
					messages: this.messages,
					self: this
				};
			
				var parentConfig = new com.ibm.team.vvc.shared.core.ui.objects.ConfigurationResource();
				
				//if you are trying to access or debug from an independent ajax module, 
				//don't use VvcUtils.getVvcUri() because BootstrapProperties.FRONTSIDE_URL is not set
				//in that case, please use the commented code to obtain the contextRoot
				//var contextRootIndex = obj.configurationUri.lastIndexOf("/configurations/");
				//var contextRoot = obj.configurationUri.substring(0, contextRootIndex);
				var link = VvcUtils.getVvcUri() + "/repository/configuration?uri=" + encodeURIComponent(obj.configurationUri);
				var deferred1 = parentConfig.loadAsync(link);

				deferred1.then(dojo.hitch(context1, function(configuration, ioArgs) {
						
						if (!configuration) {
							dojo.addClass(this.self.tableNode, "configuration-compact-rendering-hidden");
							dojo.addClass(this.self.errorNode, "configuration-compact-rendering-visible");
							return;
						}
						
						try {
							var description = configuration.getDescription();
							if (!description || description.length == 0) {
								description = this.messages.empty;
							}
							this.self.descriptionNode.appendChild(dojo.doc.createTextNode(description));
						} catch (err) {
							this.self.descriptionNode.appendChild(dojo.doc.createTextNode(this.messages.unknown));
						}
						
						var configurationDetails = [];
						if (configuration.isSnapshot()) {
							configurationDetails["configurationType"] = this.messages.configurationTypeSnapshot;
						} else if (configuration.isWorkspace()) {
							configurationDetails["configurationType"] = this.messages.configurationTypeWorkspace;
						} else if (configuration.isChangeset()) {
							configurationDetails["configurationType"] = this.messages.configurationTypeChangeset;
						} else {
							configurationDetails["configurationType"] = this.messages.configurationTypeUnknown;
						}
						
						this.self.descriptiveNode.appendChild(dojo.doc.createTextNode(dojo.string.substitute(this.messages.descriptive, configurationDetails)));
						
						try {
							var creator = configuration.getCreator();
							
							var index = creator.lastIndexOf('/');
							
							this.self.creatorNode.appendChild(dojo.create("a", {href: creator, innerHTML: creator.substring(index + 1)}));
						} catch (err) {
							this.self.creatorNode.appendChild(dojo.doc.createTextNode(this.messages.unknown));
						}

						try {
							//in 4.0.6, the created date is a String (it is done properly as an integer in 5.0)
							var created = new Date(parseInt(configuration.getCreated()));
							created = created.toLocaleDateString() + " " + created.toLocaleTimeString();
							this.self.createdNode.appendChild(dojo.doc.createTextNode(created));
						} catch (err) {
							this.self.createdNode.appendChild(dojo.doc.createTextNode(this.messages.unknown));
						}
						
						try {
							var spaceName = configuration.getSpaceName();
							if (!spaceName) {
								spaceName = this.messages.unknown;
							}
							this.self.spaceNode.appendChild(dojo.doc.createTextNode(spaceName));
						} catch (err) {
							this.self.spaceNode.appendChild(dojo.doc.createTextNode(this.messages.unknown));
						}
						
						var parentType;
						var parentUri = configuration.getParentSnapshotUri();
						var parentName;
						try {
							//is it a snapshot?
							if (parentUri) {
								parentName = configuration.getParentSnapshotName();
								if (!parentName) {
									parentName = parentUri;
								}
								parentType = this.messages.parentSnapshot;
							}
							
							//is it a workspace?
							if (!parentUri) {
								parentUri = configuration.getParentWorkspaceUri();
								
								if (parentUri) {
									parentName = configuration.getParentWorkspaceName();
									if (!parentName) {
										parentName = parentUri;
									}
									parentType = configuration.isMutable() ? this.messages.parentFlowTarget : this.messages.parentWorkspace;
								}
							}
						} catch (err) {
							parentUri = undefined;
							this.self.parentLabel.appendChild(dojo.doc.createTextNode(this.messages.parent));
							this.self.parentNode.appendChild(dojo.doc.createTextNode(this.messages.unknown));
						}
						
						//is there even a parent?
						if (parentUri) {
							this.self.parentLabel.appendChild(dojo.doc.createTextNode(parentType));
							this.self.parentNode.appendChild(dojo.create("a", {href: parentUri, innerHTML: parentName}));
						} else {
							this.self.parentLabel.appendChild(dojo.doc.createTextNode(this.messages.parent));
							this.self.parentNode.appendChild(dojo.doc.createTextNode(this.messages.empty));
						}
						
					}),
					dojo.hitch(this, function(responseerror) {
						dojo.addClass(this.self.tableNode, "configuration-compact-rendering-hidden");
						dojo.addClass(this.self.errorNode, "configuration-compact-rendering-visible");
					}));
					
				} else {
					dojo.addClass(this.tableNode, "configuration-compact-rendering-hidden");
					dojo.addClass(this.errorNode, "configuration-compact-rendering-visible");
				}
			}
		
		}
		
		if (window.postMessage && window.parent) {
			var newPreviewHeight = jazz.util.iframe.getDocumentHeight();
			var resizeMessage = "oslc-preview-height:" + newPreviewHeight;
			window.parent.postMessage(resizeMessage, "*");
			resizeMessage = "jazz-compact-rendering-preview-height:" + newPreviewHeight;
			window.parent.postMessage(resizeMessage, "*");
		} 
	}
	
});

})();
