/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2013.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function(){

dojo.provide("com.ibm.team.vvc.server.core.ui.internal.deliver.ChangesetPickerStep");

dojo.require("jazz.ui.wizard.WizardStep");
var WizardStep = jazz.ui.wizard.WizardStep;

dojo.require("com.ibm.team.vvc.shared.core.ui.objects.Constants");
var Constants = com.ibm.team.vvc.shared.core.ui.objects.Constants;

dojo.require("com.ibm.team.vvc.shared.core.ui.objects.DeliverableChangeset");
var DeliverableChangeset = com.ibm.team.vvc.shared.core.ui.objects.DeliverableChangeset;

dojo.require("dijit.form.CheckBox");
dojo.require("com.ibm.team.vvc.shared.web.ui.changesets.ChangesetsWidgit");

dojo.requireLocalization("com.ibm.team.vvc.server.core.ui.internal.deliver", "ChangesetPickerStepMessages");


dojo.declare("com.ibm.team.vvc.server.core.ui.internal.deliver.ChangesetPickerStep", [WizardStep, dijit._Widget, dijit._Templated], {
	
	messages : dojo.i18n.getLocalization("com.ibm.team.vvc.server.core.ui.internal.deliver", "ChangesetPickerStepMessages"),	
	templateString: dojo.cache('com.ibm.team.vvc.server.core', 'ui/internal/deliver/templates/ChangesetPickerStep.html'),
	widgetsInTemplate: true,
	
	session: null,
	
	/**
	 * The DeliverWizardApplication instance
	 */
	wizardApp: null,
	
	step1: null,
	
	initialized: false,
	
	_changesetMap: {},
	
	_invertedGapMap: {},
	
	_selectedChangesets: {},
	
	_changesetCheckboxAll: null,
			
	/**
	 * Arguments:
	 * - session: DeliverSession.js object
	 */
	constructor: function(args){
		if (args) {
			if (args.session) {
				this.session = args.session;
			}	
			
			if (args.wizardApp) {
				this.wizardApp = args.wizardApp;
			}		
		}
		
		this.stepTitle = this.messages.step2Title_deliver;
	},
	
	postCreate: function() {
		dojo.connect(this.partialChangesetsCheckbox, "onChange", this, this.selectAllDependent);
		
		if (this.session) {
			if (this.session.isCompareOnly()) {
				this.headline.innerHTML = this.messages.step2Title_compare;
				this.description.innerHTML = this.messages.desc_compare;
				this.partialChangesetsCheckbox.set('checked', true);
				this.partialCheckboxContainer.style.display = "none";
				this.noChanges.innerHTML = this.messages.noChanges_compare;
			}
			
			this.initialized = true;
			this.reload();
		}
	},
	
	reload: function() {
		if (!this.initialized) {
			return;
		}
		
		// destroy all previous checkboxe
		if (this._changesetMap && this.size(this._changesetMap) > 0) {
			for (var concept in this._changesetMap) {
				if (this._changesetMap[concept].checkBox) {
					this._changesetMap[concept].checkBox.destroy();
				}
			}
		}
		
		if (this._changesetCheckboxAll) {
			this._changesetCheckboxAll.destroy();
		}
		
		if (this._changesetMap) {
			for (var concept in this._changesetMap) {
				if (this._changesetMap[concept].checkBox) {
					this._changesetMap[concept].checkBox.destroy();
				}
			}
		}
		
		this._changesetMap = {};
		this._invertedGapMap = {};
		this._selectedChangesets = {};
		
		var selectedChangesets = this.session.getSelectedChangesets();
		
		if (selectedChangesets) {
			if (!(selectedChangesets instanceof Array)) {
				selectedChangesets = [selectedChangesets];
			}
			
			for (var i = 0; i < selectedChangesets.length; i++) {
				this._selectedChangesets[selectedChangesets[i]] = true;
			}
		}
		
		this.changesetListContainer.innerHTML = "";
		
		var changesets = this.session.getDeliverableChangesets();
		if (changesets) {
			if (!(changesets instanceof Array)) {
				changesets = [changesets];
			}
			
			var tempChangesetList = [];
			
			// first, we map out all the changesets
			for (var i = 0; i < changesets.length; i++) {
				var changeset = changesets[i];
				this._changesetMap[changeset.getChangesetConcept()] = changeset;
				tempChangesetList.push(changeset);
			}
			
			// before we do anything, we need to sort the changesets
			tempChangesetList.sort(this.changesetComparator);
			
			// next we construct a template
			var headerline = "<th class='headerLine headerColumnLine'></th>";
			var template = "<div class='tableContents toolbar-table'>" +
					"<table width='100%' class='jazz-ui-StyledTable' cellspacing='1'>" +
						"<thead class='table-primary table-headless'>" +
							"<tr class='headerLineTop'>" + headerline + headerline + headerline + headerline + headerline +"</tr>" +
							"<tr class='collapsableHeading'>" +
								"<th class='headerColumn checkboxCol'><div id='changesetCheckAll' /></th>" +
								"<th class='headerColumn'><div>" + this.messages.column_name +"</div></th>" +
								"<th class='headerColumn'><div>" + this.messages.column_state +"</div></th>" +
								"<th class='headerColumn'><div>" + this.messages.column_creator +"</div></th>" +
								"<th class='headerColumn'><div>" + this.messages.column_gapDependentCs +"</div></th>" +
							"</tr>" +
							"<tr class='headerLineTop'>" + headerline + headerline + headerline + headerline + headerline +"</tr>" +
						"</thead>";
			
			for (var i = 0; i < tempChangesetList.length; i++) {
				var changeset = tempChangesetList[i];
				var title = changeset.getTitle();
				var gaps = changeset.getGapDependentChangesets();
				var creator = changeset.getCreator();
				var state = changeset.getState();
				var concept = changeset.getChangesetConcept();
				
				if (state === Constants.STATE_COMMITTED) {
					state = this.messages.stateCommitted;
				} else if (state === Constants.STATE_COMPLETED) {
					state = this.messages.stateCompleted;
				} else {
					state = this.messages.statePending;
				}
				
				template += "<tr><td class='checkboxCol'><div id='changesetCheck" + i + "' /></td>"; 
				
				template += "<td>" + title + "</td><td>" + state + "</td><td>" + creator + "</td><td>";
				
				// generate changeset gaps
				if (gaps) {
					template += "<ol>";
					if (gaps instanceof Array) {
						for (var j = 0; j < gaps.length; j++) {
							var gapCs = this._changesetMap[gaps[j]];
							if (gapCs) {
								template += "<li>" + gapCs.getTitle() + "</li>";
								
								var gapCsConcept = gapCs.getChangesetConcept();
								// also put this in the inverted gap map
								if (!this._invertedGapMap[gapCsConcept]) {
									this._invertedGapMap[gapCsConcept] = [];
								}
								
								this._invertedGapMap[gapCsConcept].push(concept);
							}
						}
					} else {
						var gapCs = this._changesetMap[gaps];
						if (gapCs) {
							template += "<li>" + gapCs.getTitle() + "</li>";
							
							var gapCsConcept = gapCs.getChangesetConcept();
							// also put this in the inverted gap map
							if (!this._invertedGapMap[gapCsConcept]) {
								this._invertedGapMap[gapCsConcept] = [];
							}
							
							this._invertedGapMap[gapCsConcept].push(concept);
						}
					}
					
					template += "</ol>";
				}
				
				template += "</td></tr>";
			}
			
			template += "</table></div>";
			
			this.changesetListContainer.innerHTML = template;
			
			/*
			 * Now we have to create the checkboxes for each changeset.
			 * 
			 * To do this, we have to wait until the browser finishes 
			 * rendering changesetListContainer so that we can then call
			 * dojo.byId on each of the table cells.
			 */

			var self = this;
			this.waitUntil(
			  function () {
				  var numChecked = 0;
				  
				  // create the checkboxes for each changeset
				  for (var i = 0; i < tempChangesetList.length; i++) {
					  var changeset = tempChangesetList[i];
					  var nodeId = "changesetCheck" + i;
					  var node = dojo.byId(nodeId);
					  var checked = false;
					  if (self._selectedChangesets[changeset.getChangesetConcept()]) {
						  checked = true;
						  numChecked++;
					  }
					  
					  // create the checkbox 
					  var checkBox = new dijit.form.CheckBox({
					        name: "checkBox",
					        value: changeset.getChangesetConcept(),
					        checked: checked,
					        onChange: function(b){ self.onCheckboxSelected(this.concept, b); }
					    }, node);	
					  checkBox.concept = changeset.getChangesetConcept();
					  
					  changeset.checkBox = checkBox;
				  }

				  // create the changesetCheckAll checkbox
				  var node = dojo.byId("changesetCheckAll");
				  var checked = numChecked === tempChangesetList.length;
				  self._changesetCheckboxAll = new dijit.form.CheckBox({
				        name: "changesetCheckboxAll",
				        value: "changesetCheckboxAll",
				        checked: checked,
				        onChange: function(b){ self.onChangesetCheckboxAllSelected(b); }
				    }, node);	
				  
				  // make sure all dependent changesets are checked.
				  self.selectAllDependent();
				  
				  // disable the UI if the session is already initialized
				  self.doValidation();
			  },
			  function() {
			    return !!(dojo.byId('changesetListContainer').innerHTML !== '');
			  },
			  50 // amout to wait between checks
			)();
		}
		
		if (!this._changesetMap || this.size(this._changesetMap) == 0) {
			this.noChanges.style.display = "block";
		} else {
			this.noChanges.style.display = "none";
		}
	},
	
	 waitUntil : function (fn, condition, interval) {
	    interval = interval || 100;

	    var shell = function () {
	            var timer = setInterval(
	                function () {
	                    var check;

	                    try { check = !!(condition()); } catch (e) { check = false; }

	                    if (check) {
	                        clearInterval(timer);
	                        delete timer;
	                        fn();
	                    }
	                },
	                interval
	            );
	        };
 
	    return shell;
	},
	
	changesetComparator: function(a, b) {
		var aOrder = a.getOrder();
		var bOrder = b.getOrder();
		
		// invert the changeset order to display non-dependent, 
		// most-recent changesets at the top
		if (aOrder === bOrder) {
			return 0;
		} else if (aOrder < bOrder) {
			return 1;
		} else {
			return -1;
		}
	},
	
	onChangesetCheckboxAllSelected: function(selected) {
		for (var concept in this._changesetMap) {
			this._changesetMap[concept].checkBox.set('checked', selected);
		}
	},
	
	onCheckboxSelected: function(changesetConcept, selected) {
		if (!this.partialChangesetsCheckbox.checked) {
			if (selected) {				
				// if this changeset was selected, then check all changesets 
				// that are dependents
				var gaps = this._changesetMap[changesetConcept].getGapDependentChangesets();
				if (gaps) {
					if (gaps instanceof Array) {
						for (var i = 0; i < gaps.length; i++) {
							this._changesetMap[gaps[i]].checkBox.set('checked', true);
						}
					} else {
						this._changesetMap[gaps].checkBox.set('checked', true);
					}
				}
			} else {
				// if this changeset was unselected, then check if this changeset
				// is a dependent changeset of another that is selected. And if it is,
				// prevent this changeset from being unselected.
				var invertedGaps = this._invertedGapMap[changesetConcept];
				if (invertedGaps) {
					if (invertedGaps instanceof Array) {
						var changed = false;
						for (var i = 0; i < invertedGaps.length; i++) {
							if (this._changesetMap[invertedGaps[i]].checkBox.checked) {
								changed = true;
								this._changesetMap[changesetConcept].checkBox.set('checked', true);
							}
						}
						
						if (changed) {
							alert(this.messages.warning_cannotUnselect_deliver);
						}
					} else {
						if (this._changesetMap[invertedGaps].checkBox.checked) {
							this._changesetMap[changesetConcept].checkBox.set('checked', true);
							alert(this.messages.warning_cannotUnselect_deliver);
						}
					}
				}
			}
		}
		
		this._selectedChangesets[changesetConcept] = selected;
		
		this.doValidation();
	},
	
	selectAllDependent: function() {
		if (!this.initialized) {
			return;
		}
		
		if (!this.partialChangesetsCheckbox.checked) {
			for (var concept in this._changesetMap) {
				if (this._changesetMap[concept].checkBox.checked) {
					var gaps = this._changesetMap[concept].getGapDependentChangesets();
					if (gaps) {
						if (gaps instanceof Array) {
							for (var i = 0; i < gaps.length; i++) {
								this._changesetMap[gaps[i]].checkBox.set('checked', true);
							}
						} else {
							this._changesetMap[gaps].checkBox.set('checked', true);
						}
					}
				}
			}
		}
	},
	
	disableUi: function() {
		for (var concept in this._changesetMap) {
			this._changesetMap[concept].checkBox.set("disabled", true);
		}
		
		this.partialChangesetsCheckbox.set("disabled", true);
		this._changesetCheckboxAll.set("disabled", true);
	},
	
	doValidation: function() {
		this.triggerButtonStateUpdate();
		
		// finally, disable all of UI if session is no longer in uninitialized
		// mode or is not available entirely
		if (!this.session
				|| this.session.getState() !== Constants.DELIVER_SESSION_STATE_UNINITIALIZED) {
			this.disableUi();
		}
	},
	
	validate: function() {
		var selected = false;
		if (this._selectedChangesets) {
			for (var concept in this._selectedChangesets) {
				if (this._selectedChangesets[concept]) {
					selected = true;
					break;
				} 
			}
		}
		
		if (selected) {
			return [];
		} else {
			// just return some array to indicate we have validation problems
			return [selected];
		}
	},
	
	canFinish: function() {
		if (this.session) {
			return this.session.isCompareOnly();
		} else {
			return false;
		}
	},
	
	onFinish: function() {
		this.wizardApp.performFinish();
	},
	
	onNext: function() {
		// need to update the deliver session if need be
		if (this.session
				&& this.session.getState() === Constants.DELIVER_SESSION_STATE_UNINITIALIZED) {
			this.wizardApp.showLoadingMessage();
			
			// compile a list of changesets to select
			var selectedChangesets = [];
			for (var changeset in this._selectedChangesets) {
				if (this._selectedChangesets[changeset]) {
					selectedChangesets.push(changeset);
				}
			}
			
			this.session.setSelectedChangesets(selectedChangesets);
			this.session.setApplyCompleteChangesetsOnly(!this.partialChangesetsCheckbox.checked);
			this.session.setState(Constants.DELIVER_SESSION_STATE_PENDING);

			var self = this;
			this.session.doSaveAsync(
					function(response, ioArgs) {
						// reload the session since changeset calculation may have changed
						self.disableUi();
						self.wizardApp.updateSession(self.session, self);
						self.wizardApp.hideLoadingMessage();
						self.wizard.processNext();
					},
					
					function(response, ioArgs) {
						self.session.setState(Constants.DELIVER_SESSION_STATE_UNINITIALIZED);
						self.wizardApp.showErrorMessage(response, ioArgs);
						self.wizardApp.hideLoadingMessage();
					}
			);
		} else {
			this.inherited(arguments);	
		}		
	},
	
	size: function(obj) {
	    var size = 0;
	    for (var key in obj) {
	        if (obj.hasOwnProperty(key)) {
	        	size++;
	        }
	    }
	    return size;
	},
});

})();