/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2013.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function(){

dojo.provide("com.ibm.team.vvc.server.core.ui.internal.deliver.DeliverSummaryStep");

dojo.require("jazz.ui.wizard.WizardStep");
var WizardStep = jazz.ui.wizard.WizardStep;

dojo.require("com.ibm.team.vvc.shared.core.ui.objects.Constants");
var Constants = com.ibm.team.vvc.shared.core.ui.objects.Constants;

dojo.require("jazz.ui.StyledBox");
dojo.require('jazz.ui.Select');
dojo.require("dijit.form.Button");
dojo.require("dojo.DeferredList");

dojo.requireLocalization("com.ibm.team.vvc.server.core.ui.internal.deliver", "DeliverSummaryStepMessages");

dojo.require("com.ibm.team.vvc.shared.web.ui.util.VvcCache");
var VvcCache = com.ibm.team.vvc.shared.web.ui.util.VvcCache;

dojo.declare("com.ibm.team.vvc.server.core.ui.internal.deliver.DeliverSummaryStep", [WizardStep, dijit._Widget, dijit._Templated], {
	
	messages : dojo.i18n.getLocalization("com.ibm.team.vvc.server.core.ui.internal.deliver", "DeliverSummaryStepMessages"),	
	templateString: dojo.cache('com.ibm.team.vvc.server.core', 'ui/internal/deliver/templates/DeliverSummaryStep.html'),
	widgetsInTemplate: true,
	
	/**
	 * The session
	 */
	session: null,
	
	_cache: null,
			
	/**
	 * Arguments:
	 * - session: DeliverSession.js object
	 */
	constructor: function(args){
		this._cache = new VvcCache({
			idProperty: "uri",
			storageKey: vvcConstants.CONFIGURATION_CACHE_KEY
		});
		
		if (args) {
			if (args.session) {
				this.session = args.session;
			}
			if (args.wizardApp) {
				this.wizardApp = args.wizardApp;
			}
		}
		
		this.stepTitle = this.messages.finalStepTitle;
	},
	
	postCreate: function() {
		this.reload();
	},
	
	reload: function() {
		var sourceUri = this.session.getSourceUri();
		if (sourceUri) {
			dojo.when(this._cache.get(sourceUri), dojo.hitch(this, function(config){
				this.sourceNode.innerHTML = config.title;
			}));
		}
		
		var targetUri = this.session.getTargetUri();
		if (targetUri) {
			dojo.when(this._cache.get(targetUri), dojo.hitch(this, function(config){
				this.targetNode.innerHTML = config.title;
			}));
		}
		
		if (this.session.isApplyCompleteChangesetsOnly()) {
			this.shareAfterFinishCheckbox.set("checked", true);
			this.shareAfterFinishCheckbox.set("disabled", true);
		} else {
			this.shareAfterFinishCheckbox.set("disabled", false);
		}
	},
	
	onFinish: function() {
		this.session.setCommitChangeSet(this.shareAfterFinishCheckbox.checked);
		this.wizardApp.updateSession(this.session, this);
		this.wizardApp.performFinish();
	},
	
	canFinish: function() {
		return true;
	},
});

})();