/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.team.vvc.server.core.ui.internal.deliver.DeliverWizardApplication");

dojo.require("com.ibm.team.repository.web.client.internal.AdminClient");
var AdminClient = com.ibm.team.repository.web.client.internal.AdminClient;

dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;

dojo.require("jazz.ui.Wizard");
var Wizard = jazz.ui.Wizard;

dojo.require("jazz.ui.wizard.WizardStep");
var WizardStep = jazz.ui.wizard.WizardStep;

dojo.require("com.ibm.team.repository.web.ui.internal.admin.NotAuthorizedView");
var NotAuthorizedView = com.ibm.team.repository.web.ui.internal.admin.NotAuthorizedView;

dojo.require("com.ibm.team.vvc.shared.core.ui.objects.DeliverSession");
var DeliverSession = com.ibm.team.vvc.shared.core.ui.objects.DeliverSession;

dojo.require("com.ibm.team.vvc.server.core.ui.internal.deliver.SourceTargetPickerStep");
var SourceTargetPickerStep = com.ibm.team.vvc.server.core.ui.internal.deliver.SourceTargetPickerStep;

dojo.require("com.ibm.team.vvc.server.core.ui.internal.deliver.DeliverSummaryStep");
var DeliverSummaryStep = com.ibm.team.vvc.server.core.ui.internal.deliver.DeliverSummaryStep;

dojo.require("com.ibm.team.vvc.server.core.ui.internal.deliver.ChangesetPickerStep");
var ChangesetPickerStep = com.ibm.team.vvc.server.core.ui.internal.deliver.ChangesetPickerStep;

dojo.require("jazz.ui.wizard.ExternalWizardStep");
var ExternalWizardStep = jazz.ui.wizard.ExternalWizardStep;

dojo.require("com.ibm.team.vvc.shared.core.ui.objects.Constants");
var Constants = com.ibm.team.vvc.shared.core.ui.objects.Constants;

dojo.require("com.ibm.team.vvc.shared.web.ui.notifications.notificationCenterController.VvcNotificationCenterController");
var VvcNotificationCenterController = com.ibm.team.vvc.shared.web.ui.notifications.notificationCenterController.VvcNotificationCenterController;

dojo.require("com.ibm.team.vvc.shared.web.ui.notifications.notifications.VvcNotificationFactory")
var ncFactory = com.ibm.team.vvc.shared.web.ui.notifications.notifications.VvcNotificationFactory;

dojo.require("com.ibm.team.vvc.shared.core.ui.utils.ErrorUtil")
var ErrorUtil = com.ibm.team.vvc.shared.core.ui.utils.ErrorUtil;

dojo.require("jazz.util.browserSupport");
dojo.require("jazz.app.auth");
dojo.require("jazz.ui.UnsupportedBrowserWarning");


dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.wizard", "WizardApplicationMessages");
dojo.requireLocalization("com.ibm.team.vvc.server.core.ui.internal.deliver", "DeliverWizardApplicationMessages");


var DELIVER_PATH = "/deliver";
var DELIVER_UI_PATH = "/deliverui";

dojo.declare("com.ibm.team.vvc.server.core.ui.internal.deliver.DeliverWizardApplication", [ dijit._Widget ], {
	
	messages: dojo.i18n.getLocalization("com.ibm.team.vvc.server.core.ui.internal.deliver", "DeliverWizardApplicationMessages"),
	
	step1: null,
	step2: null,
	finalStep: null,
	session: null,
	keepAliveTimer: null,
	loadingLabel: null,
	notificationCenter: null,
	
	constructor: function() {
		this.wizardMessages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.wizard", "WizardApplicationMessages");
	},
	
	postCreate: function() {
		this.getUserRoles();
	},
	
	getUserRoles: function () {
		var srh= new ServiceResponseHandler(this, "authRequestPassed", "authRequestFailed");		
		AdminClient.getUserRoles(srh);
	},
	
	authRequestPassed: function(response) {
		if (response.userId) {
			this.loggedInUser = response.userId;
			dojo.setObject("com.ibm.team.repository.web.client.internal.AUTHENTICATED_USERID", response.userId);
		}
		var roles = response.roles;
		for (var i=0; i < roles.length; i++) {
			if (roles[i] === "JazzAdmins") {
				this.authorized = true;
				break;
			}
		}
		this._checkBrowser();
	},
	
	authRequestFailed: function() {
		this._checkBrowser();
	},
	
	_checkBrowser: function() {
		//Check to see if we are using a supported Browser
		var support = jazz.util.browserSupport.getSupportLevel();
		if (support && (support == jazz.util.browserSupport.NOT_SUPPORTED)) {
			var warn = new jazz.ui.UnsupportedBrowserWarning();
			this.connect(warn, "onContinue", function() {
				warn.destroy();
				this._createUI();
			});
			warn.placeAt(document.body);
		} else {
			this._createUI()
		}
	},
	
	createLogoutArea: function() {
		var logoutNode = dojo.create("div", { className: "logout-node" });
		this.loggedInUserNode = dojo.create("span", { innerHTML: this.loggedInUser, className: "userName" }, logoutNode);
		dojo.create("span", { innerHTML: "|", style: { color: "#999", padding: "0 4px" } }, logoutNode);
		this.logoutLink = dojo.create("a", { innerHTML: this.wizardMessages.logout, href: "#", className: "logoutLink" }, logoutNode);
		
		/*
		 * TODO need to look at Referer header and insert a back to link, as
		 * well as a link to the Configuration Explorer.
		 */
		
		this.connect(this.logoutLink, "onclick", function(e) {
			dojo.stopEvent(e);
			var dfd = jazz.app.auth.logout();
			dfd.addCallback(function() {
				window.location.reload();
			});
		});
		return logoutNode;
	},
	
	_createUI: function() {
		this.session = this.getDeliverSession();
		
		var title = this.messages.wizardName_deliver;
		if (this.session.isCompareOnly()) {
			title = this.messages.wizardName_compare;
		}
		
		document.body.style.background = "none";
		document.title = title;
		this.content = dojo.create("div", { className: "com-ibm-team-repository-wizard" });
		if (this.loggedInUser) {
			this.content.appendChild(this.createLogoutArea());
		}
		dojo.addClass(this.domNode, "com-ibm-team-repository-web-setup-RootNode");
		
		dojo.create("div", { innerHTML: title.toUpperCase(), className: "setup-header", style: {zIndex: 7}}, this.content);
		var jazzIcon = dojo.create("img", { className: "jazz-icon", style: {zIndex: 7} }, this.content);
		jazzIcon.src = dojo.moduleUrl("com.ibm.team.vvc.server.core", "graphics/jazz-identR.png");
		
		this.notificationCenter = new com.ibm.team.vvc.shared.web.ui.notifications.notificationCenterController.VvcNotificationCenterController();
		var informationCenter = document.createElement("div");
		informationCenter.appendChild(this.notificationCenter.domNode);
		informationCenter.style.padding = "0 58px 20px 60px";
		
		this.wizardBox = document.createElement("div");
		this.content.appendChild(this.wizardBox);
		var self = this;
		var wizardParams = {
			fullPageContainer: this.wizardBox,
			stepListLayout: "horizontal",
			showStepSummary: true,
			processFinish:  function(){
				document.location.href = self.session.getRefererUrl();
            },
            includeCancel: !self.session.isCompareOnly(),
            headerNode: informationCenter,
		};
		if (this.authorized) { 
			wizardParams.steps = this.createSteps();
		} else {
			var notAuthStep = new NotAuthorizedView({
					applicationName: title,
					customNoUserMessage: this.wizardMessages.userNotRecognizable,
					includeBackLink: false,
					showLogout: true });
			dojo.mixin(notAuthStep, new WizardStep());
			notAuthStep.includeNavigationUI = false;
			wizardParams.stepListLayout = "horizontal";
			wizardParams.steps = [notAuthStep];
			wizardParams.showStepSummary = false;
		}
		this.wizard = new Wizard(wizardParams);
		
		// deal with cancel on a wizard level
		this.wizard.contentWidget.processCancel = function() {
			self.deleteSession(self.session);
		};
		
		var outerFooter = new com.ibm.team.repository.web.ui.internal.wizard.LogoFooter();
		this.content.appendChild(outerFooter.domNode);
		this.domNode.appendChild(this.content);
		this.wizard.startup();
		
		dojo.style(this.content, "width", "98%");
		
		this.hideLoadingMessage();
	},
	
	createSteps: function() {		
		// make sure we keep the session alive
		// by sending a keepAlive POST every minute
		this.keepAliveTimer = setInterval(dojo.hitch(this, this.keepSessionAlive), 60000);
		
		this.finalStep = new DeliverSummaryStep({session: this.session, wizardApp: this});
		this.step2 = new ChangesetPickerStep({session: this.session, wizardApp: this});
		this.step1 = new SourceTargetPickerStep({session: this.session, wizardApp: this});
		
		var steps = [ this.step1, this.step2 ];
		
		this.createExternalSteps(this.session, steps);
		
		if (!this.session.isCompareOnly()) {
			steps.push(this.finalStep);
		}
		
		return steps;
	},
	
	createExternalSteps: function(session, steps) {
		var providerList = session.getProviders();
		var sessionUri = session.getUri() + "/resources";
		sessionUri = encodeURIComponent(sessionUri);
		
		var tempSteps = [];
		
		if (providerList && providerList.length > 0) {
			for (var i = 0; i < providerList.length; i++) {
				var compareWebUri = providerList[i].getCompareWebUri();
				var providerUri = providerList[i].getUri();
				if (compareWebUri) {
					var webUri = compareWebUri + 
						(djConfig.isDebug ? "&debug=true" : "") +
						"&provider=" + encodeURIComponent(providerUri);
						
					// TODO: compare operation?
					if (this.operation) {
						webUri += "&operation=" + this.operation;
					}
					
					if (sessionUri) {
						webUri += "&interimResultUri=" + sessionUri;
					}
					
					webUri += "&delivermode=true#embed=true&delivermode=true&s=/steps/1";
					var title = providerList[i].getTitle()
					var args = {
						displayName: title,
						stepURL: webUri
					};
					var step = new com.ibm.team.vvc.server.core.ui.internal.deliver.DeliverExternalWizardStep(args);
					step.title = title;
					step.compareOnly = session.isCompareOnly();
					step.wizardApp = this;
					tempSteps.push(step);
				}
			}
		}
		
		tempSteps.sort(this.stepsComparator);
		
		for (var i = 0; i < tempSteps.length; i++) {
			steps.push(tempSteps[i]);
		}
	},
	
	stepsComparator: function(a,b) {
		return a.title.localeCompare(b.title);
	},
	
	getDeliverSession: function() {
		var location = window.location.href;

		// change the deliverui to deliver path
		location = location.replace(DELIVER_UI_PATH, DELIVER_PATH);
		
		var queryStart = location.indexOf('?');
		if (queryStart < 0) {
			queryStart = location.length;
		}
		var hashStart = location.indexOf('#');
		if (hashStart < 0) {
			hashStart = location.length;
		}
		
		var endOfSegment = Math.min(queryStart, hashStart);
		
		var resourceNameStart = location.indexOf(DELIVER_PATH) + DELIVER_PATH.length;
		
		var resourceName = location.substring(resourceNameStart, endOfSegment);
		
		if (resourceName.length <= 1) { // account for the ending /
			window.location.replace(location.substring(0, resourceNameStart));
		} else {
			var session = new DeliverSession();
			session.load(location.substring(0, endOfSegment), /* onError */ function() {
				document.location.href = net.jazz.ajax.contextRoot() + DELIVER_PATH;				
			});		
			
			return session;
		}
	},
	
	updateSession: function(session, excludeStep) {
		this.session = session;
		if (excludeStep !== this.step1) {
			if (session
					&& session.getState() !== Constants.DELIVER_SESSION_STATE_UNINITIALIZED) {
				this.step1.disableUi();
			}
		}
		
		if (excludeStep !== this.step2) {
			this.step2.session = session;
			this.step2.reload();
		}
		
		if (excludeStep !== this.finalStep) {
			this.finalStep.session = session;
			this.finalStep.reload();
		}
	},
	
	deleteSession: function(session) {
		var url = session.getUri();
		dojo.xhrDelete({
			url: url,
			sync: true
		});
		
		if (this.keepAliveTimer) {
			delete this.keepAliveTimer;
		}
		
		// go back to the referer.
		document.location.href = session.getRefererUrl();
	},
	
	keepSessionAlive: function() {
		if (this.session) {
			var url = this.session.getUri() + "/keepAlive";
			dojo.xhrPut({
				url: url
			});
		}
	},
	
	showLoadingMessage: function(messageStr) {
		if (!messageStr) {
			messageStr = this.messages.loading;
		}
		
		this.hideLoadingMessage();
		
		this.loadingLabel = dojo.create("div", { innerHTML: "<div class='loadingDiv'><table width='100%'><tr><td align='center'>" +
				"<span class='loadingLabel'><img src='" + 
			dojo.moduleUrl("com.ibm.team.vvc.server.core", "/ui/internal/wizard/images/loading_obj.gif") + 
			"' />&nbsp;&nbsp;" + messageStr +"</span></td></tr></table></div>", className: "loadingMask"}, dojo.body());
	},
	
	hideLoadingMessage: function() {
		if (this.loadingLabel) {
			dojo.destroy(this.loadingLabel);
			this.loadingLabel = null;
		}
	},
	
	showErrorMessage: function(response, ioArgs) {
		var notification = null;
		if (response) {
			if (response.status == 404) {
				if (this.session.isCompareOnly()) {
					notification = ncFactory.createError({
						title: this.messages.sessionTimeoutTitle_compare,
						description: this.messages.sessionTimeoutDescription,
						solution: this.messages.sessionTimeoutSolution_compare
					});
				} else {
					notification = ncFactory.createError({
						title: this.messages.sessionTimeoutTitle,
						description: this.messages.sessionTimeoutDescription,
						solution: this.messages.sessionTimeoutSolution
					});
				}
			} else {
				var errorDetails = ErrorUtil.getErrorDetails(response);
				if (errorDetails.errorSubStatus === "CANNOT_LOCK_NON_PARTIAL_CHANGESETS") {
					notification = ncFactory.createError({
						title: this.messages.cannotLockTitle,
						description: this.messages.cannotLockDescription,
						solution: this.messages.cannotLockSolution
					});
				} else if (errorDetails.errorSubStatus === "CONFLICTS_REMAIN_NON_PARTIAL_CHANGESETS") {
					notification = ncFactory.createError({
						title: this.messages.conflictsRemainTitle,
						description: this.messages.conflictsRemainDescription,
						solution: this.messages.conflictsRemainSolution
					});	
				}
			}
		}
		
		if (notification == null) {
			ncFactory.createXhrError({xhrError: response, ioArgs: ioArgs}).dispatch();
		} else {
			ncFactory.createError(notification).dispatch();
		}
		
		this.hideLoadingMessage();
	},
	
	performFinish: function() {
		var previousState = this.session.getState();
		this.session.setState(Constants.DELIVER_SESSION_STATE_FINISHED);
		
		if (this.session.isCompareOnly()) {
			this.showLoadingMessage(this.messages.finishing_compare);
		} else {
			this.showLoadingMessage(this.messages.finishing_deliver);
		}
		
		var self = this;
		
		this.session.doSaveAsync(
				function(response, ioArgs) {
					self.updateSession(self.session, self);
					
					// go back to the referer.
					document.location.href = self.session.getRefererUrl();
				},
				
				function(response, ioArgs) {
					/*
					 * If this is compare only mode, and session has timed out, 
					 * then redirect back to the referer because there is no cancel button.
					 * 
					 * If the error is something else, ignore it. The session will
					 * expire automatically after some time anyway.
					 */
					if (self.session.isCompareOnly()) {
						document.location.href = self.session.getRefererUrl();
					} else {
						self.session.setState(previousState);
						self.showErrorMessage(response, ioArgs);
						self.hideLoadingMessage();
					}
				}
		);
	},
	
});
dojo.declare("com.ibm.team.repository.web.ui.internal.wizard.LogoFooter", [dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/wizard/templates/LogoFooter.html"),
	
	constructor: function() {
		this.messages = messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.wizard", "WizardApplicationMessages");
	}
});
dojo.declare("com.ibm.team.vvc.server.core.ui.internal.deliver.DeliverExternalWizardStep", [ExternalWizardStep], {

	compareOnly: false,
	wizardApp: null,
	
	canFinish: function() {
		return this.compareOnly;
	},
	
	onFinish: function() {
		if (this.compareOnly && this.wizardApp) {
			this.wizardApp.performFinish();
		} else {
			this.inherited(arguments);
		}
	},
});

})();