/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2013.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function(){

dojo.provide("com.ibm.team.vvc.server.core.ui.internal.deliver.SourceTargetPickerStep");

dojo.require("jazz.ui.wizard.WizardStep");
var WizardStep = jazz.ui.wizard.WizardStep;

dojo.require("com.ibm.team.vvc.shared.web.ui.picker.WorkspacePicker");
var WorkspacePicker = com.ibm.team.vvc.shared.web.ui.picker.WorkspacePicker;

dojo.require("com.ibm.team.vvc.shared.core.ui.objects.ConfigurationResource");
var ConfigurationResource = com.ibm.team.vvc.shared.core.ui.objects.ConfigurationResource;

dojo.require("com.ibm.team.vvc.shared.core.ui.objects.Constants");
var Constants = com.ibm.team.vvc.shared.core.ui.objects.Constants;

dojo.require("com.ibm.team.vvc.shared.web.ui.util.VvcCache");
var VvcCache = com.ibm.team.vvc.shared.web.ui.util.VvcCache;

dojo.require("com.ibm.team.vvc.shared.web.ui.util.VvcUtils");
var VvcUtils = com.ibm.team.vvc.shared.web.ui.util.VvcUtils;

dojo.require("jazz.ui.StyledBox");
dojo.require('jazz.ui.Select');
dojo.require("dijit.form.Button");
dojo.require("dojo.DeferredList");

dojo.requireLocalization("com.ibm.team.vvc.server.core.ui.internal.deliver", "SourceTargetPickerStepMessages");

dojo.declare("com.ibm.team.vvc.server.core.ui.internal.deliver.SourceTargetPickerStep", [WizardStep, dijit._Widget, dijit._Templated], {
	
	messages : dojo.i18n.getLocalization("com.ibm.team.vvc.server.core.ui.internal.deliver", "SourceTargetPickerStepMessages"),	
	templateString: dojo.cache('com.ibm.team.vvc.server.core', 'ui/internal/deliver/templates/SourceTargetPickerStep.html'),
	widgetsInTemplate: true,
	
	/**
	 * The session
	 */
	session: null,
	
	/**
	 * The DeliverWizardApplication instance
	 */
	wizardApp: null,
	
	/**
	 * ConfigurationResource 
	 */
	source: null,
	
	/**
	 * ConfigurationResource
	 */
	target: null,
	
	/**
	 * Source picker
	 */
	sourcePicker: null,
	
	/**
	 * Target picker
	 */
	targetPicker: null,
	
	/**
	 * Space picker
	 */
	spacePicker: null,
	
	/**
	 * Spaces?
	 */
	spaces: null,
	
	/**
	 * URI of space. Automatically determined
	 */
	spaceUri: null,
	
	_cache: null,
	
	_spaceUriInit: false,
			
	/**
	 * Arguments:
	 * - session: DeliverSession.js object
	 */
	constructor: function(args){
		this._cache = new VvcCache({
			idProperty: "uri",
			storageKey: vvcConstants.CONFIGURATION_CACHE_KEY
		});
		
		if (args) {
			if (args.session) {
				this.session = args.session;
			}
			if (args.wizardApp) {
				this.wizardApp = args.wizardApp;
			}
		}
		
		this.stepTitle = this.messages.step1Title;
	},
	
	resetSpace: function() {
		if (this.spacePicker) {
			this.spaceUri = this.spacePicker.value;
		}
		
		this.setSource(null);
		this.setTarget(null);
		
		this.makePickers();
		
		this.validateNext();
	},
	
	postCreate: function(){
		this.spacePicker = this._makeSpaceSelect(this.spacePickerNode);
		dojo.connect(this.swapButton, "onClick", this, this.swap);
		dojo.connect(this.resetButton, "onClick", this, this.reset);

		this.resetSpace();
		
		var sourceUri = this.session.getSourceUri();
		if (sourceUri) {
			dojo.when(this._cache.get(sourceUri), dojo.hitch(this, function(config){
				// make sure that space is set
				if (!this._spaceUriInit && config.spaceUri) {
					this.setSpace(config.spaceUri);
					this._spaceUriInit = true;
				}
				
				this.setSource(config);
			}));
		}
		
		var targetUri = this.session.getTargetUri();
		if (targetUri) {
			dojo.when(this._cache.get(targetUri), dojo.hitch(this, function(config){
				// make sure that space is set
				if (!this._spaceUriInit && config.spaceUri) {
					this.setSpace(config.spaceUri);
					this._spaceUriInit = true;
				}
				this.setTarget(config);
			}));
		}
		
		if (this.session.isCompareOnly()) {
			this.description.innerHTML = this.messages.desc_compare;
		}
	},
	
	setSpace: function(spaceUri) {
		for (var key in this.spacePicker.children) {
			if (spaceUri === this.spacePicker.children[key].value) {
				this.spaceUri = spaceUri;
				this.spacePicker.selectedIndex = key;
				break;
			}
		}
		
		this.resetSpace();
	},
	
	destroyPickers: function() {
		if (this.targetPicker) {
			this.targetPicker.destroy();
			
			while (this.targetPickerNode.hasChildNodes()) {
				this.targetPickerNode.removeChild(this.targetPickerNode.lastChild);
			}
		}
		if (this.sourcePicker) {
			this.sourcePicker.destroy();
			
			while (this.sourcePickerNode.hasChildNodes()) {
				this.sourcePickerNode.removeChild(this.sourcePickerNode.lastChild);
			}
		}
	},
	
	makePickers: function() {
		this.destroyPickers();
		
		var sourceCollapsed = (this.source || this.session.getSourceUri()) ? true : false;
		var targetCollapsed = (this.target || this.session.getTargetUri()) ? true : false;
		
		if (this.session.isCompareOnly()) {
			this.targetPicker = this._makePicker({spaceUri: this.spaceUri}, 
				this.messages.targetPickerTitle, 
				this.messages.targetPickerSecondaryTitle, 
				this.targetPickerNode, 
				this.setTarget, targetCollapsed);
		} else {
			this.targetPicker = this._makePicker({spaceUri: this.spaceUri, 
				validTypes:["workspace"]}, 
				this.messages.targetPickerTitle, 
				this.messages.targetPickerSecondaryTitle, 
				this.targetPickerNode, 
				this.setTarget, targetCollapsed);
		}
		
		this.sourcePicker = this._makePicker({spaceUri: this.spaceUri}, 
			this.messages.sourcePickerTitle, 
			this.messages.sourcePickerSecondaryTitle, 
			this.sourcePickerNode, this.setSource, sourceCollapsed);
	},
	
	swap: function() {
		var temp = this.source;
		this.setSource(this.target);
		this.setTarget(temp);
	},
	
	disableUi: function() {
		if (this.sourcePicker) {
			this.sourcePicker.disabled = true;
		}
		
		if (this.targetPicker) {
			this.targetPicker.disabled = true;
		}
		
		if (this.swapButton) {
			this.swapButton.set("disabled", true);
		}
		
		if (this.spacePicker) {
			this.spacePicker.disabled = true;
		}
		
		this.destroyPickers();
	},
	
	_makeSpaceSelect: function(node) {
		var combo = dojo.create("select", {"class" : " combobox"});
		node.appendChild(combo);
		dojo.connect(combo, 'change', this, function() {
			this.resetSpace();		
		});
		
		var dpls = [];
		dpls.push(this.getAllSpaces());
		var dl = new dojo.DeferredList(dpls);
		dl.then(dojo.hitch(this, function(result) {
			this.spaces = result[0][1];
			this.spaces.sort(this.compareSpaces);
			for (var space in this.spaces) {
				var _space = this.spaces[space];
				
				var option = dojo.create("option", {
					value: _space.uri
				});
				option.appendChild(dojo.doc.createTextNode(_space.title));
				combo.appendChild(option);
			}
		}));
		
		return combo;
	},

	compareSpaces: function(a, b) {
		return a.title.localeCompare(b.title);
	},

	getAllSpaces: function() {
		var args = {
				url: VvcUtils.getVvcUri() + "/spaces/allspaces",
				headers: {
					"Accept" : "application/json"
				},
				handleAs: "json",
				sync: true,
				load: function(response, ioArgs) {
					var spaces = [];
					for (var uri in response) {
						var r = {uri: uri, title: response[uri]};
						spaces.push(r);
					}
					return spaces;
				}
		};
		return jazz.client.xhrGet(args);
	},
	
	_makePicker: function(args, title, secondaryTitle, node, onOkCallback, collapsed){
		var pickerArgs = {};
		   
		if(args){
			var pickerArgs = args;
		}
		pickerArgs.closeable = false;
		pickerArgs.singleClickMode = true;
		var picker = new WorkspacePicker(pickerArgs);
		picker.spaceUri = this.spaceUri;
		picker._spaceUri = this.spaceUri;
		
		this._makeAddPicker(title, secondaryTitle, picker, node, collapsed);		
		
		if(onOkCallback){
			dojo.connect(picker, "onOk", dojo.hitch(this, onOkCallback));
		}
		
		return picker;
	},
	
	_makeAddPicker: function(title, secondTitle, picker, node, collapsed){
		var boxArgs = {
				closable: false,
				collapsible: true,
				collapsed: collapsed,
				primaryTitle: title,
				secondaryTitle: secondTitle,
				trim: jazz.ui.StyledBox.TrimStyles.BLUE,
				contentNode: picker.domNode
		};
		var box = new jazz.ui.StyledBox(boxArgs);
		node.appendChild(box.domNode);
		var handle = dojo.connect(this, "startup", dojo.hitch(this, function(){
			box.startup();
			dojo.disconnect(handle);
		}));
	},
	
	setSource: function(/*ConfigurationResource*/ config){
		this.source = config;
		
		if (config) {			
			this.sourceSelectionNode.innerHTML = config.title;
		} else {
			this.sourceSelectionNode.innerHTML = this.messages.none;
		}
		
		this.validateNext();
	},
	
	setTarget: function(/*ConfigurationResource*/ config){
		this.target = config;
		
		if (config) {			
			this.targetSelectionNode.innerHTML = config.title;
		} else {
			this.targetSelectionNode.innerHTML = this.messages.none;
		}
		
		this.validateNext();
	},
	
	canFinish: function() {
		if (this.session) {
			return this.session.isCompareOnly();
		} else {
			return false;
		}
	},
	
	onFinish: function() {
		this.wizardApp.performFinish();
	},
	
	validateNext: function() {
		var validate = this.validate();
		if (validate.length > 0) {
			this._showValidationWarning(validate[0]);
		} else {
			this._showValidationWarning("");
		}
		
		this.triggerButtonStateUpdate();
		
		// also make sure we update the swap button

		if (this.source == null
				|| this.target == null
				|| this.source.uri === this.target.uri
				|| (this.source != null && !this.source
						.mutable)) {
			this.swapButton.set("disabled", true);
		} else {
			this.swapButton.set("disabled", false);
		}
		
		// finally, disable all of UI if session is no longer in uninitialized
		// mode or is not available entirely
		if (!this.session
				|| this.session.getState() !== Constants.DELIVER_SESSION_STATE_UNINITIALIZED) {
			this.disableUi();
		}
	},
	
	validate: function() {
		if (this.spaceUri == null) {
			return [this.messages.noSpaceSelected];
		} else if (this.source == null || this.target == null) {
			return [this.messages.noSourceTargetSelected];
		} else if (this.source.uri === this.target.uri) {
			return [this.messages.cannotCompareToSelf];
		} else {
			return [];
		}
	},
	
	_showValidationWarning: function(message) {
		this.validationWarningLabel.innerHTML = message;
	},
	
	onNext: function() {
		// need to update the deliver session if need be
		if (this.session
				&& this.session.getState() === Constants.DELIVER_SESSION_STATE_UNINITIALIZED) {
			this.wizardApp.showLoadingMessage();
			
			this.session.setSourceUri(this.source.uri);
			this.session.setTargetUri(this.target.uri);
			
			var self = this;
			this.session.doSaveAsync(
					function(response, ioArgs) {
						// reload the session since changeset calculation may have changed
						self.session.load(self.session.getUri());
						self.wizardApp.updateSession(self.session, self);
						self.wizardApp.hideLoadingMessage();
						self.wizard.processNext();
					},
					
					function(response, ioArgs) {
						self.wizardApp.showErrorMessage(response, ioArgs);
						self.wizardApp.hideLoadingMessage();
					}
			);
		} else {
			this.inherited(arguments);			
		}
	},
});

})();