/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2013.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function(){
   
dojo.provide("com.ibm.team.vvc.server.core.ui.internal.grid.VvcTreeGrid");

dojo.require('dojo.i18n');
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.team.vvc.shared.web.ui.VvcConstants");
var vvcConstants = com.ibm.team.vvc.shared.web.ui.VvcConstants;

dojo.require("com.ibm.team.vvc.server.core.ui.internal.grid.VvcTreeGridTable");

dojo.requireLocalization("com.ibm.team.vvc.server.core.ui.internal.grid", "VvcTreeGridMessages");

dojo.require("com.ibm.team.vvc.shared.web.ui.notifications.notifications.VvcNotificationFactory");
var ncFactory = com.ibm.team.vvc.shared.web.ui.notifications.notifications.VvcNotificationFactory;

// Add indexOf support for IE8
if(dojo.isIE <=8 && !Array.prototype.indexOf){
	// prototype the method
	var indexOf = function(object, start){
		var s = start ? start : 0;
		for(var index = s; index < this.length; index++){
			if(this[index] === object){
				return index;
			}
		}
		// Not found
		return -1;
	}
	
	/*
	 * Add it to the prototype.  From this point on, the array will have the
	 * `indexOf()` method.
	 */ 
	Array.prototype.indexOf = indexOf;
}

dojo.declare("com.ibm.team.vvc.server.core.ui.internal.grid.VvcTreeGrid", [dijit._Widget, dijit._Templated], {
	  
	templatePath: dojo.moduleUrl('com.ibm.team.vvc.server.core', 'ui/internal/grid/templates/VvcTreeGrid.html'),
	rootMessages: dojo.i18n.getLocalization("com.ibm.team.vvc.server.core.ui.internal.grid", "VvcTreeGridMessages"),
	_gridTable: null,
	_configurationCache: null,
	_idRowMap: null,
	_expandedIds: null,
	_selectedRows: null,
	_pageTitle: null,
	
	constructor: function(args){
		this._showArchived = false;
		if(!args || args && !args.notificationCenter){
			// Only create a notification center if it's needed.
			var handle = dojo.connect(this, "postCreate", this, function(){
				this._notificationCenter = new com.ibm.team.vvc.shared.web.ui.notifications.notificationCenterController.VvcNotificationCenterController();
				this.notificationCenterNode.appendChild(this._notificationCenter.domNode);
				dojo.disconnect(handle);
			})
		}
		
		// Try to load from cache
		var preserveIds = sessionStorage.getItem(vvcConstants.PRESERVE_EXPANSION_STATE_STORAGE_KEY);
		if (preserveIds) {
			this._expandedIds = dojo.fromJson(preserveIds);
		}
//		this._selectedRows = dojo.fromJson(sessionStorage.getItem(vvcConstants.PRESERVE_SELECTION_STATE_STORAGE_KEY));
		
		// Make sure they're initialized to something
		if(!this._expandedIds){
			this._expandedIds = {};
		}
		if(!this._selectedRows){
			this._selectedRows = {
					data: {},
					order: []
			};
		}
	},
	
	postCreate: function(){	
		var titleNode = this.buildTitleNode();
		var actionsNode = this.buildActionsNode();
		var treeGridTable = this.buildTreeGridTableNode();
		
		this.vvcTreeGridTitleNode.appendChild(titleNode);
		this.vvcTreeGridActions.appendChild(actionsNode);
		this.vvcTreeGridTableContentNode.appendChild(treeGridTable);
		
		dojo.connect(this._gridTable, "onCreateRow", dojo.hitch(this, this.onAddRow));
		dojo.connect(this._gridTable, "onRefresh", dojo.hitch(this, this.refresh));
	},
	
	/**
	 * Creates the data store to be used by the tree.
	 * 
	 * Subclasses can override this safely, but the returned object must be compatible
	 * with the <code>dojo.Store</code> API.
	 */
	_initializeDatastore: function(args){
		// Default to a temporary store.
		this._configurationCache = new dojo.store.Memory();
	},
	
	/**
	 * Builds the title bar node.
	 * 
	 * @return The title bar
	 */
	buildTitleNode: function(){
		/* Stub */
	},
	
	/**
	 * Builds the actions bar for the tree grid.
	 * 
	 * @return the actions bar
	 */
	buildActionsNode: function(){
		/* Stub */
	},
	
	/**
	 * Builds the tree grid table node
	 * 
	 * @return The tree grid table
	 */
	buildTreeGridTableNode: function(){
		/* Stub */
	},
	
	/**
	 * Refreshes the entire tree grid.
	 * 
	 * @see #onRefreshRow(row)
	 */
	refresh: function(){
		/* Stub */
	},
	
	/**
	 * Refreshes an individual row.  Removes all cached data about this row.
	 * then expands it.
	 * 
	 * @param id The ID of the row to refresh
	 * @see #onRefreshRow(row)
	 */
	onRefreshRowById: function(id){
		/* Stub */
	},
	
	/**
	 * Refreshes an individual row.  Removes all cached data about this row, collapses,
	 * then expands it.
	 * 
	 * @param row The row to refresh
	 * @see #onRefreshRowById(row)
	 */
	onRefreshRow: function(row){
		this.onRefreshRowById(row._data.uri);
	},
	
	/**
	 * Collapses all the expanded rows.
	 */
	collapseAll: function(){
		for(var id in this._expandedIds){
			var isExpanded = this._expandedIds[id];
			if(isExpanded){
				var rows = this._idRowMap[id];
				var length = rows.length;
				for(var index = 0; index < length; index++){
					rows[index].collapse();
				}
			}
		}
	},
	
	/**
	 * Event handler for when a row is added to the tree grid table.  Allows the
	 * controller to hook up things it's interested in.  By default, there are a 
	 * number of events which are hooked up:
	 * <ul>
	 * 	<li>onRemove</li>
	 *  <li>onExpand</li>
	 *  <li>onCollapse</li>
	 *  <li>onSelect</li>
	 *  <li>onDeselect</li>
	 *  <li>onRefresh</li>
	 * </ul>
	 * 
	 * @param row The newly created row.
	 * @param data The data used to create the new row.
	 * @param childOf The newly created row is a child of this row.  Can be <code>null</code>
	 */
	onAddRow: function(row, data, childOf){
		var removeHandler = dojo.connect(row, "onRemove", dojo.hitch(this, this.onRemoveRow, row, data, childOf));
		var expandHandler = dojo.connect(row, "onExpand", dojo.hitch(this, this.onExpandRow, row, data, childOf));
		var collapseHandler = dojo.connect(row, "onCollapse", dojo.hitch(this, this.onCollapseRow, row, data, childOf));
		var selectHandler = dojo.connect(row, "onSelect", dojo.hitch(this, this.onSelectRow, row, data, childOf));
		var deselectHandler = dojo.connect(row, "onDeselect", dojo.hitch(this, this.onDeselectRow, row, data, childOf));
		var refreshHandler = dojo.connect(row, "onRefresh", dojo.hitch(this, this.onRefreshRow, row, data, childOf));
		var onFocusHandler = dojo.connect(row, "onFocus", dojo.hitch(this, this.onRowFocus, row, data, childOf));
		var onBlurHandler = dojo.connect(row, "onBlur", dojo.hitch(this, this.onRowBlur, row, data, childOf));
	
		
		// Special case: need to listen for the parent being collapsed
		if(childOf){
			var childOfHandler = dojo.connect(childOf, "onCollapse", dojo.hitch(this, function(){
				row.onCollapse();
				dojo.disconnect(childOfHandler);
				row._removeRow();
			}));
		}
		
		if(!this._idRowMap[data.uri]){
			this._idRowMap[data.uri] = [];
		}
		this._idRowMap[data.uri].push(row);
		
		if(this._expandedIds[data.uri]){
			row.expand();
		}
		
//		if(this._selectedRows.byId[data.uri]){
//			row.onSelect();
//		}
	},
	
	/**
	 * Event handler for when a row is removed. 
	 */
	onRemoveRow: function(row, data, childOf){
		var rows = this._idRowMap[data.uri];
		if(rows){
			dojo.forEach(rows, dojo.hitch(this, function(currRow, index, items){
				if(currRow == row){
					rows.splice(rows.indexOf(row), 1);
				}
			}));
		}
	},
	
	/**
	 * Event handler for when a row is expanded.
	 */
	onExpandRow: function(row, data, childOf){
		this._expandedIds[data.uri] = true;
		sessionStorage.setItem(vvcConstants.PRESERVE_EXPANSION_STATE_STORAGE_KEY, dojo.toJson(this._expandedIds));
	},
	
	/**
	 * Event handler for when a row is collapsed
	 */
	onCollapseRow: function(row, data, childOf){
		this._expandedIds[data.uri] = false;
		sessionStorage.setItem(vvcConstants.PRESERVE_EXPANSION_STATE_STORAGE_KEY, dojo.toJson(this._expandedIds));
	},
	
	/**
	 * Event handler for when a row is selected. Triggers an onRowSelectionChanged event.
	 */
	onSelectRow: function(row, data, childOf){
		dojo.setAttr(row.domNode, "selected", null);
		this._selectedRows.order.push(data.uri);
		this._selectedRows.data[data.uri] = data;
		this.onRowSelectionChanged(row, data, childOf);
//		this._selectedRows.byId[data.uri] = true;
//		sessionStorage.setItem(vvcConstants.PRESERVE_SELECTION_STATE_STORAGE_KEY, dojo.toJson(this._selectedRows));
	},
	
	/**
	 * Event handler for when a row is deselected.  Triggers an onRowSelectionChanged event.
	 */
	onDeselectRow: function(row, data, childOf){
		dojo.removeAttr(row.domNode, "selected");
		var order = this._selectedRows.order;
		var removed = order.splice(dojo.indexOf(order, data.uri), 1);
		delete this._selectedRows.data[removed.uri];
		this.onRowSelectionChanged(row, removed, childOf);
//		this._selectedRows.byId[data.uri] = false;
//		sessionStorage.setItem(vvcConstants.PRESERVE_SELECTION_STATE_STORAGE_KEY, dojo.toJson(this._selectedRows));
	},
	
	/**
	 * Event fires when the row selection has changed
	 */
	onRowSelectionChanged: function(row, data, childOf){
		/* Stub */
	},
	
	/**
	 * Event handler for when a component in a row gains focus
	 */
	onRowFocus: function(row, data, childOf){
		dojo.setAttr(row.domNode, "selected", null);
	},
	
	/**
	 * Event handler for when a component in a row loses focus
	 */
	onRowBlur: function(row, data, childOf){
//		setTimeout(dojo.removeAttr(row.domNode, "selected"), 100);
		dojo.removeAttr(row.domNode, "selected")
	}
	
});
})();
