/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2013.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function(){

	dojo.provide("com.ibm.team.vvc.server.core.ui.internal.page.CompareViewer");

	dojo.require("dijit._Widget");
	dojo.require("dijit._Templated");
	dojo.require("dijit.layout.ContentPane");
	dojo.require("dijit.layout.BorderContainer");
	dojo.require("dojo.parser");
	dojo.require("jazz.ui.Toolbar");
	dojo.require("jazz.ui.toolbar.Button");
	dojo.require("jazz.ui.MessageBox");
	dojo.require("jazz.ui.Breadcrumbs");
	dojo.require("jazz.ui.Menu");
	dojo.require("dijit.MenuItem");
	dojo.require("dijit.form.Select");
	dojo.require("com.ibm.team.vvc.shared.web.ui.picker.WorkspacePicker");
	dojo.require("jazz.ui.StyledBox");
	dojo.require("dijit.form.Button");

	dojo.require("com.ibm.team.vvc.shared.web.ui.context.ContextUtils");
	var ContextUtils = com.ibm.team.vvc.shared.web.ui.context.ContextUtils;

	dojo.require("com.ibm.team.vvc.shared.web.ui.notifications.notificationCenterController.VvcNotificationCenterController");
	dojo.require("com.ibm.team.vvc.shared.web.ui.notifications.notifications.VvcNotificationFactory");
	var ncFactory = com.ibm.team.vvc.shared.web.ui.notifications.notifications.VvcNotificationFactory;
	
	dojo.require("com.ibm.team.vvc.shared.core.ui.objects.ConfigurationResource");
	
	dojo.require("jazz.ui.wizard.WizardStep");
	var WizardStep = jazz.ui.wizard.WizardStep;

	dojo.require('jazz.util.html');
	
	dojo.require("net.jazz.ajax.ui.PlatformUI");
	var PlatformUI = net.jazz.ajax.ui.PlatformUI;
	
	dojo.require("jazz.ui.wizard.ExternalWizardStep");
	var ExternalWizardStep = jazz.ui.wizard.ExternalWizardStep;

	dojo.require("com.ibm.team.vvc.shared.web.ui.util.UIUtils");
	var UIUtils = com.ibm.team.vvc.shared.web.ui.util.UIUtils;
	
	dojo.require("jazz.app.ua.PageHoverHelp");
	
	dojo.require("com.ibm.team.vvc.server.core.ui.internal.util.PageUtil");
	var PageUtil = com.ibm.team.vvc.server.core.ui.internal.util.PageUtil;
	
	dojo.require("com.ibm.team.vvc.shared.web.ui.util.VvcUtils");
	var VvcUtils = com.ibm.team.vvc.shared.web.ui.util.VvcUtils;
	
	dojo.require("com.ibm.team.vvc.server.core.ui.internal.page.ApplicationWizard");
	var ApplicationWizard = com.ibm.team.vvc.server.core.ui.internal.page.ApplicationWizard;
	
	dojo.require("com.ibm.team.vvc.server.core.ui.internal.page.CompareExternalWizardStep");
	var CompareExternalWizardStep = com.ibm.team.vvc.server.core.ui.internal.page.CompareExternalWizardStep;
	
	dojo.require("com.ibm.team.vvc.shared.web.ui.VvcConstants");
	var vvcConstants = com.ibm.team.vvc.shared.web.ui.VvcConstants;
	
	dojo.requireLocalization("com.ibm.team.vvc.server.core.ui.internal.page", "CompareViewerMessages");
	
	/*
	 * The CompareViewer page shows a wizard for comparing and merging resources
	 * between configurations. Each step in the wizard comes from a
	 * CompareProvider (see CompareProvider.java).
	 */
	dojo.declare("com.ibm.team.vvc.server.core.ui.internal.page.CompareViewer", [dijit._Widget, dijit._Templated], {
		widgetsInTemplate: true,
		templatePath: dojo.moduleUrl('com.ibm.team.vvc.server.core', 'ui/internal/page/templates/CompareViewer.html'),
		messages : dojo.i18n.getLocalization("com.ibm.team.vvc.server.core.ui.internal.page", "CompareViewerMessages"),
		ancestorUri: null,
		descendantUri: null,
		otherDescendantUri: null,
		mergeUri: null,
		mergeOptions: null,
		hideConfigurationPicker: null,
		diffCountInfo: null,
		operation: null,
		isMergeFlag: false,
		isThreeWayFlag: false,
		compareResult: null,
		_wizard: null,
		isLoaded: false,
		title: null,
		notificationCenter: null,
		keepAliveTimer: null,
		
		constructor: function(workspace) {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.vvc.server.core.ui.internal.page", "CompareViewerMessages");
			this.notificationCenter = new com.ibm.team.vvc.shared.web.ui.notifications.notificationCenterController.VvcNotificationCenterController();
		},
		
		postCreate: function(){
			var ActionRegistry = PlatformUI.getWorkbench().getActionRegistry();
			ActionRegistry.registerAction(vvcConstants.EXPLORE_COMPARISON, this, "showPage");
			/* --- Deprecated VVC 4.0.2 (WI35709) --- */
			ActionRegistry.registerAction(vvcConstants.COMPARE_VIEWER_PAGE, this, "showPage");
						
			this.notificationCenterNode.appendChild(this.notificationCenter.domNode);
			
			// The user can, at any time, click on the Configuration Explorer at the top, which will invoke an action 
			// and simply just change the hash. This means all our widgets and compare/merge data will still be in memory. 
			// This can lead to refresh problems when the user selects a Compare/Merge action again.
			// Hence, we detect the hash change and see if we are navigating away from the CompareViewer.
			// If we are, we destroy everything.
			dojo.subscribe("/dojo/hashchange", this, "checkNavigationStatus");
		},
		
		checkNavigationStatus: function() {
			var hash = (window.location.href.split("#")[1] || "");
			if (hash) {
				// check if we navigated to somewhere other than the CompareViewer page.
				if (hash.indexOf(vvcConstants.EXPLORE_COMPARISON) === -1 
						&& hash.indexOf(vvcConstants.COMPARE_VIEWER_PAGE) === -1) {
					// mark as not loaded so that, even tho all dojo widgets still exist, 
					// we will recreate everything next time the user requests for a compare/merge operation.
					this.isLoaded = false;
					
					// also, while we are at it, destroy everything.
					if (this._wizard) {
						this._onFinish({doNotNavigate : true});
					}
				}
			} else {
				this.isLoaded = false;
			}
		},
		
		showPage: function(){			
			if (!this.isLoaded) {

				this.isLoaded = true;

				var hash = (window.location.href.split("#")[1] || "");
				var newWizard = true;
				if (hash) {
					var index = hash.indexOf("s=/steps/");
					if (index != -1) {
						newWizard = false;
					}
				}
				
				if (newWizard) {
					if (hash != -1) {
						window.location.href = window.location.href + "&s=/steps/1";
					} else {
						window.location.href = window.location.href 
							+ "#action=" + vvcConstants.EXPLORE_COMPARISON + "&s=/steps/1";
					}
				}
				
				this.ancestorUri = this.getRequestParmValue("ancestorUri");
				this.descendantUri = this.getRequestParmValue("descendantUri");
				this.otherDescendantUri = this.getRequestParmValue("otherDescendantUri");
				this.hideConfigurationPicker = this.getRequestParmValue("hideConfigurationPicker");
				this.mergeUri = this.getRequestParmValue("mergeUri");
				this.mergeOptions = this.getRequestParmValue("mergeOptions");
				if (!this.mergeOptions) {
					this.mergeOptions = "lockDeltaResources";
				}
				this.operation = this.getRequestParmValue("operation");
				
				this.title = this.messages.title;
				if (this.operation === "rebase") {
					this.title = this.messages.rebaseTitle;
				} else if (this.operation === "accept") {
					this.title = this.messages.acceptTitle;
				} else if (this.operation === "deliver") {
					this.title = this.messages.deliverTitle;
				} else if (this.operation === "compareWithParentSnapshot") {
					this.title = this.messages.compareWithParentSnapshotTitle;
				} else if (this.operation === "compareWithParentWorkspace") {
					this.title = this.messages.compareWithParentWorkspaceTitle;
				}
				
				// set the title
				var element = document.getElementById("compareViewerTitle");
				if (element) {
					element.innerHTML = this.title;
				}
				
				// use last merge result as base for data flow between parent and child workspace
				if ("accept" === this.operation || "deliver" === this.operation
					|| "rebase" === this.operation || "compareWithParentSnapshot" === this.operation) {
					this.mergeOptions += ",useLastMergeAsBase";
				}
				
				// ask compare merge to fetch the display name for each resource
				this.mergeOptions += ",includeDisplayName";
				steps = [];
				if(this.hideConfigurationPicker){
					this.doCompare(this.ancestorUri,this.descendantUri, this.otherDescendantUri, this.mergeUri, this.mergeOptions);
				} else {
					var args = {
							ancestorUri: this.ancestorUri,
							descendantUri: this.descendantUri, 
							otherDescendantUri: this.otherDescendantUri,
							mergeUri: this.mergeUri,
							operation: this.operation,
							startMergeProcess: dojo.hitch(this, function(){
								this.doCompare(step1.ancestor.uri, step1.descendant.uri, step1.otherDescendant.uri, step1.merge.uri, this.mergeOptions);
							})
					};
					var step1 = new com.ibm.team.vvc.server.core.ui.internal.page.CompareViewer._stepOne(args);
					steps.unshift(step1);
					this._buildWizard(steps);
				}
				this._initBreadcrumbs();
			}
		},
		
		getRequestParmValue: function(paramName) {
			var value = PageUtil.getRequestParmValue(paramName);
			if (value != null) {
				value = unescape(value);
			}
			return value;
		},
		
		showMessage: function(args) {
			UIUtils.hidePageMessage();
			if (args && this._messageBoxContainer) {
				args["closable"] = true;
				if (!this.messageBox) {
					this.messageBox = new jazz.ui.MessageBox();
					this._messageBoxContainer.appendChild(this.messageBox.domNode);
				}
				this.messageBox.showMessage(args);
				this._messageBoxContainer.style.display = "";
			}
		},
		
		hideMessage: function() {
			if (this._messageBoxContainer) {
				this._messageBoxContainer.style.display = "none";
			}
		},
		
		doCompare: function(ancestorUri, descendantUri, otherDescendantUri, mergeUri, mergeOptions) {
			this.showMessage({
				type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
				summary: this.messages.compareInProgress
			});
			
			var query = {};
			if(ancestorUri){
				query.ancestorUri = ancestorUri;
			} else if(otherDescendantUri){
				query.ancestorUri = otherDescendantUri;
				otherDescendantUri = null;
			}
			
			if(descendantUri){
				query.descendantUri = descendantUri;
			}
			
			if(otherDescendantUri){
				query.otherDescendantUri = otherDescendantUri;
			}
			
			mergeOptions.isMergeFlag = false;
			if(mergeUri){
				query.mergeUri = mergeUri;
				mergeOptions.isMergeFlag = true;
			}
			
			
			if (ancestorUri && descendantUri && otherDescendantUri && mergeUri) {
				mergeOptions.isThreeWayFlag = true;
			}
			
			if(mergeOptions){
				query.mergeOptions = mergeOptions;
			}
			
			var clientUri = PageUtil.getRequestParmValue(PageUtil.CLIENT_INTERFACE);
			if (clientUri != null) {
				link += "&" + PageUtil.CLIENT_INTERFACE + "=" + clientUri;
			}
			
			var link = VvcUtils.getVvcUri() + "/compare?" + dojo.objectToQuery(query);
			var args = {
				url: link,
				handleAs: "json",
				headers: {
					Accept: "application/json;application/rdf+json;text/rdf+json;text/json"
				},
				load: function(response, ioArgs) {
					return response;
				},
				error: dojo.hitch(this, function(error, args) {
					ncFactory.createXhrError({xhrError: error, ioArgs: args}).dispatch();
				})
			};
			var deferred = jazz.client.xhrGet(args);
			var self = this;
			deferred.addCallback(this, function (compareResult) {
				self.compareResult = compareResult;
				self._buildSteps(compareResult);
				self.hideMessage();
				
				// keep the session alive by pinging the server every minute.
				self.keepAliveTimer = setInterval(dojo.hitch(self, self.keepSessionAlive), 60000);
			});
		},
		
		keepSessionAlive: function() {
			if (this.compareResult) {
				var url = this.compareResult.compareInterimResultUri + "/keepAlive";
				dojo.xhrPut({
					url: url
				});
			}
		},
		
		_buildSteps: function(compareResult) {
			var steps = [];
			var providerList = compareResult.CompareProviderList;
			var interimResult = compareResult.compareInterimResultUri;
			
			if (providerList && providerList.length > 0) {
				for (var i = 0; i < providerList.length; i++) {
					if (providerList[i].compareWebUri) {
						var webUri = providerList[i].compareWebUri + 
							(djConfig.isDebug ? "&debug=true" : "") +
							"&provider=" + encodeURIComponent(providerList[i].compareProviderUri);
							
						if (this.operation) {
							webUri += "&operation=" + this.operation;
						}
						if (interimResult) {
							webUri += "&interimResultUri=" + encodeURIComponent(interimResult);
						}
						webUri += "#embed=true&action=dummy&s=/steps/1";
						var args = {
							displayName: providerList[i].title,
							stepURL: webUri
						};
						var step = new CompareExternalWizardStep(args);
						steps.push(step);
						if(!this._wizard){
							this._buildWizard(steps);
						} else {
							this._wizard.addStep(step);
						}
						step.triggerButtonStateUpdate();
					}
				}
			}

			if (steps.length === 0) {
				if (confirm(this.messages.noCompareAvailable)) {
					location.reload();
				} else {
					this._onFinish();
				}
			}
		},
		
		_buildWizard: function(steps) {
			if(this.wizard){
				return;
			}
			var wizardParams = {
				primaryTitle: this.title,
				fullPageContainer: this._mainDiv,
				stepListLayout: "horizontal",
				showStepSummary: true,
				includeCancel: false,
				width: "100%",
				customProcessFinish : dojo.hitch(this, this._onFinish)
			};
			
			wizardParams.steps = steps;
			
			this._wizard = new ApplicationWizard(wizardParams);
			this._wizard.setAction("action=" + vvcConstants.EXPLORE_COMPARISON);
			this._wizard.startup();
			
			// Have to override a few styles to make the container that contains
			// steps layout properly in full-width.
			// Cannot do this in CSS because our stylesheet can be loaded before
			// jazz's stylesheet and hence override our styles before we can override theirs.
			var query = dojo.query(".wizardPanelArea");
			query.style("float", "left");
			query.style("width", "100%");
			query.style("-moz-box-sizing", "border-box");
			query.style("-webkit-box-sizing", "border-box");
			query.style("box-sizing", "border-box");
			
		},

		_onFinish : function(args) {
			this._wizard.close();
			this._destroyWizard(args);
		},

		/**
		 * Clean up the resources and widgets used by this wizard.
		 */
		_destroyWizard : function(args) {			
			if (this.keepAliveTimer) {
				delete this.keepAliveTimer;
			}
			
			this._wizard.destroyRecursive();
			delete this._wizard;
			
			this.isLoaded = false;
			
			if (!args || !args.doNotNavigate) {
				var redirectUri = this.getRequestParmValue("redirectUri");
				if (redirectUri) {
					redirectUri = unescape(redirectUri);
					window.location.href = redirectUri;
				} else {
					PageUtil.navigateToMainPage();
				}
			}			
		},
		
		_initBreadcrumbs: function()
		{
			if (!this._breadcrumbs)
			{
				var explorerPageUri = PageUtil.createLink(vvcConstants.EXPLORE_CONFIGURATIONS);
				var parts = [];
				var segment = PageUtil.getClientAppBreadcrumbSegment();
				if (segment != null) {
					parts.push(segment);
				}
				
				var segment = {
						label: this.messages.configurations, 
						href : explorerPageUri
					};
				parts.push(segment);
				
				var operation = PageUtil.getRequestParmValue("operation");
				var breadcrumb = PageUtil.getRequestParmValue("breadcrumb");
				var redirectUri = PageUtil.getRequestParmValue("redirectUri");
				
				if (redirectUri) {
					redirectUri = decodeURIComponent(unescape(redirectUri));
				}
				
				if (breadcrumb) {
					breadcrumb = decodeURIComponent(unescape(breadcrumb));
				} else {
					breadcrumb = this.messages.workspace;
				}
				
				if (redirectUri && breadcrumb) {
					segment = {
							label: breadcrumb, 
							href : redirectUri
						};
					parts.push(segment);
				}
				
				this._breadcrumbs = new jazz.ui.Breadcrumbs({segments: parts});
				this._breadcrumbs.placeAt(dojo.byId("breadcrumbs"));
			}
		},
		
	}), // end of CompareViewer

	
	
	
dojo.declare("com.ibm.team.vvc.server.core.ui.internal.page.CompareViewer._stepOne", [jazz.ui.wizard.WizardStep, dijit._Widget, dijit._Templated], {
		
		messages : dojo.i18n.getLocalization("com.ibm.team.vvc.server.core.ui.internal.page", "CompareViewerMessages"),	
		templatePath: dojo.moduleUrl('com.ibm.team.vvc.server.core', 'ui/internal/page/templates/CompareViewer_step1.html'),
		widgetsInTemplate: true,
		
		startMergeProcess: null,
		
		descendants: null,
		otherDescendants: null,
		
		otherDescendant: null,
		descendant: null,
		ancestor: null,
		merge: null,
		
		operation: null,
				
		constructor: function(args){
			if(args){			
				if(/*function()*/args.startMergeProcess){
					this.startMergeProcess = args.startMergeProcess;
				}
				
				if(/*String*/ args.descendantUri){
					if(args.descendantUri === "previous"){
						this.descendant = new dojo.Deferred();
						var prevHandle = dojo.connect(this, "setOtherDescendant", dojo.hitch(this, function(config){
							this.descendant.resolve({
								title: this.messages.previousMessage,
								uri: "previous",
								getSpaceUri: dojo.hitch(this, function(){
									return config.getSpaceUri();
								})
							});
							dojo.disconnect(prevHandle);
						}));
					} else {
						this.descendant = ContextUtils.getConfigurationAsync(args.descendantUri);
					}
					var descendantHandle = dojo.connect(this, "postCreate", dojo.hitch(this, function(){
						dojo.when(this.descendant, dojo.hitch(this, function(config){
							this.descendantPicker = this._makePicker({spaceUri: config.getSpaceUri()/*FIXME: validTypes:["workspace", "snapshot"]*/}, this. messages.descendantPickerTitle, this.messages.descendantPickerSecondaryTitle, this.descendantPickerNode, this.setDescendant);
							this.setDescendant(config);
							dojo.disconnect(descendantHandle);
							
							if(!this.otherDescendant){
								this.otherDescendantPicker = this._makePicker({spaceUri: config.getSpaceUri(), validTypes:["workspace"]}, this.messages.otherDescendantPickerTitle, this.messages.otherDescendantPickerSecondaryTitle, this.otherDescendantPickerNode, this.setOtherDescendant);
							}
						}));
					}));
				}
				
				if(/*String*/ args.otherDescendantUri){
					if(args.otherDescendantUri === "previous"){
						this.otherDescendant = new dojo.Deferred();
						var prevHandle = dojo.connect(this, "setDescendant", dojo.hitch(this, function(config){
							this.otherDescendant.resolve({
								title: this.messages.previousMessage,
								uri: "previous",
								getSpaceUri: dojo.hitch(this, function(){
									return config.getSpaceUri();
								})							});
							dojo.disconnect(prevHandle);
						}));
					} else {
						this.otherDescendant = ContextUtils.getConfigurationAsync(args.otherDescendantUri);
					}
					var otherDescendantHandle = dojo.connect(this, "postCreate", dojo.hitch(this, function(){
						dojo.when(this.otherDescendant, dojo.hitch(this, function(config){
							this.otherDescendantPicker = this._makePicker({spaceUri: config.getSpaceUri(), validTypes:["workspace"]}, this.messages.otherDescendantPickerTitle, this.messages.otherDescendantPickerSecondaryTitle, this.otherDescendantPickerNode, this.setOtherDescendant);			
							this.setOtherDescendant(config);
							dojo.disconnect(otherDescendantHandle);
							
							if(!this.descendant){
								this.descendantPicker = this._makePicker({spaceUri: config.getSpaceUri()/*FIXME: validTypes:["workspace", "snapshot"]*/}, this. messages.descendantPickerTitle, this.messages.descendantPickerSecondaryTitle, this.descendantPickerNode, this.setDescendant);
							}
						}));
					}));
				} 
			}
						
			this.descendants = [];
			this.otherDescendants = [];
			this.stepTitle = this.messages.step1Title;
		},
		
		postCreate: function(){
			this.descendantSelection = this._makeSelect(this.descendantSelectionNode, true);
			this.otherDescendantSelection = this._makeSelect(this.otherDescendantSelectionNode, false);
			dojo.connect(this.generateButton, "onClick", this, this.generate);
			dojo.connect(this.swapButton, "onClick", this, this.swap);
		},
		
		swap: function() {
			var tempDescendant = this.descendant;
			this.setDescendant(this.otherDescendant);
			this.setOtherDescendant(tempDescendant);
		},
		
		disableUi: function(disable) {
			this.descendantSelection.disabled = disable;
			this.otherDescendantSelection.disabled = disable;
			this.descendantPicker.disabled = disable;
			this.otherDescendantPicker.disabled = disable;
			this.generateButton.set("disabled", disable);
			
			if (this.descendant && !this.descendant.isMutable()) {
				this.swapButton.set("disabled", true);
			} else {
				this.swapButton.set("disabled", disable);
			}
		},
			
		_makeSelect: function(node, isForDescendant){
			var select = dojo.create("select", {"class" : " configurationSelection"});
			node.appendChild(select);
						
			if (isForDescendant) {
				dojo.connect(select, "change", this, function() {
					for (var i = 0; i < this.descendants.length; i++) {
						if (this.descendants[i].uri === select.value) {
							this.setDescendant(this.descendants[i]);
							break;
						}
					}
				});
			} else {
				dojo.connect(select, "change", this, function(){
					for (var i = 0; i < this.otherDescendants.length; i++) {
						if (this.otherDescendants[i].uri === select.value) {
							this.setOtherDescendant(this.otherDescendants[i]);
							break;
						}
					}
				});
			}
			
			var handle = dojo.connect(this, "generate", this, function(){
				select.disabled = true;
				dojo.disconnect(handle);
			});
			
			return select;
		},
		
		_makePicker: function(args, title, secondaryTitle, descendantNode, onOkCallback){
			var pickerArgs = {};
			   
			if(args){
				var pickerArgs = args;
			}
			pickerArgs.closeable = false;
			pickerArgs.singleClickMode = true;
			var picker = new com.ibm.team.vvc.shared.web.ui.picker.WorkspacePicker(pickerArgs);
			this._makeAddPicker(title, secondaryTitle, picker, descendantNode);		
			
			if(onOkCallback){
				dojo.connect(picker, "onOk", dojo.hitch(this, function(badConfigFormat){
					// The configuration object "badConfigFormat" returned from the workspace picker is not compatible with ConfigurationDescendant
					var config = new com.ibm.team.vvc.shared.core.ui.objects.ConfigurationResource();
					dojo.when(config.load(badConfigFormat.uri), dojo.hitch(this, onOkCallback));
				}));
			}
			
			return picker;
		},
		
		_makeAddPicker: function(title, secondTitle, picker, descendantNode){
			var boxArgs = {
					closable: false,
					collapsible: true,
					collapsed: true,
					primaryTitle: title,
					secondaryTitle: secondTitle,
					trim: jazz.ui.StyledBox.TrimStyles.BLUE,
					contentNode: picker.domNode
			};
			var box = new jazz.ui.StyledBox(boxArgs);
			descendantNode.appendChild(box.domNode);
			box.startup();		
		},
		
		addDescendant: function(/*ConfigurationResource*/ config, /*optional*/index){
			this._addConfig(config, index, this.descendants, this.descendantSelection);
		},
		
		addOtherDescendant: function(/*ConfigurationResource*/ config, /*optional*/ index){
			this._addConfig(config, index, this.otherDescendants, this.otherDescendantSelection);
		},
		
		setDescendant: function(/*ConfigurationResource*/ config){
			this._duplicateConfCheck(config, this.descendants);
			this.descendant = config;
			this.addDescendant(config);
			
			if (!config.isMutable()) {
				this.swapButton.set("disabled", true);
			} else {
				this.swapButton.set("disabled", false);
			}
		},
		
		setOtherDescendant: function(/*ConfigurationResource*/ config){
			this._duplicateConfCheck(config, this.otherDescendants);
			this.otherDescendant = config;
			this.addOtherDescendant(config);
		},
		
		_duplicateConfCheck: function(config, array){
			var index, to;
			if(array){
				for(index = array.length - 1; index >= 0; index--){
					if(array[index].uri === config.uri){
						array.splice(index, 1);
					}
				}
			}
		},
		
		_addConfig: function(/*ConfigurationResource*/ config, /*optional*/index, array, select){
			//FIXME: workaround for defect 39393
			if(select.disabled){
				return;
			}
						
			if(!array){
				array = [];
			}
			
			if(!index || index < 0){
				index = 0;
			}
			
			array.splice(index, 0, config);
			
			if(index === array.length){
				this._addOption(config, select);
			} else {
				this._resetOptions(array, select);
			}
			
			var enableGenerateButton = false;
			
			if(this.otherDescendant && !(this.otherDescendant instanceof dojo.Deferred)
					&& this.descendant && !(this.descendant instanceof dojo.Deferred)){
				
				if (this.otherDescendant.uri === this.descendant.uri) {
					this._showValidationWarning(this.messages.cannotCompareToSelf);
				} else if(this.otherDescendant.properties["http://jazz.net/ns/vvc#mutable"][0].value == "false"){
					this._showValidationWarning(this.messages.targetMustBeMutable);
				} else {
					enableGenerateButton = true;
				}
			}
			
			if(!this.merge){
				this.merge = {uri: null};
			}
			
			if(!this.otherDescendant){
				this.otherDescendant = {uri: null};
			}
			
			this._updateGenerateButton(!enableGenerateButton);
		},
		
		_isMergeOperation: function() {
			return this.operation === "rebase" || this.operation === "accept" || this.operation === "deliver";
		},
		
		_updateGenerateButton: function(disableGenerateButton){
			this.generateButton.set("disabled", disableGenerateButton);
			
			if (!disableGenerateButton) {
				// clear any validation messages
				this._showValidationWarning("");
			}
		},
		
		_showValidationWarning: function(message) {
			this.validationWarningLabel.innerHTML = message;
		},
		
		_addOption: function(/*ConfigurationResource*/ config, select){
			var option;
			if(config.uri === "previous"){
				option = dojo.create("option",{
					title: config.title,
					innerHTML: config.title,
					value: config.uri
				});
			} else {
				option = dojo.create("option", {
					title: config.getTitle(),
					innerHTML: config.getTitle(),
					value: config.getUri()
				});
			}
			select.appendChild(option);
			return option;
		},
		
		_resetOptions: function(array, select){
			dojo.empty(select);
			var index, option;
			for(index = 0; index < array.length; index++){
				option = this._addOption(array[index], select);
			}
		},

		generate: function(){
			if (this.descendant.uri === "previous" || this.otherDescendant.uri === "previous") {
				this.ancestor = {uri: null};
				if (this._isMergeOperation()) {
					 this.merge = {uri: this.otherDescendant.uri};
				}

				this.disableUi(true);
				
				// clear any validation warnings and start the merge process
				this._showValidationWarning("");
				if (this.startMergeProcess) {
					this.startMergeProcess();
				} // XXX fail if we don't have this?
			} else {
				this.disableUi(true);
				this._showValidationWarning(this.messages.validating);
				this.validateOnServer();
			}
		},
		
		validateOnServer: function() {
			var link = VvcUtils.getVvcUri()
					+ "/compare/getAncestorUri?descendantUri="
					+ encodeURIComponent(this.descendant.uri)
					+ "&otherDescendantUri="
					+ encodeURIComponent(this.otherDescendant.uri);
			
			var args = {
				url: link,
				handleAs: "json",
				headers: {
					Accept: "application/json;text/json"
				},
				load: function(response, ioArgs) {
					return response;
				},
				error: dojo.hitch(this, function(error, args) {
					ncFactory.createXhrError({xhrError: error, ioArgs: args}).dispatch();
				})
			};
			
			var deferred = jazz.client.xhrGet(args);
			var self = this;
			deferred.addCallback(this, function (validateResult) {
				if (!validateResult.ancestorUri) {					
					if (this._isMergeOperation()) {
						this.disableUi(false);
						this._showValidationWarning(this.messages.noCommonSnapshot);
						return;
					} else {
						// this is a two-way compare with dynamic workspaces...
						
						// make one of the ancestors the descendant
						this.ancestor = {uri: this.otherDescendant.uri};
						
						// and set the otherDescendant to null
						this.otherDescendant.uri = null;
					}
				} else {
					// set the ancestor as given by the server
					this.ancestor = {uri: validateResult.ancestorUri};
					
					// the merge URI is the same as the otherDescendant (target)
					if (this._isMergeOperation()) {
						 this.merge = {uri: this.otherDescendant.uri};
					}
				}

				// clear any validation warnings and start the merge process
				this._showValidationWarning("");
				if (this.startMergeProcess) {
					this.startMergeProcess();
				} // XXX fail if we don't have this?
			});
		}
		
	})
})();