/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011, 2013.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function(){

	dojo.provide("com.ibm.team.vvc.server.core.ui.internal.page.RebasePicker");

	dojo.require("dijit._Widget");
	dojo.require("dijit._Templated");
	dojo.require("jazz.ui.Breadcrumbs");

	dojo.require("net.jazz.ajax.ui.PlatformUI");
	var PlatformUI = net.jazz.ajax.ui.PlatformUI;

	dojo.require("com.ibm.team.vvc.server.core.ui.internal.util.PageUtil");
	var PageUtil = com.ibm.team.vvc.server.core.ui.internal.util.PageUtil;

	dojo.require("com.ibm.team.vvc.shared.web.ui.VvcConstants");
	var vvcConstants = com.ibm.team.vvc.shared.web.ui.VvcConstants;
	
	dojo.require("com.ibm.team.vvc.shared.web.ui.util.UIUtils");
	var UIUtils = com.ibm.team.vvc.shared.web.ui.util.UIUtils;
	
	dojo.require("com.ibm.team.vvc.shared.web.ui.VvcConstants");
	var vvcConstants = com.ibm.team.vvc.shared.web.ui.VvcConstants;
	
	dojo.require("com.ibm.team.vvc.shared.web.ui.util.VvcUtils");
	var VvcUtils = com.ibm.team.vvc.shared.web.ui.util.VvcUtils;
	
	dojo.require("jazz.app.ua.PageHoverHelp");
	
	dojo.require("com.ibm.team.vvc.server.core.ui.snapshots.SnapshotsViewer");
	dojo.requireLocalization("com.ibm.team.vvc.server.core.ui.internal.page", "RebasePickerMessages");
	
	var VVC_APP = "com.ibm.team.vvc.server.core";
		
	dojo.declare("com.ibm.team.vvc.server.core.ui.internal.page.RebasePicker", [dijit._Widget, dijit._Templated], {

		widgetsInTemplate: true,
		templatePath: dojo.moduleUrl(VVC_APP, "ui/internal/page/templates/RebasePicker.html"),

		_pickConfigViewer: null,
		_workspaceUri: null,
		_workspace: null,
		_parentWorkspace: null,

		constructor: function(workspace) {
			if (workspace) {
				this._workspace = workspace;
			}
			this.messages = dojo.i18n.getLocalization("com.ibm.team.vvc.server.core.ui.internal.page", "RebasePickerMessages");
		},
	
		postCreate: function() {
			if (!this._workspace) {
				var context = PageUtil.getRequestParmValue("contextUri");
				var uri = unescape(context);
				this._workspace = new com.ibm.team.vvc.shared.core.ui.objects.ConfigurationResource();
				var link = VvcUtils.getVvcUri() + "/repository/configuration?uri=" + uri;
				this._workspace.load(link);
				
				var parentUri = this._workspace.getParentWorkspaceUri();
				if (parentUri) {
					this._parentWorkspace = new com.ibm.team.vvc.shared.core.ui.objects.ConfigurationResource();
					link = VvcUtils.getVvcUri() + "/repository/configuration?uri=" + parentUri;
					this._parentWorkspace.load(link);
				}
			}
			if (this.rebaseButtonContainer) {
				var rebaseButton = dojo.create("button");
				rebaseButton.appendChild(dojo.doc.createTextNode(this.messages.run));
				rebaseButton.style.minWidth = "80px";
				dojo.connect(rebaseButton, "onclick", this, "onRebase");
				this.rebaseButtonContainer.appendChild(rebaseButton);
			}
			
			// Register the page action
			var ActionRegistry = PlatformUI.getWorkbench().getActionRegistry();
			ActionRegistry.registerAction(vvcConstants.EDIT_BASE_CONFIGURATION, this, "showPage");
			ActionRegistry.registerAction(vvcConstants.OLD_REBASE_PICKER, this, "showPage");
		},

		showPage: function() {
			this._initBreadcrumbs();
			this._initializeCSH();
			
			this.showMessage({
				type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
				summary: this.messages.selectConfig
			});
			
			// remove the old viewer first
			if (this._pickConfigViewer) {
				var node = this._pickConfigViewer.domNode;
				node.parentNode.removeChild(node);
			}
			
			// update labels
			if (this._workspace) {
				this.targetWorkspceLabel.innerHTML = this._workspace.getTitle();
				this.parentConfigLabel.innerHTML = this._workspace.getParentSnapshotName();
			}
			if (this._parentWorkspace) {
				this.parentWorkspaceLabel.innerHTML = this._parentWorkspace.getTitle();
			}
			
			if (this._parentWorkspace) {
				var args = { workspace: this._parentWorkspace, style: 1 };
				this._pickConfigViewer = new com.ibm.team.vvc.server.core.ui.snapshots.SnapshotsViewer(args);
				this.tableContainer.appendChild(this._pickConfigViewer.domNode);
				this._pickConfigViewer.startup();
			}
		},
		
		_initializeCSH: function() {	
			this._ua = new jazz.app.ua.PageHoverHelp({uaKey: "com.ibm.team.vvc.server.core.page_RebasePicker001"});
			this._pageHeader.appendChild(this._ua.domNode);
			this._ua.domNode.style.margin = "0px 0px 0px 5px";	
	   	},
	   	
		_initBreadcrumbs: function()
		{
			if (!this._breadcrumbs)
			{
				var parts = [];
				
				var explorerPageUri = PageUtil.createLink(vvcConstants.SHOW_EXPLORER);
				segment = { label: this.messages.configurations, href : explorerPageUri };
				parts.push(segment);
				
				var workspaceName = this.messages.workspace;
				if (this._workspace) {
					workspaceName = this._workspace.getTitle();
				}
				var redirectUri = PageUtil.getRequestParmValue("redirectUri");
				
				if (redirectUri) {
					redirectUri = decodeURIComponent(unescape(redirectUri));
					segment = { label: workspaceName, href : redirectUri };
					parts.push(segment);
				}
				
				this._breadcrumbs = new jazz.ui.Breadcrumbs({segments: parts });
				
				this._breadcrumbs.placeAt(dojo.byId("breadcrumbs"));
			}
		},
		
		onRebase: function(e) {
			var selection = this._pickConfigViewer.getSelection();
			if (selection.length === 1 ) {
				var selectedUri = selection[0].uri;
				if (this.isValidSelection(selectedUri) && this._workspace) {
					var args = {};
					args.operation = "rebase";
					args.ancestorUri = this._workspace.getParentSnapshotUri();
					args.descendantUri = selectedUri;
					args.otherDescendantUri = this._workspace.uri;
					args.mergeUri = this._workspace.uri;
					args.breadcrumb = encodeURIComponent(this._workspace.getTitle() + ' - ' + this.messages.title);
					args.redirectUri = encodeURIComponent(window.location.href);
					args.hideConfigurationPicker = true;
					PageUtil.showPage(vvcConstants.EXPLORE_COMPARISON, dojo.objectToQuery(args));
				}
			} else if (selection.length === 0) {
				this.showMessage({
					type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
					summary: this.messages.noSelectionMessage
				});
			}
		},
		
		isValidSelection: function(selectedUri) {
			if (selectedUri) {
				return true;
				if (selectedUri == this._workspace.getParentSnapshotUri()) {
					alert(this.messages.pickSameSnapshot);
				} else {
					return true;
				}
			}
		},
		
		showMessage: function(args) {
			UIUtils.hidePageMessage();
			if (args && this._messageBoxContainer) {
				args["closable"] = true;
				if (!this.messageBox) {
					this.messageBox = new jazz.ui.MessageBox();
					this._messageBoxContainer.appendChild(this.messageBox.domNode);
				}
				this.messageBox.showMessage(args);
				this._messageBoxContainer.style.display = "";
			}
		},
		
		hideMessage: function() {
			if (this._messageBoxContainer) {
				this._messageBoxContainer.style.display = "none";
			}
		},

	})   
})();
