/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011, 2013.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function() {

	dojo.provide("com.ibm.team.vvc.server.core.ui.internal.util.PageUtil");
	dojo.require("jazz.ui.Dialog");
	dojo.require("jazz.ui.MessageBox");
	dojo.require("dojox.xml.DomParser");
	dojo.require("com.ibm.team.vvc.shared.web.ui.VvcConstants");
	dojo.require("net.jazz.ajax.BootstrapProperties");
	dojo.require("com.ibm.team.vvc.shared.core.ui.utils.ErrorUtil");
	
	dojo.require("com.ibm.team.vvc.shared.web.ui.notifications.notifications.VvcNotificationFactory");
	var ncFactory = com.ibm.team.vvc.shared.web.ui.notifications.notifications.VvcNotificationFactory;
	
	dojo.requireLocalization("com.ibm.team.vvc.server.core.ui.internal.util", "PageUtilMessages");
	var messages = dojo.i18n.getLocalization("com.ibm.team.vvc.server.core.ui.internal.util", "PageUtilMessages");
	var vvcConstants = com.ibm.team.vvc.shared.web.ui.VvcConstants;

	dojo.require("com.ibm.team.vvc.shared.web.ui.util.VvcUtils");
	var VvcUtils = com.ibm.team.vvc.shared.web.ui.util.VvcUtils;
	
	dojo.require("com.ibm.team.vvc.shared.web.ui.util.VvcUtils");
	var VvcUtils = com.ibm.team.vvc.shared.web.ui.util.VvcUtils;
	
	com.ibm.team.vvc.server.core.ui.internal.util.PageUtil = {
		
		/* Deprecated since 4.0.2 */
		CLIENT_INTERFACE: "clientInterface",
		
		VVC_URI: null,
		
		/**
		 * If the operation= parameter is set to create, this will override the editor action.  The case where
		 * you have a new editor '#action=' fragment mixed with the old 'operation=' query parameter does not
		 * really make sense.  New code should not be aware of the old 'operation=' parameter; if it is present
		 * then we will assume a legacy operation and respect that parameter
		 * 
		 * @deprecated VVC 4.0.2 (WI35709)
		 * @return True if 'operation=create' is present anywhere in the URI.
		 */
		isCreate: function() {
			if(dojo.config.isDebug){
				console.warn("Using the 'operation' parameter in the URI has been deprecated in 4.0.2 and will be removed in the future.  Please use a proper editor name to specify the action you are trying to accomplish.")
			}
			var loc = window.location.href;
			if (loc.indexOf(vvcConstants.CREATE_OPERATION) !== -1) {
				return true;
			}
			return false;
		},
				
		navigateToMainPage: function() {
			// go back to VVC home page if we don't have a redirect URL.
			window.location.href = VvcUtils.getVvcUri();
		},
		
		getCurrentPageId: function() {
			return net.jazz.ajax.ui.PlatformUI.getWorkbench().currentPageId;
		},

		setWorkbenchTitle: function(title) {
			return net.jazz.ajax.ui.PlatformUI.getWorkbench().setTitle(title);
		},
		
		showMessage: function(msg) {
			 PlatformUI.getWorkbench().showMessage(msg);
		},
		
		getClientAppBreadcrumbSegment: function() {
			var link = this.getRequestParmValue(this.CLIENT_INTERFACE);
			if (!link) {
				return;
			}
			link = unescape(link);
			var context = this.getRequestParmValue("context", link);
			if (!context) {
				return;
			}
			context = unescape(context);
			var name = this.getRequestParmValue("contextClientName", link);
			if (name) {
				name = decodeURIComponent(unescape(name));
			} else {
				name = messages.clientApplication
			}
			return {label: name, href: context };
		},
		
		openMessageDialog: function(title, messages) {
			if (this.dialog) this.dialog.destroy(); 
			
			var contentNode = dojo.create("span");
			this.dialog = new jazz.ui.Dialog({
		    	contentNode: contentNode,
		    	primaryTitle: title,
		    	width: "400px",
		    	closeOnBlur: true
			});
			contentNode.innerHTML = messages;
		},
		
		getUser: function(userUri) {
			var link = VvcUtils.getVvcUri() + "/repository/user?uri=" + userUri;
			return this.httpGet(link);
		},
		
		httpGet: function(uri) {
			var result;
			var args = {
				url: uri,
				handleAs: "json",
				sync: true,
				headers: {
					Accept: "application/json;application/rdf+json;text/rdf+json;text/json"
				},
			};
			var deferred = jazz.client.xhrGet(args);
			deferred.addCallback(this, function (param) {
				result = param;
			});
			return result;
		},
		
		getBaseline: function(){
			return jazz.app.context.get().baseline;
		},
		
		htmlEscape: function(text) {
			return jazz.util.html.escape(text);
		},
				
		createWorkspace: function(name, description, parent, space, unique, renameAutomatically) {
			return this._createConfigurationResource(name, description, parent, space, true, unique, renameAutomatically);
		},
		
		createSnapshot: function(name, description, parent, space, unique, renameAutomatically) {
			return this._createConfigurationResource(name, description, parent, space, false, unique, renameAutomatically);
		},

		//unique: whether the name must be unique
		//renameAutomatically: whether rename should occur if the name is not unique 
		//renameAutomatically has no meaning and is ignored if unique is set to false
		_createConfigurationResource: function(name, description, parent, space, mutable, unique, renameAutomatically) {
			var data = {name: name, description: description, mutable: mutable, space: space};
			var childUriParams = {};
			
			if(parent){
				childUriParams.oneGreaterThan = parent;
			} else if(parent == "null"){
				// If this is hit, please re-open defect 33801
				console.log("Trying to create a configuration \"" + name + "\" without a base snapshot or flow target. Check the server logs for details.");
			}
			
			if(unique){
				childUriParams.unique = true;
			}
			
			if(renameAutomatically){
				childUriParams.renameAutomatically = true;
			}
			
			var args = {
					url: VvcUtils.getVvcUri() + "/configurations/child?" + dojo.objectToQuery(childUriParams),
					headers: {
						"Accept" : "application/json",
						"Content-Type" : "application/json"
					},
					sync: true,
					handleAs: "json",
					postData: dojo.toJson(data),
					load: function(response, ioArgs) {
						if (ioArgs) {
							var xhr = ioArgs.xhr;
							if (xhr) {
								return xhr.getResponseHeader("Location");
							}
						}
						return response;
					},
					error: dojo.hitch(this, function(err, args){
						var errorDetails = com.ibm.team.vvc.shared.core.ui.utils.ErrorUtil.getErrorDetails(err);
						var args = {
							title: messages.defaultError,
							description: messages['defaultError.explanation'],
							solution: messages['defaultError.useraction']
						};
						if (errorDetails.errorStatus === 403) {
							args.description = messages.forbiddenDescription;
							args.solution = messages.forbiddenAction;
							ncFactory.createError(args).dispatch();
						} else if (errorDetails.errorSubStatus === "EXISTS") {
							args.description = messages.configurationExistsDescription;
							args.solution = messages.configurationExistsAction;
							ncFactory.createError(args).dispatch();
						} else if (errorDetails.errorSubStatus === "PARENT_ARCHIVED") {
							args.description = messages.configurationArchivedDescription;
							args.solution = messages.configurationArchivedAction;
							ncFactory.createError(args).dispatch();
						} else {
							ncFactory.createXhrError({xhrError: err, ioArgs: args}).dispatch();
						}
						
					})
			};
			return jazz.client.xhrPost(args);
		},
		
		/**
		 * <p>Finds a configurations URI by name.  If 'isSync' is <code>false</code> then instead of returning the configuration URI (if found),
		 * this method will return the deferred which is used to find the configuration name; if synchronous, expect <code>NULL</code> or
		 * the configuration name, if asynchronous expect a deferred.</p>
		 * 
		 * <p><b>New invocations should use asynchronous mode!</b></p>
		 * 
		 * @param configName Name of the configuration
		 * @param spaceUri Configuration Space to search
		 * @param mutable If the type of configuration is mutable (workspace) or immutable (snapshot)
		 * @param isSync Optional parameter to run in synchronous mode.  Defaults to true if unspecified
		 * @return The URI of the configuration, if found; <code>NULL</code> if not, or a dojo.Deferred if in asynchronous mode.
		 */
		findConfigurationName: function(configName, spaceUri, mutable, /*Optional*/isSync) {
			var serviceName;
			
			if (mutable) {
				serviceName = "findWorkspaceName";
			} else {
				serviceName = "findSnapshotName";
			}
			
			// Default to true
			if(isSync !== false){
				isSync = true;
			}
			
			var uri = VvcUtils.getVvcUri() + "/repository/" + serviceName + "?configurationName="
				+ encodeURIComponent(configName) + "&spaceUri=" + encodeURIComponent(spaceUri) + "&mutable=" + mutable;
			var args = {
				url: uri,
				headers: {
					"Accept" : "application/json"
				},
				sync: isSync,
				handleAs: "text",
				load: function(response, ioArgs) {
					if (ioArgs) {
						var xhr = ioArgs.xhr;
						if (xhr) {
							return xhr.getResponseHeader("Location");
						}
					}
				},
			};
			var deferred = jazz.client.xhrGet(args);
			
			if(isSync){
				var configUri = null;
				deferred.addCallback(this, function(param){
					if (param)
						configUri = param;
				});
				deferred.addErrback(function(error) {
					alert('error '+e.message);
				});
				return configUri;
			} /* else if asynchronous */
			return deferred;
		},
		
		getRequestParmValue: function(paramName, uri) {
			var params = uri;
			if (!params) {
				params = window.location.href;
			}
			var i = params.indexOf("?");
			if (i !== -1) {
				params = params.substring(i+1);
				params = params.replace("#", "&");
			} else {
				i = params.indexOf("#");
				params = params.substring(i);
			}
			var nvps = [];
			var rawNvps = params.split("&");
			for(var i=0; i<rawNvps.length; i++) {
				var nvp = rawNvps[i].split("=");
				if (nvp[0] === paramName) {
					return nvp[1];
				}
			}
			return null;
		},
		
		getRequestParms: function(url) {
			if (!url) {
				url = new dojo._Url(window.location.href);
			} else if (typeof url === 'string') {
				url = new dojo._Url(url);
			}
			var nvps = [];
			var rawNvps = url.query ? url.query.split("&") : [];
			for(var i=0; i<rawNvps.length; i++) {
				var nvp = rawNvps[i].split("=");
				nvps.push({n: nvp[0], v: nvp[1]});
			}
			return nvps;
		},
		
		createXhrErrorMessage: function(ioArgs) {
			var xhrError = this.createXhrError(ioArgs);
			var msg = { type: jazz.ui.MessageBox.STATUS_TYPES.ERROR, summary: xhrError.message };
			if (xhrError.detailedMessage) {
				msg.details = xhrError.detailedMessage;
			}
			return msg;
		},
		
		createXhrError: function(ioArgs) {
			var contentType = ioArgs.xhr.getResponseHeader("Content-Type");
			if (ioArgs.xhr.responseText && contentType && contentType.toLowerCase() === "application/rdf+xml") {
				var rdf = dojox.xml.DomParser.parse(ioArgs.xhr.responseText);
				rdfDescription = rdf.getElementsByTagName("rdf:Description")[0];
				return this._createErrorObject(rdfDescription);
			}
			
			var xhrError = {};
			if (ioArgs.xhr.statusText) {
				xhrError.message = ioArgs.xhr.statusText + " - " + ioArgs.url;
			} else {
				xhrError.message = dojo.string.substitute(messages.defaultXhrErrorMessage, [ioArgs.url]);
			}
			
			if (ioArgs.xhr.responseText) {
				xhrError.detailedMessage = [ {message: ioArgs.xhr.responseText} ];
			}
			xhrError.date = ioArgs.xhr.getResponseHeader("Date");
			xhrError.status = ioArgs.xhr.status;
			return xhrError;
		},
		
		/* Error Object structure
			errorStatus      The major status code, usually the corresponding HTTP status code.
			errorMessage     Some textual response.
			errorSubStatus   The minor status code, usually a Foundation specified value.
			errorSeverity    One of the values: info, warning, error, critical.
			detailedMessage  Any additional information the application needs to convey.
			seeAlso          A URL to a resource that may contain information such as how to resolve this error.
			errorCause       Another err:Report that denotes a condition that caused this error report to be generated.
		 */
		_createErrorObject: function (rdfDescription) {
			var xhrError = {};		
			for (var i = 0; i < rdfDescription.childNodes.length; ++i) {
				var childNode = rdfDescription.childNodes[i];
				if (childNode.nodeType === 1 && childNode.childNodes.length > 0 && childNode.childNodes[0].nodeValue) {
					switch (String(childNode.nodeName)) {
						case 'err:errorStatus':
							xhrError.status = childNode.childNodes[0].nodeValue;
							break;
						case 'err:errorMessage':
							xhrError.message = childNode.childNodes[0].nodeValue;
							break;
						case 'err:errorSubStatus':
							xhrError.subStatus = childNode.childNodes[0].nodeValue;
							break;
						case 'err:errorSeverity':
							xhrError.severity = childNode.childNodes[0].nodeValue;
							break;
						case 'err:detailedMessage':
							var value = childNode.childNodes[0].nodeValue;
							xhrError.detailedMessage = [ {message: value} ];
							break;
						case 'err:seeAlso':
							xhrError.seeAlso = childNode.childNodes[0].nodeValue;
							break;
						case 'err:errorCause':
							xhrError.cause = this._createErrorObject(childNode.getElementsByTagName("rdf:Description")[0]);
							break;
					}
				}
			}
			
			return xhrError;
		},
		
		setLinkField: function(field, value, uri, pageId, tooltip) {
			if (field) {
				if (field.firstChild) {
					field.removeChild(field.firstChild);
				}

				if (uri) {
					uri = this.createLink(pageId, "operation=open&contextUri=" + uri);
					var link = dojo.create("a");
					link.href = uri;
					link.innerHTML = this.htmlEscape(value);
					if (tooltip) {
						link.title = tooltip;
					}
					field.appendChild(link);
				} else {
					var node = dojo.doc.createTextNode(value);
					field.appendChild(node);
				}
			}
		},
		
		setLinkUri: function(field, value, uri) {
			if (field.firstChild) {
				field.removeChild(field.firstChild);
			}

			var link = dojo.create("a");
			link.href = uri;
			link.innerHTML = this.htmlEscape(value);
			field.appendChild(link);
		},		

		createLink : function(pageId, queryString, fragmentString) {
			var loc = window.location.href;
			var i = loc.indexOf("?");
			var uri = loc;
			if (i !== -1) {
				uri = loc.substring(0, i);
			} else {
				i = loc.indexOf("#");
				if (i !== -1) {
					uri = loc.substring(0, i);
				}
			}
			
			// check for debug flag
			var debugFlag = "debug=true";
			if (loc.indexOf(debugFlag) !== -1) {
				queryString = this.addQueryParam(queryString, "debug", "true");
			}
			
			// keep client interface parameter
			var value = this.getRequestParmValue(this.CLIENT_INTERFACE);
			if (value) {
				queryString = this.addQueryParam(queryString, this.CLIENT_INTERFACE, value);
			}
			
			if (queryString) {
				uri += "?" + queryString;
			}
			if (pageId) {
				uri = uri + "#action=" + pageId;
				if (fragmentString) {
					uri += "&" + fragmentString;
				}
				return uri;
			}
		},
		
		addQueryParam: function(queryString, paramName, paramValue) {
			if (!queryString) {
				return paramName + "=" + paramValue;
			}
			return queryString + "&" + paramName + "=" + paramValue;
		},

		showPage : function(pageId, queryString, fragmentString) {
			window.location.href = this.createLink(pageId, queryString, fragmentString);
		},
		
		showAction : function(action, queryString) {
			this.showPage(action, queryString);
		},
		
		openEditor: function(type, uri, spaceUri)
		{
			if (type == "snapshot")
				page = vvcConstants.EDIT_SNAPSHOT;
			else
				page = vvcConstants.EDIT_WORKSPACE;
			this.showAction(page, "contextUri=" + escape(uri)+"&spaceUri="+escape(spaceUri));
		},
		
		createEditorLink: function(type, uri, spaceUri){
			var pageId = type === "snapshot" ? vvcConstants.EDIT_SNAPSHOT : vvcConstants.EDIT_WORKSPACE;
			var queryString = "contextUri=" + escape(uri) + "&spaceUri=" + escape(spaceUri);
			return this.createLink(pageId, queryString);
		},
		
		getReturnLink: function(configurationUri)
		{
			var returnUrl = this._getReturnUrl(configurationUri);
			
			if (returnUrl)
				return " <a href=\'"+returnUrl+"\'> "+this._getReturnTitle()+"</a>";
			return " <a href=\"#\" onclick=\"history.back()\" > "+messages.returnBack+"</a>";
		},
		
		_getReturnTitle: function()
		{
			var returnTitle = this.getRequestParmValue("redirectTitle");
			if (returnTitle)
				return returnTitle;
			return messages.returnBackToApplication;
		},
		
		_getReturnUrl: function(configurationUri) {
			var redirectUri = this.getRequestParmValue("redirectUri");
			if (redirectUri) {
				redirectUri = unescape(redirectUri);
				// insert the newly created configuration URI
				var i = redirectUri.indexOf('?');
				if (i == -1) {
					redirectUri += "?configurationUri=" + escape(configurationUri);
				} else {
					redirectUri = redirectUri.substring(0,i+1) + "configurationUri=" 
					+ escape(configurationUri) + "&"
					+ redirectUri.substring(i+1);
				}
				
				return redirectUri;
			}
			return null;
		},
		
		queryForConcept: function(conceptUrl, configurationUrl){
			var queryUrl = VvcUtils.getVvcUri() + "/configurations/query?concept=" + conceptUrl + "&configuration=" + configurationUrl;
			var args = {
					url: queryUrl,
					headers: {
						"Accept" : "application/rdf+xml",
						"Content-Type" : "application/json"
					},
					handleAs: "json"
			};
			return jazz.client.xhrGet(args);
		}
	};

})();
