/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2013.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function() {
dojo.provide("com.ibm.team.vvc.server.core.ui.internal.util.VvcButtonFactory");
dojo.require("jazz.ui.toolbar.Button");
dojo.require("jazz.ui.toolbar.TwoStateButton");

dojo.require("com.ibm.team.vvc.shared.web.ui.util.TeamAreaDialogContent");

dojo.require("com.ibm.team.vvc.shared.web.ui.VvcConstants");
var vvcConstants = com.ibm.team.vvc.shared.web.ui.VvcConstants;

dojo.require("com.ibm.team.vvc.shared.web.ui.notifications.notifications.VvcNotificationFactory");
var ncFactory = com.ibm.team.vvc.shared.web.ui.notifications.notifications.VvcNotificationFactory;

dojo.require("com.ibm.team.vvc.shared.web.ui.util.SecurityConstants");
var SecurityConstants = com.ibm.team.vvc.shared.web.ui.util.SecurityConstants;

dojo.require("com.ibm.team.vvc.shared.web.ui.util.SecurityServiceUtils");
var SecurityServiceUtils = com.ibm.team.vvc.shared.web.ui.util.SecurityServiceUtils;

dojo.require("com.ibm.team.vvc.shared.web.access.AccessUtils");
var AccessUtils = com.ibm.team.vvc.shared.web.access.AccessUtils;

dojo.require("com.ibm.team.vvc.shared.web.ui.util.InfoContentUtils");
var InfoContentUtils = com.ibm.team.vvc.shared.web.ui.util.InfoContentUtils;

dojo.require("com.ibm.team.vvc.server.core.ui.internal.util.PageUtil");
var PageUtil = com.ibm.team.vvc.server.core.ui.internal.util.PageUtil;

dojo.require("com.ibm.team.vvc.shared.web.ui.context.ContextUtils");
var ContextUtils = com.ibm.team.vvc.shared.web.ui.context.ContextUtils;

dojo.require("com.ibm.team.vvc.shared.web.ui.util.VvcUtils");
var VvcUtils = com.ibm.team.vvc.shared.web.ui.util.VvcUtils;

dojo.requireLocalization("com.ibm.team.vvc.server.core.ui.internal.util", "VvcButtonFactoryMessages");
var messages = dojo.i18n.getLocalization("com.ibm.team.vvc.server.core.ui.internal.util", "VvcButtonFactoryMessages");

com.ibm.team.vvc.server.core.ui.internal.util.VvcButtonFactory = {
	
		/**
		 * A factory for creating buttons useful in VVC.
		 * 
		 * General rundown of parameters:
		 * 
		 * @param data 	(optional) The data available at button-creation time.  For example, the Create Workspace 
		 * 				button uses this parameter to extract the configuration which the new workspace
		 * 				will be based on, since this information is available when the button is created.
		 * 
		 * @param options 	(optional) These are options which affect how the button is to be created.  For example, 
		 * 					a Checkbox Button might be created and, using the options parameter, it will
		 * 					be created as ticked
		 * 
		 *  @param onClikcArgs 	(optional) Arguments which are only available when the button has been clicked.  
		 *  					Argument should be a function which is hitched back into the calling context.  
		 *  					For example, the Compare Button button uses this to determine the selected 
		 *  					configurations 
		 */
		
		// -------------------------------------------------------------------------------
		
		getCompareWithPreviousButton: function(data, options, onClickArgs){
			if(!options){
				options = {};
			}
			dojo.mixin(data, {ancestorUri: "previous"}); // "previous" is a macro to find the ancestor on the server
			dojo.mixin(options, {title: messages.compareWithPreviousButtonTitle});
			return this.getCompareButton(data, options , onClickArgs);
		},
		
		getCompareButton: function(data, options, onClickArgs){
			var args = {
				imgClass: "icons-compare",
				title: (options && options.title) ? options.title : messages.compareButtonTitle,
				onClick: dojo.hitch(this, function(){
					// onClickArgs are required!
					var args = data ? data : {};
					args.operation = "compare";
					if(onClickArgs){
						// onClick args override create-time args
						dojo.mixin(args, onClickArgs());
					}
					this._launchMergeToolHelper(args);
				}),
			};
			var applicable = function(){
				if(data && data.underConstruction){
					return false;
				}
				return !(data) || (data.ancestor && data.descendant) && (data.ancestor.archive ? ! data.ancestor.archive : true) || (data.descendant ? data.descendant.archive : true); 
			};
			return this._makeButton(args, options, applicable);
		},
		
		getDeliverButton: function(data, options, onClickArgs){
			var args = {
				imgClass: "icons-deliver",
				title: messages.deliverButtonTitle,
				onClick: dojo.hitch(this, function(){
					// onClickArgs are required!
					var args = data ? data : {};
					args.operation = "deliver";
					if(onClickArgs){
						// onClick args override create-time args
						dojo.mixin(args, onClickArgs());
					}
					this._launchMergeToolHelper(args);
				})
			};
			var applicable = function(){
				if(data && data.underConstruction){
					return false;
				}
				return true;
			};
			var button = this._makeButton(args, options, applicable);
			return button;
		},
		
		getCreateWorkspaceButton: function(data, options, onClickArgs){
			var args = {
				imgClass: "icons-create-workspace",
				title: messages.createWorkspaceButtonTitle,
				onClick: dojo.hitch(this, function(){
					PageUtil.showAction(vvcConstants.CREATE_WORKSPACE, dojo.objectToQuery({
						contextUri: data.uri,
						spaceUri: data.spaceUri
					}));
					if(options && options.actionCallback){
						options.actionCallback(param);
					}
				}),
			};
			var applicable = function(){
				if(data && data.underConstruction){
					return false;
				}
				if(!data.archive){
					return dojo.indexOf(data.allowedActions, SecurityConstants.permissions.createOrModifySnapshot) >= 0;
				}
				return false;
			};
			return this._makeButton(args, options, applicable);
		},
		
		getCreateSnapshotButton: function(data, options, onClickArgs){
			var args = {
				imgClass: "icons-create-snapshot",
				title: messages.createSnapshotButtonTitle,
				onClick: dojo.hitch(this, function(){
					PageUtil.showAction(vvcConstants.CREATE_SNAPSHOT, dojo.objectToQuery({
						contextUri: data.uri,
						spaceUri: data.spaceUri
					}));
					if(options && options.actionCallback){
						options.actionCallback(param);
					}
				}),
			};
			var applicable = function(){
				if(data && data.underConstruction){
					return false;
				}
				if(!data.archive && data.mutable == true){
					return dojo.indexOf(data.allowedActions, SecurityConstants.permissions.createOrModifySnapshot) >= 0;
				}
				return false;
			};
			return this._makeButton(args, options, applicable);
		},

		getArchiveUnarchiveConfigurationButton: function(data, options, onClickArgs){
			var args = {
				imgClass: data.archive ? "icons-unarchive" : "icons-archive",
				title: data.archive ? messages.unarchiveButtonTitle : messages.archiveButtonTitle,
				onClick: dojo.hitch(this, function(){
					var result;
					var args = {
						url: VvcUtils.getVvcUri() + "/configurations/" + (data.archive === true ? "un" : "") + "archive?configuration=" + data.uri,
						handleAs: "json",
						headers: {
							Accept: "application/json;application/rdf+json;text/rdf+json;text/json"
						}
					};
					var deferred = jazz.client.xhrPost(args);
					var notification;
					deferred.then(function(param){
						if(options && options.actionCallback){
							options.actionCallback(param);
						}
						if(notification){
							notification.close();
						}
					}, /*Error Handler */function(err){
						/*
						 * Archiving gets a special error message because we have
						 * to alert the user which configurations are the problem.
						 * Demonstrates how to use the VvcNotificationCenter to
						 * easily display HTML elements. 
						 */
						if(notification){
							notification.close();
						}
						
						var args = {
							ioArgs: err.response
						};
						
						if(args.ioArgs){
							ncFactory.getRdfParts(args);
							var message = args.title;
							
							var type;
							var resources;
							
							var description;
							var solution;
							
							var typeTitle = "Archiving exception:";
							var resourcesTitle = "Resources:";
							
							var typeIndex = message.indexOf(typeTitle);
							var resourcesIndex = message.indexOf(resourcesTitle);

							type = message.substring(typeIndex + typeTitle.length, 
									resourcesIndex >= 0 ? resourcesIndex : message.length).trim();
							
							var messageId = "error_" + type;;
							
							if(resourcesIndex >= 0){
								var resourcesAsString =  message.substring(resourcesIndex + resourcesTitle.length, 
										message.length).trim();
								resources = resourcesAsString.split(',');
								description = dojo.string.substitute(messages[messageId + ".explanation"], [resources.length]);
								
								var blockingResources = dojo.create("span");
								var list = dojo.create("textarea", {
									style: "width: 850px; height: 100px",
									readonly: true
								});
								dojo.forEach(resources, dojo.hitch(this, function(resource){
									list.appendChild(dojo.doc.createTextNode(resource + "\n"));
								}));
								blockingResources.appendChild(dojo.doc.createTextNode(description));
								blockingResources.appendChild(dojo.create("br"));
								blockingResources.appendChild(list);
								description = blockingResources.outerHTML;
							}

							if (messages[messageId]) {
								ncFactory.createError({
									"title": dojo.string.substitute(messages[messageId], {
										operation: data.archive ? messages.restore_operation_title : messages.archive_operation_title,
										title: data.title, 
										type: data.type
									}),
									"description": description,
									"solution": messages[messageId + ".useraction"],
								}).dispatch();
							} else {
								ncFactory.createXhrError({
									title: data.archive ? messages.unarchiveConfigurationErrorTitle : messages.archiveConfigurationErrorTitle,
									xhrError: args.ioArgs.xhr, 
									ioArgs: args.ioArgs}
								).dispatch();
							}
						} else {
							ncFactory.createXhrError({
								title: data.archive ? messages.unarchiveConfigurationErrorTitle : messages.archiveConfigurationErrorTitle,
								xhrError: args.xhr, 
								ioArgs: err.response}
							).dispatch();
						}
					});
					if(options && !options.supressNotifications){
						var title = data.archive ? messages.unarchvingConfigurationNotificationTitle : messages.archivingConfigurationNotificationTitle; 
						var description = data.archive ? messages.unarchivingConfigurationNotificationDescription : messages.archivingConfigurationNotificationDescription;
						notification = ncFactory.createInfo({
							"title" : title,
							"description" : description
						}).dispatch();
					}
				})
			};

			var applicable = function(){
				if(data && data.underConstruction){
					return false;
				}
				var applicable = dojo.indexOf(data.allowedActions, SecurityConstants.permissions.archive_restore) >= 0;
				if(!applicable) {
					button.set("title", SecurityServiceUtils.getNoPermissionMessage(data.controllingAreaName));
				}
				return applicable;
			};
			return this._makeButton(args, options, applicable);
		},
		
		getProjectAssociationButton: function(data, options, onClickArgs){
			var args = {
				imgClass: "icons-associate-project-or-team-area",
				title: messages.associateProjectOrTeamAreaAction,
				onClick: dojo.hitch(this, function(){
					InfoContentUtils.showDialog(com.ibm.team.vvc.shared.web.ui.util.TeamAreaDialogContent, messages.associateProjectOrTeamAreaDialog, {
						uri: data.uri, 
						spaceUri: data.spaceUri, 
						title: data.title,
						okFunction: dojo.hitch(this, function(projectOrTeamAreaUri, type) {
							var deferred = AccessUtils.setConfigurationOwner(data.uri, type[0] === "projectArea" ? null : projectOrTeamAreaUri);
							deferred.addCallback(this, function (param) {
								if(options && options.actionCallback){
									options.actionCallback(param);
								}
							});
							return deferred;
						})
					});
				})
			}
			var applicable = function(){
				if(data && data.underConstruction){
					return false;
				}
				if(!data.archive && data.mutable == true){
					return dojo.indexOf(data.allowedActions, SecurityConstants.permissions.setControlledBy) >= 0;
				}
				return false;
			}
			return this._makeButton(args, options, applicable);
		},
		
		getCheckboxButton: function(data, options, onClickArgs){
			var checked = false;
			var shaded = false;		
			var args = {
				imgClass: "icons-checkbox-unchecked",
				onClick: dojo.hitch(this, function(){
					checked = !checked;
					var imageClass = "icons-checkbox-" + (checked ? "" : "un") + "checked" + (shaded ? "-shaded" : "");
					button.set("imgClass", imageClass);
					if(options && options.actionCallback){
						options.actionCallback({
							"checked" : checked
						});
					}
				})
			}
			var applicable = function(){
				if(data && data.underConstruction){
					return false;
				}
				return true;
			};
			var button = this._makeButton(args, options, applicable);
			return button;
		},
		
		getRefreshButton: function(data, options, onClickArgs){
			var args = {
				imgClass: "icons-refresh",
				title: messages.refreshButtonTitle,
				onClick: dojo.hitch(this, function(){
					if(options && options.actionCallback){
						options.actionCallback();
					}
				})
			};
			var applicable = function(){
				return true;
			};
			var button = this._makeButton(args, options, applicable);
			return button;
		},
			
		getExpandAllButton: function(data, options, onClickArgs){
			var args = {
				imgClass: "icons-expand-all",
				title: messages.expandAllButtonTitle,
				onClick: dojo.hitch(this, function(){
					if(options && options.actionCallback){
						options.actionCallback();
					}
				})
			};
			var applicable = function(){
				return true;
			};
			var button = this._makeButton(args, options, applicable);
			return button;
		},
		
		getCollapseAllButton: function(data, options, onClickArgs){
			var args = {
				imgClass: "icons-collapse-all",
				title: messages.collapseAllButtonTitle,
				onClick: dojo.hitch(this, function(){
					if(options && options.actionCallback){
						options.actionCallback();
					}
				})
			};
			var applicable = function(){
				return true && !options.disabled === true;
			};
			var button = this._makeButton(args, options, applicable);
			return button;
		},
		
		getShowArchivedButton: function(data, options, onClickArgs){
			var args = {
				imgClass: "icons-show-archived",
				title: messages.showArchivedButtonTitle,
				onClick: dojo.hitch(this, function(){
					button.set("title", button.pressed ? messages.hideArchivedButtonTitle : messages.showArchivedButtonTitle);
					if(options && options.actionCallback){
						options.actionCallback();
					}
				})
			};
			var applicable = function(){
				return true && !options.disabled === true;
			};
			var button = this._make2StateButton(args, options, applicable);
			return button;
		},
		
/* ----------------------------------------------------------------------------------------------------- */
// Button helpers
		_makeButton: function(args, options, applicable){
			// Force enabled to be false, wait for the isApplicable code to run
			args.enabled = false;
			var button = new jazz.ui.toolbar.Button(args);
			dojo.when(dojo.hitch(this, applicable), dojo.hitch(this, function(applicable){
				button.set("enabled", options && options.disabled != undefined ? options.disabled === false : true && applicable);
			}));
			return button;
		},
		
		_make2StateButton: function(args, options, applicable){
			// Force enabled to be false, wait for the isApplicable code to run
			args.enabled = false;
			var button = new jazz.ui.toolbar.TwoStateButton(args);
			dojo.when(dojo.hitch(this, applicable), dojo.hitch(this, function(applicable){
				button.set("enabled", options && options.disabled != undefined ? options.disabled === false : true && applicable);
			}));
			return button;
		},
		
/* ----------------------------------------------------------------------------------------------------- */
// onClick Helpers
		_launchMergeToolHelper: function(mergeArgs){
			var uriToFindFor, otherUri;
	    	if (mergeArgs.otherDescendantUri === "previous") {
	    		uriToFindFor = mergeArgs.descendantUri;
	    	} else if (mergeArgs.descendantUri === "previous") {
	    		uriToFindFor = mergeArgs.otherDescendantUri;
	    	} else {
	    		if (mergeArgs.operation === "compare") {
	    			// this is either a two-way or three-way compare.
	    			
	    			if (mergeArgs.ancestorUri) {
	    				// we have all the components for a three way merge.
	    				// so let's begin.
	    				
	    				// We don't want to show step one for a three-way compare.
	    				mergeArgs.hideConfigurationPicker = true;
	    				PageUtil.showAction(vvcConstants.EXPLORE_COMPARISON, dojo.objectToQuery(mergeArgs));
	    			} else {
	    				// we'll try to get a common snapshot if we don't have
		    			// an ancestor URI...
	    				uriToFindFor = mergeArgs.descendantUri;
	    				otherUri = mergeArgs.otherDescendantUri;
	    			}
	    		} else {
	    			// launch the compare viewer with step one enabled
	    			// so that users know what they are getting into...
		    		PageUtil.showAction(vvcConstants.EXPLORE_COMPARISON, dojo.objectToQuery(mergeArgs));
		    		return;
	    		}
	    	}
	    	
	    	if (uriToFindFor) {
	    		if (!otherUri) {
	    			var config = ContextUtils.getConfiguration(uriToFindFor);
			    	if (config && config.getParentUri()) {
			    		otherUri = config.getParentUri();
			    	} else if (mergeArgs.operation === "merge"){
			    		// Still OK to do arbitrary comparisons between thigns, so don't abort for compare operations.
			    		if (this.displayErrorMessage) {
			    			this.displayErrorMessage(messages.noParent);
			    		}
			    		
			    		return;
			    	}
	    		}
		    		
	    		// let's try to get the ancestor...
	    		var link = VvcUtils.getVvcUri()
				+ "/compare/getAncestorUri?descendantUri="
				+ encodeURIComponent(uriToFindFor)
				+ "&otherDescendantUri="
				+ encodeURIComponent(otherUri);
		
				var args = {
					url: link,
					handleAs: "json",
					headers: {
						Accept: "application/json;text/json"
					},
					load: function(response, ioArgs) {
						return response;
					},
					error: dojo.hitch(this, function(error, args) {				
						// Note: use this pattern to create notification which clears itself 
						ncFactory.createXhrError({xhrError: error, ioArgs: args, closeOnEvent: {context: this, event: "_launchMergeTool"}}).dispatch();
					})
				};
				
				var deferred = jazz.client.xhrGet(args);
				var self = this;
				deferred.addCallback(this, function (result) {
					if (!result.ancestorUri) {
						if (mergeArgs.operation === "compare") {
							// let's go ahead with a two-way compare...
							mergeArgs.descendantUri = otherUri;
							mergeArgs.otherDescendantUri = null;
							mergeArgs.ancestorUri = uriToFindFor;
							mergeArgs.hideConfigurationPicker = true;
							
							PageUtil.showAction(vvcConstants.EXPLORE_COMPARISON, dojo.objectToQuery(mergeArgs));
							
						} else if (this.displayErrorMessage) { // this was a merge
			    			this.displayErrorMessage(messages.noCommonSnapshot);
			    		}
					} else {
						if (mergeArgs.operation === "compare") {
							mergeArgs.descendantUri = otherUri;
							mergeArgs.otherDescendantUri = uriToFindFor;
						} else if (mergeArgs.operation === "deliver") {
							mergeArgs.otherDescendantUri = otherUri; // target
							mergeArgs.descendantUri = uriToFindFor; // source
							mergeArgs.mergeUri = mergeArgs.otherDescendantUri;
						} else {
							mergeArgs.otherDescendantUri = uriToFindFor;
							mergeArgs.descendantUri = otherUri;
							mergeArgs.mergeUri = mergeArgs.otherDescendantUri;
						}
						
						mergeArgs.hideConfigurationPicker = true;
						mergeArgs.ancestorUri = result.ancestorUri;
						
						PageUtil.showAction(vvcConstants.EXPLORE_COMPARISON, dojo.objectToQuery(mergeArgs));
					}
				});	
	    	}
	    }
}
})();
