/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.team.vvc.server.core.ui.internal.wizard.FinalizeStep");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("jazz.ui.Button");
dojo.require("jazz.ui.wizard.WizardStepHeader");
dojo.require("jazz.ui.wizard.WizardStep");
dojo.require("jazz.ui.StyledBox");
dojo.require("com.ibm.team.repository.web.client.internal.HttpHeaders");
dojo.require("com.ibm.team.repository.web.transport.TeamServerClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.client.internal.ServerStatusClient");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.util");
dojo.require("jazz.client.xhr");
dojo.require("jazz.app.auth");
dojo.require("net.jazz.ajax.contextRoot");

dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.team.vvc.server.core.ui.internal.wizard", "FinalizeStepMessages");

dojo.require("com.ibm.team.vvc.shared.web.ui.util.VvcUtils");
var VvcUtils = com.ibm.team.vvc.shared.web.ui.util.VvcUtils;

(function () {
	var messages;
	var WizardStepHeader = jazz.ui.wizard.WizardStepHeader;
	var WizardStep = jazz.ui.wizard.WizardStep;
	var TrimStyles = jazz.ui.StyledBox.TrimStyles;
	var TeamServerClient = com.ibm.team.repository.web.transport.TeamServerClient;
	var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
	var ServerStatusClient = com.ibm.team.repository.web.client.internal.ServerStatusClient;

	dojo.declare("com.ibm.team.vvc.server.core.ui.internal.wizard.FinalizeStep", [WizardStep, dijit._Widget, dijit._Templated], {

		templatePath: dojo.moduleUrl("com.ibm.team.vvc.server.core", "ui/internal/wizard/templates/FinalizeStep.html"),
		widgetsInTemplate: true,
		innerBoxTrim:  TrimStyles.EDITOR,
	
		constructor: function () {
			this.messages = messages = dojo.i18n.getLocalization("com.ibm.team.vvc.server.core.ui.internal.wizard", "FinalizeStepMessages");
			this.stepTitle = messages.stepTitle;
		},

		postCreate: function () {
			this.header = new WizardStepHeader({stepTitle: this.stepTitle});
			this.stepHeaderArea.appendChild(this.header.domNode);
		
			//Set StyledBox titles
			this.step1_header.innerHTML = "<span class=\"step\">" + this.messages.step1Label + "</span><span>" + this.messages.step1Title + "</span>";
			this.step2_header.innerHTML = "<span class=\"step\">" + this.messages.step2Label + "</span><span>" + this.messages.step2Title + "</span>";
		
			//Prepare for raw XHR requests
			this.headers = com.ibm.team.repository.web.client.internal.HttpHeaders._getHeaders();
		
			//Could improve this to be more robust...
			this.servicePrefix = VvcUtils.getVvcUri() + "/service/";
		
			//Controls whether or not the step can go next
			this._setCompletedState(false);
		
			this.hasProcessed = false;
			this.startButton.disabled = true;
		
			//Create DOM for the individual finalize operations
			this.domFinalize = this._addOperationDiv(this.messages.operation1title, this.messages.operation1desc);
			this.domUserSync = this._addOperationDiv(this.messages.operation2title, this.messages.operation2desc);
			this.domApplyLicense = this._addOperationDiv(this.messages.operation3title, this.messages.operation3desc);
			this.domImportTemplate = this._addOperationDiv("Import Process Template", "Loads the default process template for creating configuration space project areas");
			this.domConfigureStorage = this._addOperationDiv(this.messages.operation4title, this.messages.operation4desc);

			this.connect(this.startButton, "onclick", "start");
		},
	
		focusStep: function () {
			var self = this;
			var srh = new ServiceResponseHandler(self, "setJTSUrl", "handleAppNotRegistered");
			ServerStatusClient.getConfiguredJazzTeamServerURI(srh);
		},
	
		setJTSUrl: function (response) {
			this.configuredJTS = response;
			this.startButton.disabled = false;
			this.triggerButtonStateUpdate();
		},
	
		handleAppNotRegistered: function (error) {
			this.header.showMessage({
				errorObject: error, 
				summary: this.messages.appNotRegistered 
			});
		},
	
		//STEP 1: REGISTRATION HANDLER, FINALIZE
		start: function (e) {
			dojo.stopEvent(e);
			this.startButton.disabled = true;
			if (!this.hasProcessed) {	
				var self = this;
			
				//Show Pending Message
				this.header.showMessage({
					type: "INFO",
					summary: this.messages.finalizePending
				});	
				this.triggerButtonStateUpdate();
			
				//Create DOM to display the progress
				var domNode = this.domFinalize;
				dojo.addClass(domNode, "loading");
				var dFinalize = this.finalize();
				dFinalize.addCallback(function (response) {
					self.hasProcessed = true;
					dojo.removeClass(domNode, "loading");
					dojo.addClass(domNode, "ok");
					self.userSynchonization();
				});
				dFinalize.addErrback(function (errObject) {
					self._showInlineError(domNode, self.messages.finalizeError, errObject);
					self.startButton.disabled = false;
				});
			}
		},
	
		showCompletedMessage: function () {
			var buttonLabel = this.useFinishButton() === true ? this.messages.finish : this.messages.next;
			this.header.showMessage({
				type: "OK",
				summary: dojo.string.substitute(this.messages.allComplete, [ buttonLabel ])
			});	
			this._setCompletedState(true);
		},
	
		finalize: function () {
			var deferred = new dojo.Deferred();
			jazz.client.rawXhrPost({
				url: VvcUtils.getVvcUri() + '/setup-descriptor/finalize',
				headers: this.headers,
				load: function (response) {
					deferred.callback(response);
				},
				error: function (errObject) {
					deferred.errback(errObject);		
				}
			});	
			return deferred;
		},
	
		_showInlineError: function (node, message, errObject) {
			dojo.addClass(node, "error");
			var errorDiv = node.getElementsByTagName("div");
			dojo.removeClass(errorDiv[0], "noDisplay");
			var errorObj = this._getErrorObject(errObject);
			node._errorMsgBox.showMessage({
				errorObject: errorObj,
				summary: message
			});
			this._setCompletedState(false);
		},
	
		_getErrorObject: function (response) {
			var errorObj = {};
			if (response && response.responseText) {
				//Code from TeamServerClient.js......
				try {
					var responseObj = dojo.fromJson(response.responseText);
					dojo.mixin(errorObj, {
						errorClass: responseObj.errorClass,
						message: responseObj.errorMessage,
						exception: responseObj.errorTrace,
						errorData: responseObj.errorData,
						nestedError: responseObj.nestedError
					});
				} 
				catch (/* Error */ e) {
					// for non-JSON responses, include it as a message
					errorObj.message = response.responseText;
				}
			}
			return errorObj;
		},
	
		//Creates a new DIV element for the operation and appends it to the DOM
		//Returns: a handle to the DIV element
		_addOperationDiv: function (title, description, customClass) {
			var div = dojo.create("div", { className: "operation"}, this.operations);
			if (customClass) {
				dojo.addClass(div, customClass);
			}
			dojo.create("h3", { className: "operationTitle", innerHTML: title}, div);
			dojo.create("p", { className: "operationDesc", innerHTML: description}, div);
			var errDiv = dojo.create("div", { className: "operationError noDisplay"}, div);
			div._errorMsgBox = new jazz.ui.MessageBox({
				onToggle: dojo.hitch(this, this._handleMsgBoxToggle)
			});
			errDiv.appendChild(div._errorMsgBox.domNode);
			return div;
		},
	
		_handleMsgBoxToggle: function () {
			this.triggerButtonStateUpdate();
		},
	
		_setCompletedState: function (state) {
			this.isCompleted = state;
			this.triggerButtonStateUpdate();
		},
	
		customOnNext: function () {
			jazz.app.auth.logout();	
		},
	
		customOnFinish: function () {
			this.customOnNext();
		},

		canFlipToNextPage: function () {
			return this.isCompleted;
		},
	
		useFinishButton: function () {
			return !this.isEmbedded && this.finalStep();
		},
	
		canFinish: function () {
			return this.isCompleted && this.useFinishButton();
		},

		finalStep: function () {
			// This wizard step will never be the final step.
			return false;
		},
	
		nowSetCompletedState: function (state) {
			this.isCompleted = state;
			this.triggerButtonStateUpdate();
		},
	
		nowShowCompletedMessage: function () {
			var buttonLabel = this.useFinishButton() === true ? this.messages.finish : this.messages.next;
			this.header.showMessage({
				type: "OK",
				summary: dojo.string.substitute(this.messages.allComplete, [ buttonLabel ])
			});
			this.nowSetCompletedState(true);
		},
	
		//STEP 2: USER SYNCHRONIZATION
		userSynchonization: function () {
			var self = this;
			var domNode = this.domUserSync;
			var deferred = new dojo.Deferred();
			dojo.addClass(domNode, "loading");
			jazz.client.xhrPost({
				url: VvcUtils.getVvcUri() + '/setup-descriptor/usersync',
				headers: this.headers,
				load: function (response) {
					deferred.callback(response);
				},
				error: function (errObject) {
					deferred.errback(errObject);
				}
			});	
			deferred.addCallback(function (response) {
				dojo.removeClass(domNode, "loading");
				dojo.addClass(domNode, "ok");
				self.applyInternalLicense();
			});
			deferred.addErrback(function (errObject) {
				self._showInlineError(domNode, self.messages.userSyncError, errObject);
			});
		},
	
		//STEP 3: APPLY INTERNAL LICENSE
		applyInternalLicense: function () {
			var self = this;
			var domNode = this.domApplyLicense;
			var deferred = new dojo.Deferred();
			dojo.addClass(domNode, "loading");
			jazz.client.xhrPost({
				url: VvcUtils.getVvcUri() + '/setup-descriptor/applylicense',
				load: function (response) {
					deferred.callback(response);
				},
				error: function (errObject) {
					deferred.errback(errObject);
				}
			});	
			deferred.addCallback(function (response) {
				dojo.removeClass(domNode, "loading");
				dojo.addClass(domNode, "ok");
				self.importProcessTemplate();
			});
			deferred.addErrback(function (errObject) {
				self._showInlineError(domNode, self.messages.licenseError, errObject);
			});
		},
		
		//STEP 4: Import Process Template
		importProcessTemplate: function() {
			var self = this;
			var domNode = this.domImportTemplate;
			var deferred = new dojo.Deferred();
			dojo.addClass(domNode, "loading");
			jazz.client.xhrPost({
				url: VvcUtils.getVvcUri() + '/setup-descriptor/importprocess',
				load: function(response) {
					deferred.callback(response);
				},
				error: function(errObject) {
					deferred.errback(errObject);
				}
			});	
			deferred.addCallback(function(response){
				dojo.removeClass(domNode, "loading");
				dojo.addClass(domNode, "ok");
				self.createStorageAreas();
			});
			deferred.addErrback(function(errObject){
				//Allow the user to try to finalize since we don't know if they have already done so.
				self._showInlineError(domNode, "Unable to import the process template", errObject);
			});
		},
	
		//STEP 5: CREATE STORAGE AREAS
		createStorageAreas: function () {
			var self = this;
			var domNode = this.domConfigureStorage;
			var deferred = new dojo.Deferred();
			dojo.addClass(domNode, "loading");
			jazz.client.xhrPost({
				url: VvcUtils.getVvcUri() + '/setup-descriptor/createstorage',
				load: function (response) {
					deferred.callback(response);
				},
				error: function (errObject) {
					deferred.errback(errObject);
				}
			});	
			deferred.addCallback(function (response) {
				dojo.removeClass(domNode, "loading");
				dojo.addClass(domNode, "ok");
				self.nowShowCompletedMessage();
			});
			deferred.addErrback(function (errObject) {
				self._showInlineError(domNode, self.messages.createStorageError, errObject);
			});
		}
	});

})();
