/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011, 2013.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide('com.ibm.team.vvc.shared.web.ui.picker.WorkspacePicker');

dojo.require('jazz.app.auth');
dojo.require("jazz.ui.MessageBox");
dojo.require('jazz.ui.SearchBox');
dojo.require("jazz.ui.StyledTable");
dojo.require('jazz.util.html');
dojo.require("jazz.ui.Toolbar");
dojo.require("jazz.ui.toolbar.TwoStateButton");

dojo.require("dijit.layout.BorderContainer");
dojo.require("dijit.layout.ContentPane");

dojo.require('dojo.NodeList-traverse');
dojo.require('dojo.i18n');

dojo.require("com.ibm.team.vvc.shared.web.ui.util.VvcCache");
dojo.require("com.ibm.team.vvc.shared.web.ui.VvcConstants");
vvcConstants = com.ibm.team.vvc.shared.web.ui.VvcConstants;

dojo.require("com.ibm.team.vvc.shared.web.ui.util.VvcUtils");
var VvcUtils = com.ibm.team.vvc.shared.web.ui.util.VvcUtils;

dojo.requireLocalization("com.ibm.team.vvc.shared.web.ui.picker", "WorkspacePickerMessages");

(function ()
{
	
	function ignoreFieldKeyEvent(e)
	{
		if (e.ctrlKey || e.altKey) return true;

		switch(e.keyCode)
		{
		case dojo.keys.ENTER:
		case dojo.keys.ESCAPE:
		case dojo.keys.LEFT_ARROW:
		case dojo.keys.RIGHT_ARROW:
		case dojo.keys.TAB:
		case dojo.keys.SHIFT:
		case dojo.keys.ALT:
		case dojo.keys.CAPS_LOCK:
		case dojo.keys.PAGE_DOWN:
		case dojo.keys.PAGE_UP:
		case dojo.keys.LEFT_WINDOW:
		case dojo.keys.RIGHT_WINDOW:
		case dojo.keys.HOME:
		case dojo.keys.END:
		case dojo.keys.CTRL:
		case dojo.keys.F5:
			{
			return true;
			}
			
		default:
			return false;
		}
	}
	
	// TODO: This should use a utiliy method instead...
	function escapeHtml(html) {
		return html.replace(/&/g,'&amp;').replace(/</g,'&lt;').replace(/>/g,'&gt;');
	}
	
	/**
	 * Set "spaceUri" property to set the space uri you would like this dialog to constrain
	 * it's search results to, in the first argument of the constructor.
	 */
	dojo.declare(
			'com.ibm.team.vvc.shared.web.ui.picker.WorkspacePicker',
			[dijit._Widget, dijit._Templated], 
			{
				widgetsInTemplate: true,
				templatePath: dojo.moduleUrl('com.ibm.team.vvc.shared.web', 'ui/picker/templates/WorkspacePicker.html'),
				
			// Hooks
			onCancel: null,
			onOk: null,
			onSelect: null,
			onShowDimensions: null,
			onHideDimensions: null,
			closeable: null,
			defaultSearchTerm: null,
			singleClickMode: null,
			_cache: null,
			
			_validTypes: null,
			
			constructor: function (args)
			{
				if (args)
				{
					if (args.spaceUri){
						this._spaceUri = args.spaceUri;
					}
										
					// CSV of types,  can be "workspace", "snapshot", or combination thereof
					if(args.validTypes){
						this._validTypes = args.validTypes;
					}
					
					if(args.queryOnLoad){
						var handle = dojo.connect(this, "startup", this, function(){
							this._runSearch(null, args.queryOnLoad)
							dojo.disconnect(handle);
						});
					}
					
					if(args.defaultSearchTerm){
						this.defaultSearchTerm = args.defaultSearchterm;
					}
					
					if(args.singleClickMode){
						this.singleClickMode === args.singleClickMode;
					}
					
					if(args.closeable === false){
						this.closeable = false;
						var handle = dojo.connect(this, "postCreate", dojo.hitch(this, function(){
							this.hideOkCancelButtons();
							dojo.disconnect(handle);
						}));
					} else{
						this.closeable = true;
					}
				}
				this.messages = dojo.i18n.getLocalization("com.ibm.team.vvc.shared.web.ui.picker", "WorkspacePickerMessages");
				this.spinnerUrl = dojo.moduleUrl('com.ibm.team.vvc.shared.web', 'graphics/spinner.gif');
				this._iconSrc = dojo.moduleUrl('com.ibm.team.vvc.shared.web', 'graphics/VVC.gif');
				
				this._cache = new com.ibm.team.vvc.shared.web.ui.util.VvcCache({
					idProperty: "uri",
					storageKey: vvcConstants.CONFIGURATION_CACHE_KEY
				});
				
			},

			cancel: function() {
				this.onCancel && this.onCancel();
				this.destroyRecursive();
			},

			postCreate: function() {
				var self = this;
				
				this._detailsOn = false;
				this._createMenubar();
				this._createResultsTable();
				this._createDimensionsTable();
				
				// set initial focus
				if (this._searchBox) {
					var context = this;
					setTimeout(function() {
						if (context._searchBox) {
							context._searchBox.focus();
						}
					}, 100);
				}
			},
			
			_createMenubar: function(){
				this.toolbar = new jazz.ui.Toolbar();
				this._resultsTableToolbar.appendChild(this.toolbar.domNode);
				
				this.detailsToggleButton = new jazz.ui.toolbar.TwoStateButton({
					title: this.messages.toggleDetails,
					imgSrc: dojo.moduleUrl('com.ibm.team.vvc.shared.web', 'graphics/show_details.gif'),
					onClick: dojo.hitch(this, this._toggleDetails)
				});
				
				this.toolbar.addChild(this.detailsToggleButton);
			},

			_toggleDetails: function()
			{
				if (this._detailsOn)
				{
					this._detailsOn = false;
					this._dimensionsTableSection.style.display = "none";
					if(this.onHideDimensions){
						this.onHideDimensions();
					}
				}
				else
				{
					this._detailsOn = true;
					this._dimensionsTableSection.style.display = "";
					if(this.onShowDimensions){
						this.onShowDimensions();
					}
				}
			},
			_createResultsTable: function()
			{
				var table = new jazz.ui.StyledTable({
					   columns: [
					      {label: "", width: "16px", sortable: false},
					      {label: this.messages.name, width: "40%", sortable: false},
					      {label: this.messages.description, sortable: false}
					   ]
				});
				
				table.domNode.style.width = "100%";
				this._resultsTable = table;
				this._resultsTableDiv.appendChild(table.domNode);
			},
			
			_createDimensionsTable: function()
			{
				var table = new jazz.ui.StyledTable({
					   columns: [
					      {label: this.messages.dimensionName, sortable: false},
					      {label: this.messages.dimensionValue, sortable: false}
					   ],
				});
				
				table.domNode.style.width = "100%";
				this._dimensionsTable = table;
				this._dimensionsTableDiv.appendChild(table.domNode);
			},

			startup: function() {
				this.inherited(arguments);
				var self = this;
				window.setTimeout(function() {
					self._searchBox.focus();
				}, 1);
			},

			getSelected: function()
			{
				return this._selectedRow ? this._selectedRow.workspace : undefined;
			},
			
			_selectRow: function(row) {
				if (this._selectedRow) dojo.removeClass(this._selectedRow, 'selected');
				
				if (!row || !row.workspace)
					return;
				
				dojo.addClass(row, 'selected');
				this._selectedRow = row;

				this._submitButton.disabled = false;
				
				this._loadDimensionValuesIntoTable(row.workspace.dimensionValueSet);

			},
			
			_loadDimensionValuesIntoTable: function(dimensionValueSet)
			{
				var frag = document.createDocumentFragment();
				
				this._dimensionsTable.clearBody();

				var tr, td;
				for (var i = 0; i < dimensionValueSet.length; i++) {
					var dimensionValue = dimensionValueSet[i];
					
					tr = document.createElement("tr");
					
					td = document.createElement("td");
					td.innerHTML = jazz.util.html.escape(dimensionValue.dimensionName);
					tr.appendChild(td);
					
					td = document.createElement("td");
					td.innerHTML = jazz.util.html.escape(dimensionValue.dimensionValue);
					tr.appendChild(td);
					frag.appendChild(tr);
				}
				
				this._dimensionsTable.tbody.appendChild(frag);
			},

			_handleArrow: function(key) {
				if (!this._lastResult) return;

				var row;
				if (key == dojo.keys.DOWN_ARROW)
				{	
					if (!this._selectedRow) {
						// select first on arrow down if no selection
						row = this._getFirstRow(this._resultsTable.tbody);
					}
					else if (this._selectedRow.nextSibling && this._selectedRow.nextSibling.nodeName == 'TR') {
						row = this._selectedRow.nextSibling;
					}
				}
				else if (key == dojo.keys.UP_ARROW)
				{
					if (!this._selectedRow) {
						// select last on arrow up if no selection
						row = this._getFirstRow(this._resultsTable.tbody);
					}
					else if (this._selectedRow.previousSibling && this._selectedRow.previousSibling.nodeName == 'TR') {
						row = this._selectedRow.previousSibling;
					}
				}
				
				if(!row && this._selectedRow){
					row = this._selectedRow;
				}
				
				this._selectRow(row);
				if(this.onSelect && row){
					this.onSelect(row.workspace);
				}
			},

			_onFieldKeyDown: function(e) {
				if (ignoreFieldKeyEvent(e)) return;
				e.stopPropagation();
				var k = e.keyCode;
				
				if (k === dojo.keys.UP_ARROW
					|| k ===  dojo.keys.DOWN_ARROW) {
					dojo.stopEvent(e);
					this._handleArrow(k);
					return;
				}
				
				if(this.singleClickMode && k === dojo.keys.ENTER){
					dojo.stopEvent(e);
					this._onSubmit();
					return;
				}
				
				this._deselectAll();
				window.clearTimeout(this._pendingSearch);
				this._pendingSearch = window.setTimeout(dojo.hitch(this, this._runSearch, null), 400);
			},
			
			_onListKeyDown: function(e) {
				var k = e.keyCode;
				
				if (k == dojo.keys.UP_ARROW
					|| k ==  dojo.keys.DOWN_ARROW) {
					dojo.stopEvent(e);
					this._handleArrow(k);
					return;
				}
				
				if(this.singleClickMode && k === dojo.keys.ENTER){
					dojo.stopEvent(e);
					this._onSubmit();
					return;
				}
			}, 
			
			_onDblClickResource: function(e) {
				var row = e.target;

				if (e.target.nodeName != 'TR') {
					row = dojo.NodeList(e.target).closest('TR');
					if (!row) {
						console.warn('no list item found on click');
						return;
					}
					row = row[0];
				}

				dojo.stopEvent(e);
				this._selectRow(row);
				this._onSubmit();
			},

			_onClickResource: function(e) {
				var row = e.target;

				if (e.target.nodeName != 'TR') {
					row = dojo.NodeList(e.target).closest('TR');
					if (!row) {
						console.warn('no list item found on click');
						return;
					}
					row = row[0];
				}

				if (!dojo.hasClass(row, 'selected')) {
					this._deselectAll();
					this._selectRow(row);
				}
				
				if(this.singleClickMode){
					this._onSubmit(e);
				}
				
				if(this.onSelect && row){
					this.onSelect(row.workspace);
				}
			},
			
			_deselectAll: function()
			{
				if (this._selectedRow) {
					dojo.removeClass(this._selectedRow, 'selected');
					delete this._selectedRow;
				}

				this._submitButton.disabled = true;
			},
			
			_showLoading: function(show)
			{
				if (show) {
					this._loadingNode.style.display = '';
					this._noResultsNode.style.display = 'none';
				}
				else {
					this._loadingNode.style.display = 'none';
				}
			},

			_updateCount: function(count) {
				if (count == 0) {
					this._loadingNode.style.display = 'none';
					this._noResultsNode.style.display = '';
					this._noResultsNode.innerHTML = escapeHtml(this.messages.noResults);
				}
				else {
					this._noResultsNode.style.display = 'none';
				}
			},
			
			uninitialize: function()
			{
				this.inherited(arguments);
			},

			/*
			 * The provided searchTerm overrides the user's input seach term.  If the picker
			 * was created using a defaultSearcTerm, then this will be used in the provided
			 * searchTerm or the user's searchTerm are undefined.
			 */
			_runSearch: function(e, /*Optional*/searchTerm) {
				e && dojo.stopEvent(e);

				if (e && e.target == this._searchIcon) {
					this._searchBox.select();
				}

				window.clearTimeout(this._pendingSearch);
				
				var searchString;
				if(searchTerm){
					searchString = searchTerm;
				} else {
					searchString = dojo.trim(this._searchBox.value);
				}
				this.hideErrorMessage();
				
				delete this._lastResult;
				this._submitButton.disabled = true;
				this._showLoading(true);
				this._resultsTable.clearBody();
				this._dimensionsTable.clearBody();
				
				if (searchString.length == 0) {
					if(this.defaultSearchTerm){
						searchString = this.defaultSearchTerm;
					} else {
						this._updateCount(0, 0);
					}
				}

				var self = this;
				
				var content = {
					query: searchString
				};

				if (this.spaceUri){
					content.space = this._spaceUri;
				}
				
				if(this._validTypes){
					content.validTypes = this._validTypes ;
				}
				
				var params = dojo.objectToQuery(content);
				var args = {
						handleAs: 'json',
						url: VvcUtils.getVvcUri() + "/WorkspacesQuery/query?" + params, //$NON-NLS-1$
						"headers": { //$NON-NLS-1$
							"Content-Type" : "text/plain", //$NON-NLS-1$ //$NON-NLS-2$
						},
						handle: dojo.hitch(this, this._handleResponse)
				};
				
				var deferred = jazz.client.xhrGet(args);  
			},
			
			_handleResponse: function(response, ioargs) {
				this._showLoading(false);
				this._deselectAll();
				dojo.removeClass(this._searchIcon, 'loading');
				
				if (ioargs.xhr.status == 200)
				{
					if ("errorMessage" in response) //TODO: This is to be modified to whatever the error is supposed to be in the error json format
						this.displayErrorMessage(response.errorMessage);
					else  
						this._handleLoad(response);
				}
				else {
					console.log('error', response);
					this.displayErrorMessage("Invalid Response (Status:"+ioargs.xhr.status+"): "+ response.responseText);
				}
			},
			
			displayErrorMessage: function(errorMessage) 
			{
				var mb = new jazz.ui.MessageBox();
				mb.placeAt(dojo.byId("errorMessages"));
				mb.showMessage({summary: errorMessage, type: jazz.ui.MessageBox.STATUS_TYPES.ERROR});
				dojo.style("errorMessages", {display: ""});
			},
			
			hideErrorMessage: function() {
				dojo.empty("errorMessages");
				dojo.style("errorMessages", {display: "none"});
			},
			
			_handleLoad: function(results) {
				var uris = results.resultUris;
				var frag = document.createDocumentFragment();

				var missingIcon = dojo.moduleUrl('com.ibm.team.vvc.shared.web', 'graphics/missing_icon.gif');
								
				var tr, td;
				for (var i = 0; i < uris.length; i++) {
					dojo.when(this._cache.get(uris[i]), dojo.hitch(this, function(workspace){
						tr = document.createElement("tr");
						td = document.createElement("td");
						td.appendChild(dojo.create('img', {
							src: this._getImgURL(workspace),
							height: '16',
							width: '16'
							}));
						tr.appendChild(td);
						
						td = document.createElement("td");
						var titleNode = document.createTextNode(workspace.title);
						td.appendChild(titleNode);
						tr.appendChild(td);
						
						td = document.createElement("td");
						td.innerHTML = jazz.util.html.escape(workspace.description);
						tr.appendChild(td);
					
						tr.workspace = workspace;
						frag.appendChild(tr);
						
						this._lastResult = workspace;
						
						this._resultsTable.tbody.appendChild(frag);
					}));
				}
				this._updateCount(workspaces.length);
				this._resultsTable.clearBody();
			},
			
			_getImgURL: function(workspace) {
				if (workspace.type == 'snapshot')
					return dojo.moduleUrl('com.ibm.team.vvc.shared.web', 'graphics/snapshot.gif');
				else if (workspace.type == 'changeset')
				{	
					if (workspace.mutable)
						return dojo.moduleUrl('com.ibm.team.vvc.shared.web', 'graphics/changeset.gif');
					else
						return dojo.moduleUrl('com.ibm.team.vvc.shared.web', 'graphics/changeset_complete.gif');
				}
				return dojo.moduleUrl('com.ibm.team.vvc.shared.web', 'graphics/workspace.gif');
			},
			
			_onSubmit: function(e) {
				e && dojo.stopEvent(e);

				if (this._selectedRow) {
					// There are selected result, so OK the dialog.
					// Can get selected result still with getter until this is destroyed.

					if (this.onOk) {
						// get off the DOM before destroy
						var workspace = this._selectedRow.workspace;
						var self = this;

						// Delay the callback until after the 'destroy' so that the dialog goes away
						window.setTimeout(function() {
							self.onOk(workspace);
						}, 1);
						
					}
					
					if(this.closeable != false){
						this.destroyRecursive();
					}
				}
			},
			
			_getFirstRow: function(table) {
				if(table.domNode){
					table = table.domNode;
				}
				var children = table.childNodes;
	
				for (var i = 0; i < children.length; i++) {
					var child = children[i];
					if (child.nodeName == 'TR')
						return child;
				}
				return null;
			},
			
			_getLastRow: function(table) {
				if(table.domNode){
					table = table.domNode;
				}
				var children = table.childNodes;
				for (var i = children.length - 1; i > -1; i--) {
					var child = children[i];
					if (child.nodeName == 'TR')
						return child;
				}
				return null;
			},
			
			hideOkCancelButtons: function(){
				dojo.addClass(this.okCancelButtons, "hidden");
				dojo.removeClass(this.okCancelButtons, "visible-block");
			},
			
			showOkCancelButtons: function(){
				dojo.addClass(this.okCancelButtons, "visible-block");
				dojo.removeClass(this.okCancelButtons, "hidden");
			}
			
		}
	);
})();
