/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011, 2013.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function(){

	dojo.provide("com.ibm.team.vvc.server.core.ui.snapshots.SnapshotsViewer");

	dojo.require("dijit._Widget");
	dojo.require("dijit._Templated");
	dojo.require("net.jazz.ajax.ui.PlatformUI");

	dojo.require("dojox.grid.LazyTreeGrid");
	dojo.require("dijit.tree.ForestStoreModel");
	dojo.require("dojo.data.ItemFileWriteStore");

	dojo.require('dojo.i18n');
	dojo.require("dojo.date.stamp");
	dojo.require("dojo.date.locale");
	dojo.require("jazz.ui.Breadcrumbs");
	dojo.require("jazz.ui.Toolbar");
	dojo.require("jazz.ui.toolbar.Button");
	dojo.require("jazz.ui.StyledTable");
	dojo.require("jazz.ui.MessageBox");
	
	dojo.require('jazz.util.html');
	
	dojo.require("com.ibm.team.vvc.server.core.ui.internal.util.PageUtil");
	var PageUtil = com.ibm.team.vvc.server.core.ui.internal.util.PageUtil;
	
	dojo.require("com.ibm.team.vvc.shared.web.ui.context.ContextUtils");
	var ContextUtils = com.ibm.team.vvc.shared.web.ui.context.ContextUtils;

	dojo.require("com.ibm.team.vvc.shared.web.ui.util.SecurityServiceUtils");
	var SecurityServiceUtils = com.ibm.team.vvc.shared.web.ui.util.SecurityServiceUtils;
	
	dojo.requireLocalization("com.ibm.team.vvc.server.core.ui.snapshots", "SnapshotsViewerMessages");
	var messages = dojo.i18n.getLocalization("com.ibm.team.vvc.server.core.ui.snapshots", "SnapshotsViewerMessages");

	dojo.require("com.ibm.team.vvc.shared.web.ui.VvcConstants");
	var vvcConstants = com.ibm.team.vvc.shared.web.ui.VvcConstants;
	
	dojo.require("com.ibm.team.vvc.shared.web.ui.util.VvcUtils");
	var VvcUtils = com.ibm.team.vvc.shared.web.ui.util.VvcUtils;
	
	dojo.require("com.ibm.team.vvc.shared.core.ui.objects.ConfigurationResource");
	dojo.require("com.ibm.team.vvc.shared.web.ui.changesets.ChangesetsWidgit");
	
	var VVC_APP = "com.ibm.team.vvc.server.core";
	var EDITABLE_STYLE = 0; // snapshots viewer in the workspace editor
	var PICKER_STYLE = 1; // picker for rebase operation
	var WORKSPACE_STYLE = 2;
	var CHANGESET_STYLE = 3;
	
	var NEW_WORKSPACE_ICON = dojo.moduleUrl(VVC_APP, "graphics/create_workspace-enabled.gif");
	var COMPARE_ICON = dojo.moduleUrl(VVC_APP, "graphics/compare-enabled.gif");
	var DELETE_ICON = dojo.moduleUrl(VVC_APP, "graphics/delete.gif");
	
	dojo.declare("com.ibm.team.vvc.server.core.ui.snapshots.SnapshotsViewer", [dijit._Widget, dijit._Templated], {
		widgetsInTemplate: true,
		templatePath: dojo.moduleUrl(VVC_APP, 'ui/snapshots/templates/SnapshotsViewer.html'),

		_actionRegistry: [ 
		{
			label: function(_workspace) {
				return (_workspace.permissions && _workspace.permissions.createOrModifyWorkspace) ? messages.newWorkspace : SecurityServiceUtils.getNoPermissionMessage(_workspace.permissions.controllingAreaName);
			},
			imgSrc: NEW_WORKSPACE_ICON,
			isApplicable: function(_workspace) {
				return _workspace.permissions && _workspace.permissions.createOrModifyWorkspace; 
			},
			handler: function (snapshot) {
				var query = "contextUri=" + escape(snapshot.uri);
				PageUtil.showPage(vvcConstants.CREATE_WORKSPACE, query);
			}
		},
		{
			label: function(_workspace) {
				return messages.compare;
			},
			imgSrc: COMPARE_ICON,
			handler: function (snapshot) {
				var query = {};
				query.ancestorUri = "previous"; // keyword for compare service to figure out the actual uri
				query.descendantUri = snapshot.uri;
				query.descendantTitle = snapshot.getTitle();
				query.redirectUri = encodeURIComponent(window.location.href);
				query.hideConfigurationPicker = true;
				PageUtil.showPage(vvcConstants.EXPLORE_COMPARISON, dojo.objectToQuery(query));
			}
		},
		/*{
			label:  messages.remove,
			imgSrc: DELETE_ICON,
			handler: function (id) {
				alert(messages.notSupportedYet);
			}
		}*/
		],
		
		_workspace: null,
		_style: null,
		_lazyStart: false,
		
		constructor: function(args) {
			if (args) {
				this._workspace = args.workspace;
				this._style = args.style;
				this._lazyStart = args.lazyStart;
			}
			
			// set up reference to messages in the html template
			this.messages = messages;
		},
		
		postCreate: function() {
			if (!this._lazyStart) {
				this.load();
			}
		},

		load: function() {
			// show loading message
			this.showMessage({
				type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
				summary: this.messages.loading
			});
			
			var query = {};
			if (this._workspace) {
				query.parentWorkspace = this._workspace.getUri();
			} else {
				var uri = ContextUtils.getContextUri();
				if (uri) {
					query.parentWorkspace = uri;
				}
			}

			var objectType = "snapshots";
			if (this._style == WORKSPACE_STYLE) {
				objectType = "workspaces";
			} else if (this._style == CHANGESET_STYLE) {
				objectType = "changesets";
				
				dojo.removeClass(this.tableContainerNode, "vvc-snapshots-table");
				if (this._changesetsWidgit) {
					this.tableContainerNode.removeChild(this._changesetsWidgit);
				}
				this._changesetsWidgit = new com.ibm.team.vvc.shared.web.ui.changesets.ChangesetsWidgit({hideNotificationCenter: true, standalone: false});
				this.tableContainerNode.appendChild(this._changesetsWidgit.domNode);
			    this._changesetsWidgit.resizeExplorer();
			    
			    // remove the loading message
			    this.hideMessage();
				
			    return;
			}
			var uri = VvcUtils.getVvcUri() + "/repository/" + objectType + "?" + dojo.objectToQuery(query) + "&includePermissions=true";
			var args = {
					url: uri,
					headers: {
						"Accept" : "application/rdf+json"
					},
					sync: true,
					handleAs: "json",
					load: function(response, ioArgs) {
						var configResources = [];
						for (var uri in response) {
							var r = response[uri];
							var config = new com.ibm.team.vvc.shared.core.ui.objects.ConfigurationResource();
							config.onLoad(r);
							configResources.push(config);
						}
						return configResources;
					}
			};
			var deferred = jazz.client.xhrGet(args);
			deferred.addCallback(this, function (snapshots) {
				var table = this.rebuildTable(snapshots);
				
			    // remove the loading message
				this.hideMessage();
			});
			deferred.addErrback(this, function(error) {
				this.showMessage(PageUtil.createXhrErrorMessage(deferred.ioArgs));
			});
		},
		
		rebuildTable: function(snapshots) {
			// create table
			var columnData = [];
			if (this._style === PICKER_STYLE) {
				columnData.push({ label: "", width: "16px" }); // check box or radio button column
			}
			columnData.push({ label: messages.name });
			columnData.push({ label: messages.description });
			columnData.push({ label: messages.creator });
			columnData.push({ label: messages.created, width: "150px"});

			if (this._style === EDITABLE_STYLE) {
				columnData.push({ label: messages.actions, width: "45px" });
			}
			
			if (this._table) {
				this.tableContainerNode.removeChild(this._table);
			}
			var table = new jazz.ui.StyledTable({
				trim: jazz.ui.StyledTable.TrimStyles.PRIMARY_HEADLESS,
				columns: columnData
			});
			this._table = table;
			this.tableContainerNode.appendChild(table.domNode);
			table.domNode.style.width = "100%";
			
			// Now add the rows
			var userUri, userName;
			var tr, td;
			for (var i = 0; i < snapshots.length; i++){
				var configResource = snapshots[i];
				tr = document.createElement("tr");
				
				// check box or radio button column
				if (this._style === PICKER_STYLE) {
					td = document.createElement("td");
					var buttonId = PageUtil.htmlEscape(configResource.uri);
					td.innerHTML = "<input type='radio' name='snapshotGroup' id='" + buttonId + "'/>";
					tr.appendChild(td);
				}
				
				// add name column
				td = document.createElement("td");
				var uri = configResource.uri;
				if (this._style == CHANGESET_STYLE) {
					uri += "&style=changesetview";
				}
				var pageId = vvcConstants.SNAPSHOT_PAGE;
				if (this._style == WORKSPACE_STYLE) {
					pageId = vvcConstants.EDIT_WORKSPACE;
				}
				PageUtil.setLinkField(td, configResource.getTitle(), uri, pageId);
				tr.appendChild(td);

				// add description column
				td = document.createElement("td");
				td.innerHTML = jazz.util.html.escape(configResource.getDescription());
				tr.appendChild(td);
				
				// set creator field
				var creator = configResource.getCreator();
				var created = configResource.getCreated();

				var name;
				if (creator === userUri) {
					name = userName;
				} else if (creator) {
					var user = PageUtil.getUser(creator);
					if (user) {
						userUri = creator;
						userName = user.name;
						name = userName;
					}
				}
				if (name) {
					// add creator name column
					td = document.createElement("td");
					td.innerHTML = name;
					tr.appendChild(td);
				}

				// set the created field
				if (created) {
					var createdDate = dojo.date.stamp.fromISOString(created);
					var date = dojo.date.locale.format(createdDate, {formatLength: "medium"});
					
					// add created date column
					td = document.createElement("td");
					td.innerHTML = date;
					tr.appendChild(td);
				}
				
				// add actions column
				if (this._style == EDITABLE_STYLE) {
					td = document.createElement("td");
					var tb = this.createActions(configResource, td);
					td.appendChild(tb.domNode);
					tr.appendChild(td);
				}
				
				table.tbody.appendChild(tr);
			}
			return table;
		},
		
		getSelection: function() {
			var selection = [];
			var arr = document.getElementsByTagName("input");
			for (i = 0; i < arr.length; i++) {
			   if (arr[i].name == "snapshotGroup" && arr[i].checked) {
				   var uri = arr[i].id;
				   selection.push({uri: uri});
			   }
			}
			return selection;
		},
		
		showMessage: function(args) {
			if (args && this._messageBoxContainer) {
				args["closable"] = true;
				if (!this.messageBox) {
					this.messageBox = new jazz.ui.MessageBox();
					this._messageBoxContainer.appendChild(this.messageBox.domNode);
				}
				this.messageBox.showMessage(args);
				this._messageBoxContainer.style.display = "";
			}
		},
		
		hideMessage: function() {
			if (this._messageBoxContainer) {
				this._messageBoxContainer.style.display = "none";
			}
		},
		
		createActions: function(snapshot, td) {
			var toolbar = new jazz.ui.Toolbar();
			var actions = this._actionRegistry;
			dojo.forEach(actions, dojo.hitch(this, function (action)
				{
				var button = jazz.ui.toolbar.Button({
					title: action.label(this._workspace),
					imgSrc: action.imgSrc,
					onClick: dojo.hitch(this, function(){
						action.handler.call(this, snapshot, td);
					})
				});
				button.setAttribute("enabled", action.isApplicable ? action.isApplicable(this._workspace) : true);
				toolbar.addChild(button);
					}));
			dojo.addClass(toolbar.domNode, "vvc-snapshots-toolbar");		
			return toolbar;
		}

	});
})();