/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011, 2012.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function() {

	dojo.provide("com.ibm.team.vvc.server.core.ui.snapshots.WorkspacesAndSnapshotsViewer");

	dojo.require("dijit._Widget");
	dojo.require("dijit._Templated")
	dojo.require("dojo.date.stamp");
	dojo.require("dojo.date.locale");
	dojo.require("jazz.ui.MessageBox");
	dojo.require('jazz.util.html');

	dojo.require("com.ibm.team.vvc.shared.web.ui.VvcConstants");
	
	dojo.require("net.jazz.ajax.ui.PlatformUI");
	var PlatformUI = net.jazz.ajax.ui.PlatformUI;

	dojo.require("com.ibm.team.vvc.server.core.ui.internal.util.PageUtil");
	var PageUtil = com.ibm.team.vvc.server.core.ui.internal.util.PageUtil;
	
	dojo.require("com.ibm.team.vvc.shared.web.ui.context.ContextUtils");
	var ContextUtils = com.ibm.team.vvc.shared.web.ui.context.ContextUtils;

	dojo.require("com.ibm.team.vvc.shared.web.ui.util.SecurityServiceUtils");
	var SecurityServiceUtils = com.ibm.team.vvc.shared.web.ui.util.SecurityServiceUtils;
	
	dojo.require("com.ibm.team.vvc.shared.web.ui.util.VvcUtils");
	var VvcUtils = com.ibm.team.vvc.shared.web.ui.util.VvcUtils;
	
	dojo.require("com.ibm.team.vvc.shared.core.ui.objects.ConfigurationResource");

	dojo.requireLocalization("com.ibm.team.vvc.server.core.ui.snapshots", "SnapshotsViewerMessages");
	var messages = dojo.i18n.getLocalization("com.ibm.team.vvc.server.core.ui.snapshots", "SnapshotsViewerMessages");

	var vvcConstants = com.ibm.team.vvc.shared.web.ui.VvcConstants;
	
	var VVC_APP = "com.ibm.team.vvc.server.core";
	var EDITABLE_LIST_STYLE = 0;
	var PICKER_STYLE = 1;
	var DEPENDENCIES_LIST_STYLE = 2;
	var DEPENDENCIES_LIST_WITHOUT_ACTIONS_STYLE = 3;
	var DEPENDENCY_SERVICE = "dependencies";

	var NEW_WORKSPACE_ICON = dojo.moduleUrl(VVC_APP, "graphics/create_workspace-enabled.gif");
	var NEW_SNAPSHOT_ICON = dojo.moduleUrl(VVC_APP, "graphics/create_snapshot-enabled.gif");
	var COMPARE_ICON = dojo.moduleUrl(VVC_APP, "graphics/compare-enabled.gif");
	var DELETE_ICON = dojo.moduleUrl(VVC_APP, "graphics/delete.gif");

	dojo.declare("com.ibm.team.vvc.server.core.ui.snapshots.WorkspacesAndSnapshotsViewer", [dijit._Widget, dijit._Templated], {
		messages: messages, // needed for the html template
		widgetsInTemplate: true,
		templatePath: dojo.moduleUrl(VVC_APP, "ui/snapshots/templates/WorkspacesAndSnapshotsViewer.html"),
		
		_configurationResources: null,
		_configResourcesInfo: null,
		_table: null,
		_style: null,
		_workspace: null,
		_workspaceUri: null,
		_configUris: null,
		_mutableIndex: 0,
		_lazyStart: false,

		//consider deleting -- appears to be unused
		_workspaceActions: [
		{
			label: messages.createSnapshot,
			imgSrc: NEW_SNAPSHOT_ICON,
			isApplicable: function(configResource) {
				return configResource.permissions.createOrModifySnapshot;
			},
			handler: function (workspace) {
				var query = "contextUri=" + escape(workspace.uri);
				PageUtil.showPage(vvcConstants.SNAPSHOT_PAGE, query);
			}
		},
		{
			label: messages.compare,
			imgSrc: COMPARE_ICON,
			handler: function (id) {
				window.location.href = "#action=" + vvcConstants.COMPARE_CONFIGURATIONS;
			}
		},
		{
			label: messages.remove,
			imgSrc: DELETE_ICON,
			handler: function (id) {
				alert(messages.notSupportedYet);
			}
		}              
		],

		//consider removing -- appears to be unused
		_snapshotActions: [ 
		{
			label: messages.newWorkspace,
			imgSrc:NEW_WORKSPACE_ICON,
			isApplicable: function(configResource) {
				return configResource.permissions.createOrModifyWorkspace;
			},
			handler: function (snapshot) {
				var query = "contextUri=" + escape(snapshot.uri);
				PageUtil.showPage(vvcConstants.CREATE_WORKSPACE, query);
			}
		},
		{
			label: messages.compare,
			imgSrc: COMPARE_ICON,
			handler: function (id) {
				window.location.href= "#action=" + vvcConstants.COMPARE_CONFIGURATIONS;
			}
		},
		{
			label: messages.remove,
			imgSrc: DELETE_ICON,
			handler: function (id) {
				alert(messages.notSupportedYet);
			}
		}              
		],

		constructor: function(args) {
			if (args) {
				this._style = args.style;
				this._workspaceUri = args.workspaceUri; //consider keeping workspace parameter only
				this._workspace = args.workspace;
				this._lazyStart = args.lazyStart;
			}
		},
		
		postCreate: function() {
			if (!this._lazyStart) {
				this.load();
			}
		},

		load: function() {
			// show loading message
			this.showMessage({
				type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
				summary: this.messages.loading
			});
			
			var deferred;
			deferred = this.getConfigurationResources();
			deferred.addCallback(this, function (confResources) {
				this._configurationResources = confResources;
				this.rebuildTable(confResources);
			});
			deferred.addErrback(this, function(error) {
				this.showMessage(PageUtil.createXhrErrorMessage(deferred.ioArgs));
			});
		},

		rebuildTable: function(configResources) {
			// remove loading message
			this.hideMessage();
			
			// create table
			var columnData = [];
			if (this._style == PICKER_STYLE) {
				columnData.push({ label: "", width: "16px" });  // check box
				this._mutableIndex = 1;
			}
			columnData.push({ label: "",  width: "16px" }); //icon
			columnData.push({ label: messages.name, sortable: true });
			columnData.push({ label: messages.description, sortable: true });
			columnData.push({ label: messages.parent, sortable: true });
			columnData.push({ label: messages.space, sortable: true });
			columnData.push({ label: messages.creator, width: "150px", sortable: true  });
			columnData.push({ label: messages.created, width: "150px", sortable: true });
			if (this.isShowActionBar()) {
				columnData.push({ label: messages.actions, width: "75px" });
			}
			
			if (this._table) {
				this.tableContainerNode.removeChild(this._table);
			}
			var self = this;
			var table = new jazz.ui.StyledTable({
				trim: jazz.ui.StyledTable.TrimStyles.PRIMARY_HEADLESS,
				columns: columnData,
				sortBy: this._mutableIndex + 1, // name field
				sortByAscending: false,
				handleSort: function() {
					this.clearBody();
					self._sort(configResources, table);
					self._fillTable(configResources, table);
				}
			});
			this._table = table;
			this.tableContainerNode.appendChild(table.domNode);
			table.domNode.style.width = "100%";

			// mixed in the creator and created fields
			var userUri, userName;
			for (var i = 0; i < configResources.length; i++) {
				var configResource = configResources[i];

				configResource.mutable = configResource.getMutable();
				configResource.title = configResource.getTitle();
				configResource.description = configResource.getDescription();
				if (!configResource.description) {
					configResource.description = "";
				}
				configResource.parentName = configResource.getParentWorkspaceName();
				configResource.spaceName = configResource.getSpaceName();

				// set creator field
				var creator = configResource.getCreator();
				var created = configResource.getCreated();
				if (!creator || !created) {
					var info = this.getResourceInfo(configResource.uri);
					creator = info.creator;
					created = info.created;
				}
				var name;
				if (creator === userUri) {
					name = userName;
				} else if (creator) {
					var user = PageUtil.getUser(creator);
					if (user) {
						userUri = creator;
						userName = user.name;
						name = userName;
					}
				}
				if (!name) {
					name = "";
				}
				configResource.creator = name;

				// set the created field
				if (created) {
					configResource.created = created;
				} else {
					configResource.created = "";
				}
			}

			this._sort(configResources, table);
			this._fillTable(configResources, table);
		},
		
		isShowActionBar: function() {
			if (this._style == EDITABLE_LIST_STYLE || this._style == DEPENDENCIES_LIST_STYLE) {
				return true;
			}
			return false;
		},

		_sort: function(configResources, table) {
			var tableColumn = this._getTableColumn;
			var mutableIndex = this._mutableIndex;
			var f = function(a, b) {
				var c1 = tableColumn(a, table.sortBy, mutableIndex);
				var c2 = tableColumn(b, table.sortBy, mutableIndex);
				var r = 0;
				if (c1 && c2) {
					if (c1 < c2) {
						r = -1; 
					} else if (c1 > c2) {
						r = 1;
					}
					if (table.sortByAscending) {
						r = r * -1;
					}
				} else if (c1) {
					return -1;
				} else {
					return 1;
				}
				return r;
			};
			configResources.sort(f);
		},

		_getTableColumn: function(configResource, columnIndex, index) {
			if (columnIndex === index++) {
				return configResource.mutable;
			}
			if (columnIndex === index++) {
				return configResource.title;
			}
			if (columnIndex === index++) {
				return configResource.description;
			}
			if (columnIndex === index++) {
				return configResource.parentName;
			}
			if (columnIndex === index++) {
				return configResource.spaceName;
			}
			if (columnIndex === index++) {
				return configResource.creator;
			}
			if (columnIndex === index++) {
				return configResource.created;
			}
		},

		_fillTable: function(configResources, table) {
			// Now add the rows
			var userUri, userName;
			var workspaceIcon = dojo.moduleUrl(VVC_APP, "graphics/workspace.gif");
			var snapshotIcon = dojo.moduleUrl(VVC_APP, "graphics/snapshot.gif");
			var td, tr;
			for (var i = 0; i < configResources.length; i++) {
				var configResource = configResources[i];
				tr = document.createElement("tr");

				// check box or radio button column
				if (this._style == PICKER_STYLE) {
					td = document.createElement("td");
					var buttonId = this.getCheckBoxId(i);
					td.innerHTML = "<input type='checkbox' id='" + buttonId + "'/>";
					tr.appendChild(td);
				}

				// add icon column
				td = document.createElement("td");
				var icon;
				if (this._isMutable(configResource)) {
					icon = "<img src='" + workspaceIcon + "'>";
				} else {
					icon = "<img src='" + snapshotIcon + "'>";
				}
				td.innerHTML = icon;
				tr.appendChild(td);

				// add name column
				td = document.createElement("td");
				var uri = configResource.uri;
				if (uri) {
					if (configResource.isMutable()) {
						PageUtil.setLinkField(td, configResource.title, uri, vvcConstants.EDIT_WORKSPACE);
					} else {
						PageUtil.setLinkField(td, configResource.title, uri, vvcConstants.SNAPSHOT_PAGE);
					}					
				} else {
					td.innerHTML = configResource.title;
				}
				tr.appendChild(td);

				// add description column
				td = document.createElement("td");
				td.innerHTML = jazz.util.html.escape(configResource.description);
				tr.appendChild(td);

				// add parent column
				td = document.createElement("td");
				var uri = configResource.getParentWorkspaceUri();
				if (uri) {
					PageUtil.setLinkField(td, configResource.parentName, uri, vvcConstants.EDIT_WORKSPACE);
				} else {
					td.innerHTML = configResource.parentName;
				}
				tr.appendChild(td);
				
				// add space column
				td = document.createElement("td");
				td.innerHTML = configResource.spaceName;
				tr.appendChild(td);
				
				// add creator column
				td = document.createElement("td");
				td.innerHTML = configResource.creator;
				tr.appendChild(td);

				// add created date column
				td = document.createElement("td");
				var created = configResource.created;
				if (created) {
					var createdDate = dojo.date.stamp.fromISOString(created);
					created = dojo.date.locale.format(createdDate, {formatLength: "medium"});
				}
				td.innerHTML = created;
				tr.appendChild(td);

				// add actions column
				if (this.isShowActionBar()) {
					td = document.createElement("td");
					var tb = this.createActions(configResource, td);
					td.appendChild(tb.domNode);
					tr.appendChild(td);
				}

				table.tbody.appendChild(tr);
			}
			return table;
		},
		
		getCheckBoxId: function(index) {
			return "checkbox_" + index;
		},
		
		getSelected: function() {
			var selected = [];
			var configArray = this._configurationResources;
			if (configArray) {
				for (var i = 0; i < configArray.length; i++) {
					var buttonId = this.getCheckBoxId(i);
					var button = dojo.byId(buttonId);
					if (button && button.checked) {
						var uri = configArray[i].uri;
						selected.push(uri);
					}
				}
			}
			return selected;
		},

		_isMutable: function(configResource) {
			var mutable = configResource.getMutable();
			if (mutable === "true") {
				return true;
			}
			return false;
		},

		createActions: function(configResource, td) {
			var toolbar = new jazz.ui.Toolbar();
			var actions;
			if (this._style == DEPENDENCIES_LIST_STYLE) {
				var self = this;
				actions = [{label: this._workspace.permissions.updateDependencies ? messages.remove : SecurityServiceUtils.getNoPermissionMessage(this._workspace.permissions.controllingAreaName),
							imgSrc: DELETE_ICON,
							isApplicable: dojo.hitch(this, function() {
								return this._workspace.permissions.updateDependencies;
							}),
							handler: function (config, tableData) {
									var args = {};
									args.configuration = ContextUtils.getContextUri();
									args.operation = "REMOVE";
									uri = VvcUtils.getVvcUri() + "/" + DEPENDENCY_SERVICE;
									var selected = [];
									selected.push(config.uri);
									args.dependencies = selected;
									var args = {
											url: uri,
											headers: {
												"Accept" : "application/json",
												"Content-Type" : "application/json"
											},
											sync: true,
											handleAs: "json",
											postData: dojo.toJson(args),
											load: function(response, ioArgs) {
												// remove table row
												var tr = tableData.parentNode;
												var table = tr.parentNode;
												if (table) {
													table.removeChild(tr);
												}
												
												// remove data entry in configuration resource array
												var configArray = self._configurationResources;
												for (var i = 0; i < configArray.length; i++) {
													var configResource = configArray[i];
													if (configResource === config) {
														self._configurationResources.splice(i, 1);
														break;
													}
												}
											},
											error: dojo.hitch(this, function(e, ioArgs) {
												this.showMessage(PageUtil.createXhrErrorMessage(ioArgs));
											})
									};
									var deferred = jazz.client.xhrPost(args);
								}
							}              
						];
			} else if (this._isMutable(configResource)) {
				actions = this._workspaceActions;
			} else {
				actions = this._snapshotActions;
			}
			dojo.forEach(actions, dojo.hitch(this, function (action)
			{
				var button = jazz.ui.toolbar.Button({
					title: action.label,
					imgSrc: action.imgSrc,
					onClick: dojo.hitch(this, function() {
						action.handler.call(this, configResource, td);
					})
				});
				button.setAttribute("enabled", action.isApplicable ? action.isApplicable(configResource) : true);
				toolbar.addChild(button);
			}));
			dojo.addClass(toolbar.domNode, "vvc-workspace-snapshot-toolbar");
			return toolbar;
		},
		
		showMessage: function(args) {
			if (args && this._messageBoxContainer) {
				args["closable"] = true;
				if (!this.messageBox) {
					this.messageBox = new jazz.ui.MessageBox();
					this._messageBoxContainer.appendChild(this.messageBox.domNode);
				}
				this.messageBox.showMessage(args);
				this._messageBoxContainer.style.display = "";
			}
		},
		
		hideMessage: function() {
			if (this._messageBoxContainer) {
				this._messageBoxContainer.style.display = "none";
			}
		},

		getResourceInfo: function(uri) {
			if (!this._configResourcesInfo) {
				var deferred = this.getConfigurationResourceInfo();
				deferred.addCallback(this, function (configResourcesInfo) {
					this._configResourcesInfo = configResourcesInfo;
				});
				deferred.addErrback(this, function(error) {
					this.showMessage(PageUtil.createXhrErrorMessage(deferred.ioArgs));
				});
			}
			
			if (this._configResourcesInfo) {
				for (var r in this._configResourcesInfo) {
					var item = this._configResourcesInfo[r];
					if (uri === item.uri) {
						return item;
					}
				}
			}
		},

		getConfigurationResourceInfo: function() {
			var args = {
					url: net.jazz.ajax.contextRoot() + "/repository/info/configurations",
					headers: {
						"Accept" : "application/json"
					},
					handleAs: "json",
					sync: true
			};
			return jazz.client.xhrGet(args);
		},

		getConfigurationResources: function() {
			var uri = VvcUtils.getVvcUri();
			var args = {};
			args.configurationUri = this._workspaceUri;
			if ((this._style == DEPENDENCIES_LIST_STYLE
				|| this._style == DEPENDENCIES_LIST_WITHOUT_ACTIONS_STYLE)
				&& this.workspaceUri != null) {
				// get list of existing dependencies
				// actions in the column have permissions restricted based on
				// editor's configuration, not each dependency's configuration
				uri += "/" + DEPENDENCY_SERVICE + "?" + dojo.objectToQuery(args);
			} else if (this._style == PICKER_STYLE) {
				// get list of snapshot can be picked as dependencies
				// there are no actions displayed
				uri +=  "/" + DEPENDENCY_SERVICE + "/getSelectableDependencies?" + dojo.objectToQuery(args);
			} else {
				// consider removing -- front service does not apepar to exist anymore
				uri +=  "/repository/configurations";
			}
			var args = {
					url: uri,
					headers: {
						"Accept" : "application/json"
					},
					handleAs: "json",
					sync: true,
					load: function(response, ioArgs) {
						var configResources = [];
						for (var uri in response) {
							var r = response[uri];
							var config = new com.ibm.team.vvc.shared.core.ui.objects.ConfigurationResource();
							config.onLoad(r);
							configResources.push(config);
						}
						return configResources;
					}
			};
			return jazz.client.xhrGet(args);
		}

	})   
})();
