/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corporation 2012, 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide('com.ibm.team.vvc.shared.web.ui.util.LinkToWorkItemUtil');

dojo.require("com.ibm.team.vvc.shared.web.ui.context.ContextUtils");
dojo.require("com.ibm.team.vvc.shared.oslc.web.ui.dialogs.AddCALMLinksDialog");
dojo.require("com.ibm.team.vvc.shared.oslc.web.ui.actions.ShowLinkedElementsDialogAction");
dojo.require("com.ibm.team.vvc.shared.web.ui.util.InfoContentUtils");

dojo.requireLocalization("com.ibm.team.vvc.shared.web.ui.util", "LinkToWorkItemUtilMessages");

var ContextUtils = com.ibm.team.vvc.shared.web.ui.context.ContextUtils;
var Messages = dojo.i18n.getLocalization("com.ibm.team.vvc.shared.web.ui.util", "LinkToWorkItemUtilMessages");
var InfoContentUtils = com.ibm.team.vvc.shared.web.ui.util.InfoContentUtils;

dojo.require("com.ibm.team.vvc.shared.web.ui.util.VvcUtils");
var VvcUtils = com.ibm.team.vvc.shared.web.ui.util.VvcUtils;

/**
 * This utility class is used to support links to work items.
 */
com.ibm.team.vvc.shared.web.ui.util.LinkToWorkItemUtil = {
		
	showLinkedWorkItemsAction: function() {
		com.ibm.team.vvc.shared.oslc.web.ui.actions.ShowLinkedElementsDialogAction.showDialog(ContextUtils.getCurrentContext().uri,
			function() {
				var urlArgs = {
					changesetConfiguration: ContextUtils.getCurrentContext().uri,
					consumerUrl: VvcUtils.getVvcUri()
				};
				return {
					url : VvcUtils.getVvcUri() + "/changesetLinks?" + dojo.objectToQuery(urlArgs),
				}
			},
			
			function(targetUrl) {
				var urlArgs = {
						changesetConfiguration: ContextUtils.getCurrentContext().uri,
						predicate: "https://jazz.net/ns/vvc/link/tracksChanges/rtc/",
						backlinkUrl: "http://open-services.net/ns/cm#tracksChangeSet",
						target: targetUrl,
						consumerUrl: VvcUtils.getVvcUri()
				}
				
				return { 
					url : VvcUtils.getVvcUri() + '/changesetLinks?' + dojo.objectToQuery(urlArgs)
				};
			}
		);
	},
	
	//this method is intended for consumers, which typically are in a changeset configuration
	linkToWorkItemAction: function() {
		
		var context = ContextUtils.getCurrentContext();
		//otherwise, we could throw an error (although this should never really happen because the caller checks in isApplicable())
		if (context.isChangeset()) {
			if (!context.changesetResource) {
				context.changesetResource = new com.ibm.team.vvc.shared.core.ui.objects.ChangesetResource();
				context.changesetResource.loadByConfiguration(context);
			}
			
			var subjectUri = context.changesetResource.uri;
			var title = context.changesetResource.title;
			
			var dialog = com.ibm.team.vvc.shared.oslc.web.ui.dialogs.AddCALMLinksDialog;
			var params = {
				consumerMode: true,
	            dialogTitle: Messages.LinkToWorkItemDialogTitle,
	            //no need to add dialogSecondaryTitle or item
				onOk: function onOk(item){
					var targetUrl = item.url;
					
					var args = {
						url: targetUrl + "?oslc.properties=oslc_cm:tracksChangeSet",
						headers: {
							"OSLC-Core-Version": "2.0",
							"Accept": "application/json",
							"Content-Type" : "application/json"
						}
					};
					
					var dfd = jazz.client.xhrGet(args);
					dfd.then(dojo.hitch(this, function(result) {
						var jsonObject = dojo.fromJson(result);
						var changeSets = jsonObject["oslc_cm:tracksChangeSet"];
						
						for (var i = 0; i < changeSets.length; i++) {
							if (changeSets[i]["rdf:resource"] === subjectUri) {
								//don't do anything since the work item is already linked to the change set
								return;
							}
						}
						
						var changeSetsToPut = [];
						changeSetsToPut.push({"dcterms:title" : title, "rdf:resource" : subjectUri});

						var putObject = { "oslc_cm:tracksChangeSet" : changeSetsToPut};
						args["putData"] = dojo.toJson(putObject);
						args["headers"]["Content-Type"] = "application/json";
						var dfd2 = jazz.client.xhrPut(args);

						dfd2.then(dojo.hitch(this, function(result) {
							//do nothing on success
						}), dojo.hitch(this, function(result) {
							InfoContentUtils.showSimpleContentDialog(Messages.errorAddingLinkTitle, Messages.errorAddingLink);
						}));
						
					}), dojo.hitch(this, function(result) {
						InfoContentUtils.showSimpleContentDialog(Messages.errorAddingLinkTitle, Messages.errorAddingLink);
					}));
				},
	            resourceUri: subjectUri,
	            acceptableLinkTypes: ["http://open-services.net/ns/cm#"]
			};
			dialog = new dialog(params);
			
		}
	},

	//this method is intended for vvc directly, so there is direct access to the changeset
	vvcLinkToWorkItemAction: function(subjectUri, deferred) {
		
		var context = {
			deferred: deferred
		};
		
		var dialog = com.ibm.team.vvc.shared.oslc.web.ui.dialogs.AddCALMLinksDialog;
		var params = {
            dialogTitle: Messages.LinkToWorkItemDialogTitle,
            //no need to add dialogSecondaryTitle or item
			onOk: dojo.hitch(context, function onOk(item){
				var targetUrl = item.url;
				if (targetUrl) {
					
					var urlArgs = {
							changeset: subjectUri,
							predicate: "http://jazz.net/ns/vvc/link/tracksChanges/rtc",
							backlinkPredicate: "http://open-services.net/ns/cm#tracksChangeSet",
							target: targetUrl,
							consumerUrl: VvcUtils.getVvcUri(),
							backlinkUrl: "http://open-services.net/ns/cm#tracksChangeSet"
					}
										
					var args = {
						headers: {
							"Content-Type": "application/x-oslc-common-link+xml",
							"Accept": "application/atom+json"
						},						
						url: VvcUtils.getVvcUri() + "/changesetLinks?" + dojo.objectToQuery(urlArgs),
						handleAs: "json",
						hasBody: false
					};
					var xhrPostDeferred = jazz.client.xhrPost(args);
					var context = {
						deferred: this.deferred
					}
					xhrPostDeferred.addCallback(context, dojo.hitch(this, function(result) {
						if (this.deferred) {
							this.deferred.resolve(result);
						}
					}));
					xhrPostDeferred.addErrback(context, dojo.hitch(this, function(error) {
						alert(Messages.errorAddingLink);
						if (this.deferred) {
							this.deferred.reject(error);
						}
					}));
					 
				}
			}),
            resourceUri: subjectUri,
            acceptableLinkTypes: ["http://open-services.net/ns/cm#"]
		};
		dialog = new dialog(params);
	}	
};

})();			