/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012, 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.vvc.shared.web.ui.widgets.tab.TabManager");
	
(function() {
	
	dojo.require("com.ibm.team.vvc.shared.web.ui.context.ContextUtils");
	var ContextUtils = com.ibm.team.vvc.shared.web.ui.context.ContextUtils;

	dojo.declare(
			"com.ibm.team.vvc.shared.web.ui.widgets.tab.TabManager",
			null, 
		{
			constructor: function() {
				this._tabDescriptors = [];
			},
			
			destroy: function() {
				dojo.forEach(this._tabDescriptors, function(tab) {
					if(tab.connectHandle) {
						dojo.disconnect(tab.connectHandle);
					}
				});
			},
			
			registerTab: function(tab, content, style, load) {
				var descriptor= { tab: tab, content: content, styleName: style, load: load};
				descriptor.id = tab.getAttribute('dojoAttachPoint');
				descriptor.oldDisplay= content.style.display;
				descriptor.loaded = false;
				descriptor.connectHandle = dojo.connect(tab, "onclick", this, "_handleClick"); // nonwidget- must be dojo connect
				this._tabDescriptors.push(descriptor);
			},
			
			_getTabDescriptor: function(id) {
				var tabDescriptors= this._tabDescriptors;
				var td = undefined;
				for (var i= 0; i < tabDescriptors.length; i++) {
					td = tabDescriptors[i];
					if (td.id === id) {
						return td;
					}
				}
				return null;
			},
			
			showTabById: function(id) {
				var td = this._getTabDescriptor(id);
				if (td == null) {
					return null;
				}
				return this.showTab(td.tab);
			},
			
			showTab: function(tabNode) {
				var tabDescriptors= this._tabDescriptors;
				var td = undefined;
				var tabToActivate = undefined;
				
				for (var i= 0; i < tabDescriptors.length; i++) {
					td = tabDescriptors[i];
					if (td.tab === tabNode) {
						tabToActivate = td;
					} else {
						this._deactivate(td);
					}
				}
				
				if (tabToActivate) {
					this._activate(tabToActivate);
					return tabToActivate;
				}
				return null;
			},
			
			_handleClick: function(event) {
				dojo.stopEvent(event);
				this.showTab(event.currentTarget);
			},
			
			_activate: function(tabDescriptor) {
				dojo.addClass(tabDescriptor.tab, tabDescriptor.styleName);
//				tabDescriptor.content.style.display= tabDescriptor.oldDisplay;
				tabDescriptor.content.style.display= 'block';
				if (!tabDescriptor.loaded && tabDescriptor.load != null)
				{
					tabDescriptor.load();
					tabDescriptor.loaded = true;
				}
				
				//TODO: Delete this line once defect 31896 is fixed!
				this._setLastTabViewed(tabDescriptor.tab);
			},
			
			_deactivate: function(tabDescriptor) {
				dojo.removeClass(tabDescriptor.tab, tabDescriptor.styleName);
				tabDescriptor.content.style.display= "none";
			},
			
			//TODO: Delete these methods once defect 31896 is fixed!
			showActiveTab: function(defaultTab)
			{
				var lastConfigurationViewed = this._getLastConfigurationViewed();
				if (lastConfigurationViewed == ContextUtils.getContextUri())
				{
					var lastViewedTabId = this._getLastTabViewed();
					if (this.showTabById(lastViewedTabId)) {
						return;
					}
				}
				
				this._setLastTabViewed(defaultTab);
				this.showTab(defaultTab);
			},
			
			COOKIE_KEY: "VVC_LAST_VIEWED_TAB",
			
			COOKIE_DELIMITER: "==", 
			
			_setLastTabViewed: function(tab)
			{
				var cookieValue = ContextUtils.getContextUri()+this.COOKIE_DELIMITER+ tab.getAttribute('dojoAttachPoint');	
				
				dojo.cookie(this.COOKIE_KEY, cookieValue, {
					"path": window.location.pathname,
					"domain" : window.location.hostname,
					"expires" : 90 // Expires in 90 days
				});
			},
			
			_getLastConfigurationViewed: function()
			{
				var cookie = dojo.cookie(this.COOKIE_KEY);
				if(cookie) {
					var cookieParts = cookie.split(this.COOKIE_DELIMITER);
					return cookieParts[0];
				}
				return null;
			},
			
			_getLastTabViewed: function()
			{
				var cookie = dojo.cookie(this.COOKIE_KEY);
				if(cookie) {
					var cookieParts = cookie.split(this.COOKIE_DELIMITER);
					return cookieParts[1];
				}
				return null;
			},
		});

})();